@@ -365,6 +365,7 @@
 ("end" "einde")
 ("english" "engels")
 ("enter a title" "geef een titel op")
+("enter data" "gegevens invoeren")
 ("entire table" "hele tabel")
 ("enumerate" "nummeren")
 ("enumerate|Alpha" "Letters")
@@ -1097,6 +1098,7 @@
 ("proportional typewriter" "proportionele schrijfmachine")
 ("proposition" "propositie")
 ("query replace" "tekst op gecontroleerde wijze vervangen")
+("question" "vraag")
 ("quit" "verlaat")
 ("quotation" "citaat")
 ("quote" "citeren")
@@ -1106,6 +1108,7 @@
 ("really huge" "echt enorm")
 ("really quit" "wilt u echt ophouden")
 ("really tiny" "werkelijk miniem")
+("recent" "recent")
 ("recover autosave file" "herstel veiligheidsbestand")
 ("red" "rood")
 ("redo" "opnieuw")
@@ -1149,7 +1152,7 @@
 ("reset" "herzet")
 ("resize object" "verander grootte object")
 ("resized" "uitgerekt")
-("results of the search for" "resultaten voor het zoeken naar")
+("search results for" "resultaten voor het zoeken naar")
 ("return" "return")
 ("revert" "herlaad")
 ("revert buffer" "herlaad")
