@@ -1216,7 +1216,7 @@ veg msolsa")
 ("reset" "")
 ("resize object" "objektum tmretezse")
 ("resized" "tmretezve")
-("results of the search for" "keressi eredmnyek:")
+("search results for" "keressi eredmnyek:")
 ("return" "")
 ("revert" "visszallts")
 ("revert buffer" "puffer visszalltsa")
