@@ -1,4 +1,4 @@
-<TeXmacs|1.0.6.6>
+<TeXmacs|1.0.7.13>
 
 <style|<tuple|source|std-pattern>>
 
@@ -13,11 +13,10 @@
     <src-copyright|1998--2004|Joris van der Hoeven>
 
     <\src-license>
-      This software falls under the <hlink|GNU general public
-      license, version 3 or later|$TEXMACS_PATH/LICENSE>.
-      It comes WITHOUT ANY WARRANTY WHATSOEVER.
-      You should have received a copy of the license which the software.
-      If not, see <hlink|http://www.gnu.org/licenses/gpl-3.0.html|http://www.gnu.org/licenses/gpl-3.0.html>.
+      This software falls under the <hlink|GNU general public license,
+      version 3 or later|$TEXMACS_PATH/LICENSE>. It comes WITHOUT ANY
+      WARRANTY WHATSOEVER. You should have received a copy of the license
+      which the software. If not, see <hlink|http://www.gnu.org/licenses/gpl-3.0.html|http://www.gnu.org/licenses/gpl-3.0.html>.
     </src-license>
   </src-title>>
 
@@ -60,7 +59,7 @@
 
   <drd-props|doc-date|border|no>
 
-  <drd-props|doc-note|border|no>
+  <drd-props|doc-note|arity|1|border|no|accessible|all>
 
   <drd-props|doc-inactive|arity|1|border|no|accessible|all>
 
@@ -80,7 +79,9 @@
 
   <drd-props|author-homepage|border|no>
 
-  <drd-props|author-note|border|no>
+  <drd-props|author-note|arity|1|border|no|accessible|all>
+
+  <drd-props|abstract|arity|1|accessible|all>
 
   <\active*>
     <\src-comment>
@@ -169,8 +170,10 @@
   <assign|doc-keywords|<xmacro|args|<style-with|src-compact|none|<no-indent><theorem-name|<keywords-text><localize|:>
   ><concat-tuple|<copy|<quote-arg|args>>|, >>>>
 
+  <assign|msc-ref|<macro|msc-id|<style-with|src-compact|none|<hlink|<arg|msc-id>|<style-with|src-compact|none|<merge|http://www.ams.org/mathscinet/search/mscbrowse.html?sk=default&sk=|<arg|msc-id>|&submit=Search>>>>>>
+
   <assign|doc-AMS-class|<xmacro|args|<style-with|src-compact|none|<no-indent><theorem-name|<AMS-class-text><localize|:>
-  ><concat-tuple|<copy|<quote-arg|args>>|, >>>>
+  ><concat-tuple|<copy|<map|msc-ref|<quote-arg|args>>>|, >>>>
 
   \;
 
@@ -287,22 +290,22 @@
 
   <assign|author-from-authors|<xmacro|x|<style-with|src-compact|none|<if|<unequal|<get-arity|<quote-arg|x>>|0>|<arg|x|0><map-args|author-from-authors*|concat|x|1>>>>>
 
-  <assign|author-render-name|<macro|x|<surround|<vspace*|0.5fn>|<vspace|0.5fn>|<doc-author-block|<name|<arg|x>>>>>>
+  <assign|author-render-name|<macro|author|<surround|<vspace*|0.5fn>|<vspace|0.5fn>|<doc-author-block|<name|<arg|author>>>>>>
 
-  <assign|author-name|<macro|x|<author-render-name|<style-with|src-compact|none|<author-by|<arg|x>><style-with|src-compact|none|<doc-footnote-ref|<quasi|<doc-author-data-note|<unquote*|<quote-value|the-author-data>>>>>>>>>>
+  <assign|author-name|<macro|author|<author-render-name|<style-with|src-compact|none|<author-by|<arg|author>><style-with|src-compact|none|<doc-footnote-ref|<quasi|<doc-author-data-note|<unquote*|<quote-value|the-author-data>>>>>>>>>>
 
-  <assign|author-address|<\macro|x>
-    <surround|<vspace*|0.5fn>|<vspace|0.5fn>|<doc-author-block|<arg|x>>>
+  <assign|author-address|<\macro|address>
+    <surround|<vspace*|0.5fn>|<vspace|0.5fn>|<doc-author-block|<arg|address>>>
   </macro>>
 
-  <assign|author-email|<macro|x|<doc-author-block|<style-with|src-compact|none|<with|font-shape|italic|<email-text><localize|:>
-  ><with|font-family|tt|<arg|x>>>>>>
+  <assign|author-email|<macro|email|<doc-author-block|<style-with|src-compact|none|<with|font-shape|italic|<email-text><localize|:>
+  ><with|font-family|tt|<arg|email>>>>>>
 
-  <assign|author-homepage|<macro|x|<doc-author-block|<style-with|src-compact|none|<with|font-shape|italic|<homepage-text><localize|:>
-  ><with|font-family|tt|<arg|x>>>>>>
+  <assign|author-homepage|<macro|homepage|<doc-author-block|<style-with|src-compact|none|<with|font-shape|italic|<homepage-text><localize|:>
+  ><with|font-family|tt|<arg|homepage>>>>>>
 
-  <assign|author-note|<\macro|x>
-    <arg|x>
+  <assign|author-note|<\macro|note>
+    <arg|note>
   </macro>>
 
   <\active*>
