@@ -70,6 +70,7 @@
   (mathd "\\mathrm{d}")
   (mathe "\\mathrm{e}")
   (matheuler "\\gamma")
+  (mathlambda "\\lambda")
   (mathi "\\mathrm{i}")
   (mathpi "\\pi")
   (Alpha "\\mathrm{A}")
@@ -87,16 +88,25 @@
   (Zeta "\\mathrm{Z}")
 
   ;; other extra symbols
+  (exterior "\\wedge")
   (Exists "\\exists")
   (bigintwl "\\int")
   (bigointwl "\\oint")
-  (bignone "")
   (asterisk "*")
-  (nonesep "")
+  (point ".")
+  (cdummy "\\cdot")
+  (comma "{,}")
+  (bignone "")
+  (nobracket "")
+  (nospace "")
+  (nocomma "")
+  (noplus "")
+  (nosymbol "")
   (nin "\\not\\in")
   (nni "\\not\\ni")
   (notni "\\not\\ni")
   (nequiv "\\not\\equiv")
+  (nleadsto "\\not\\leadsto")
   (dotamalg "\\mathaccent95{\\amalg}")
   (dottimes "\\mathaccent95{\\times}")
   (dotoplus "\\mathaccent95{\\oplus}")
@@ -157,6 +167,8 @@
   (pari "{\\sc Pari}"))
 
 (drd-table latex-texmacs-1%
+  (tmrsub (ensuremath (!append "_{" (textrm 1) "}")))
+  (tmrsup (ensuremath (!append "^{" (textrm 1) "}")))
   (tmtextrm (!group (rmfamily) (!group 1)))
   (tmtextsf (!group (sffamily) (!group 1)))
   (tmtexttt (!group (ttfamily) (!group 1)))
@@ -233,6 +245,14 @@
    (!append (noindent) (textbf (!append 1 "\\ "))
 	    ---
 	    (hspace* (fill)) (!math (Box)) (medskip)))
+  ("left-aligned"
+   (!append (!begin "flushleft")
+	    ---
+            (!end "flushleft")))
+  ("right-aligned"
+   (!append (!begin "flushright")
+	    ---
+            (!end "flushright")))
   ("tmparmod"
    ((!begin "list" "" (!append "\\setlength{\\topsep}{0pt}"
 			       "\\setlength{\\leftmargin}{" 1 "}"
@@ -361,12 +381,49 @@
     "    \\end{center}\n"
     "  \\end{minipage}}\n")))
 
+;;(define-macro (latex-texmacs-long prim x l m r)
+;;  `(drd-table latex-texmacs-preamble%
+;;     (,(string->symbol (substring prim 1 (string-length prim)))
+;;      (!append
+;;       "\\def" ,prim "fill@{\\arrowfill@" ,l ,m ,r "}\n"
+;;       "\\providecommand{" ,prim "}[2][]{"
+;;       "\\ext@arrow 0099" ,prim "fill@{#1}{#2}}\n"))))
+
+(define-macro (latex-texmacs-long prim x l m r)
+  `(drd-table latex-texmacs-preamble%
+     (,(string->symbol (substring prim 1 (string-length prim)))
+      (!append
+       "\\providecommand{" ,prim "}[2][]{"
+       "\\mathop{" ,x "}\\limits_{#1}^{#2}}\n"))))
+
+(latex-texmacs-long "\\xminus" "-"
+                    "\\DOTSB\\relbar" "\\relbar" "\\DOTSB\\relbar")
+(latex-texmacs-long "\\xleftrightarrow" "\\longleftrightarrow"
+                    "\\leftarrow" "\\relbar" "\\rightarrow")
+(latex-texmacs-long "\\xmapsto" "\\longmapsto"
+                    "\\vdash" "\\relbar" "\\rightarrow")
+(latex-texmacs-long "\\xmapsfrom" "\\leftarrow\\!\\!\\dashv"
+                    "\\leftarrow" "\\relbar" "\\dashv")
+(latex-texmacs-long "\\xequal" "="
+                    "\\DOTSB\\Relbar" "\\Relbar" "\\DOTSB\\Relbar")
+(latex-texmacs-long "\\xLeftarrow" "\\Longleftarrow"
+                    "\\Leftarrow" "\\Relbar" "\\Relbar")
+(latex-texmacs-long "\\xRightarrow" "\\Longrightarrow"
+                    "\\Relbar" "\\Relbar" "\\Rightarrow")
+(latex-texmacs-long "\\xLeftrightarrow" "\\Longleftrightarrow"
+                    "\\Leftarrow" "\\Relbar" "\\Rightarrow")
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;; Plain style theorems
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
 (define-macro (latex-texmacs-thmenv prim name before after)
   (with env-sym (string->symbol (string-append "begin-" prim))
     `(begin
        (drd-table latex-texmacs-env-preamble%
 	 (,prim (!append ,@before (newtheorem ,prim (!translate ,name))
-			 ,@after "\n")))
+			 ,@after "\n")
+		no-amsthm-package%))
        ;;;
        (drd-group latex-texmacs-tag% ,env-sym)
        (drd-group latex-environment-0% ,env-sym))))
@@ -395,9 +452,47 @@
 (latex-texmacs-remark "example" "Example")
 (latex-texmacs-remark "convention" "Convention")
 (latex-texmacs-remark "warning" "Warning")
+(latex-texmacs-remark "acknowledgments" "Acknowledgments")
 (latex-texmacs-exercise "exercise" "Exercise")
 (latex-texmacs-exercise "problem" "Problem")
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;; AMS style theorems
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+(define-macro (ams-texmacs-theorem abbr full)
+  `(begin
+     (drd-table latex-texmacs-env-preamble%
+       (,abbr (!append "\\theoremstyle{plain}\n"
+		       (newtheorem ,abbr (!translate ,full))
+		       "\n")
+	      amsthm-package%))))
+
+(define-macro (ams-texmacs-remark abbr full)
+  `(begin
+     (drd-table latex-texmacs-env-preamble%
+       (,abbr (!append "\\theoremstyle{remark}\n"
+		       (newtheorem ,abbr (!translate ,full))
+		       "\n")
+	      amsthm-package%))))
+
+(ams-texmacs-theorem "theorem" "Theorem")
+(ams-texmacs-theorem "proposition" "Proposition")
+(ams-texmacs-theorem "lemma" "Lemma")
+(ams-texmacs-theorem "corollary" "Corollary")
+(ams-texmacs-theorem "axiom" "Axiom")
+(ams-texmacs-theorem "definition" "Definition")
+(ams-texmacs-theorem "notation" "Notation")
+(ams-texmacs-theorem "conjecture" "Conjecture")
+(ams-texmacs-remark "remark" "Remark")
+(ams-texmacs-remark "note" "Note")
+(ams-texmacs-remark "example" "Example")
+(ams-texmacs-remark "convention" "Convention")
+(ams-texmacs-remark "acknowledgments" "Acknowledgments")
+(ams-texmacs-remark "warning" "Warning")
+(ams-texmacs-remark "exercise" "Exercise")
+(ams-texmacs-remark "problem" "Problem")
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Style-dependent extra macros
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -470,73 +565,167 @@
 ;; Catcode tables
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-(drd-table iso-latin-catcodes%
-  ("" "\\`a")
-  ("" "\\`A")
-  ("" "\\'a")
-  ("" "\\'A")
-  ("" "\\\"a")
-  ("" "\\\"A")
-  ("" "\\^a")
-  ("" "\\^A")
-  ("" "{\\aa}")
-  ("" "{\\AA}")
-  ("" "\\~A")
-  ("" "\\~a")
-  ("" "{\\AE}")
-  ("" "{\\ae}")
-  ("" "\\c{c}")
-  ("" "\\c{C}")
-  ("" "{\\dh}")
-  ("" "{\\DH}")
-  ("" "\\`e")
-  ("" "\\`E")
-  ("" "\\'e")
-  ("" "\\'E")
-  ("" "\\\"e")
-  ("" "\\\"E")
-  ("" "\\^e")
-  ("" "\\^E")
-  ("" "\\`{\\i}")
-  ("" "\\`I")
-  ("" "\\'{\\i}")
-  ("" "\\'I")
-  ("" "\\\"{\\i}")
-  ("" "\\\"I")
-  ("" "\\^{\\i}")
-  ("" "\\^I")
-  ("" "\\~N")
-  ("" "\\~n")
-  ("" "\\`o")
-  ("" "\\`O")
-  ("" "\\'o")
-  ("" "\\'O")
-  ("" "\\\"o")
-  ("" "\\\"O")
-  ("" "\\^o")
-  ("" "\\^O")
-  ("" "\\~O")
-  ("" "\\~o")
-  ("" "{\\O}")
-  ("" "{\\o}")
-  ("" "{\\ss}")
-  ("" "{\\th}")
-  ("" "{\\TH}")
-  ("" "\\`u")
-  ("" "\\`U")
-  ("" "\\'u")
-  ("" "\\'U")
-  ("" "\\\"u")
-  ("" "\\\"U")
-  ("" "\\^u")
-  ("" "\\^U")
-  ("" "\\'y")
-  ("" "\\'Y")
-  ("" "\\\"y")
-  ("" "\\\"Y")
-  ("" "!`")
-  ("" "?`")
-  ("" "?`"))
+(drd-table corkT1-to-latex-catcodes%
+  ("\x00" "\\`{ }")
+  ("\x01" "\\'{ }")
+  ("\x02" "\\^{ }")
+  ("\x03" "\\~{ }")
+  ("\x04" "\\\"{ }")
+  ("\x05" "\\H{ }")
+  ("\x06" "\\r{ }")
+  ("\x07" "\\v{ }")
+  ("\x08" "\\u{ }")
+  ("\x09" "\\b{ }")
+;  ("\x0A" "\\.{ }") ; newline may still be present in strings!!!
+  ("\x0B" "\\c{ }")
+  ("\x0C" "\\k{ }")
+  ("\x0D" ",")
+  ("\x0E" "{\\guilsinglleft}")
+  ("\x0F" "{\\guilsinglright}")
+  ("\x10" "``")
+  ("\x11" "''")
+  ("\x12" ",,")
+  ("\x13" "<<")
+  ("\x14" ">>")
+  ("\x15" "--")
+  ("\x16" "---")
+  ("\x17" "{}")
+  ("\x18" "{}")
+  ("\x19" "\\i")
+  ("\x1A" "\\j")
+  ("\x1B" "ff")
+  ("\x1C" "fi")
+  ("\x1D" "fl")
+  ("\x1E" "ffi")
+  ("\x1F" "ffl")
+  ("\x80" "\\u{A}")
+  ("\x81" "\\k{A}")
+  ("\x82" "\\'C")
+  ("\x83" "\\v{C}")
+  ("\x84" "\\v{D}")
+  ("\x85" "\\v{E}")
+  ("\x86" "\\k{E}")
+  ("\x87" "\\u{G}")
+  ("\x88" "\\'L")
+  ("\x89" "\\v{L}")
+  ("\x8A" "\\L")
+  ("\x8B" "\\'N")
+  ("\x8C" "\\v{N}")
+  ("\x8D" "{\\NG}")
+  ("\x8E" "\\H{O}")
+  ("\x8F" "\\'R")
+  ("\x90" "\\v{R}")
+  ("\x91" "\\'S")
+  ("\x92" "\\v{S}")
+  ("\x93" "\\c{S}")
+  ("\x94" "\\v{T}")
+  ("\x95" "\\c{T}")
+  ("\x96" "\\H{U}")
+  ("\x97" "\\r{U}")
+  ("\x98" "\\\"Y")
+  ("\x99" "\\'Z")
+  ("\x9A" "\\v{Z}")
+  ("\x9B" "\\.Z")
+  ("\x9C" "IJ")
+  ("\x9D" "\\.I")
+  ("\x9E" "{\\dj}")
+  ("\x9F" "{\\S}")
+  ("\xA0" "\\u{a}")
+  ("\xA1" "\\k{a}")
+  ("\xA2" "\\'c")
+  ("\xA3" "\\v{c}")
+  ("\xA4" "\\v{d}")
+  ("\xA5" "\\v{e}")
+  ("\xA6" "\\k{e}")
+  ("\xA7" "\\u{g}")
+  ("\xA8" "\\'l")
+  ("\xA9" "\\v{l}")
+  ("\xAA" "{\\l}")
+  ("\xAB" "\\'n")
+  ("\xAC" "\\v{n}")
+  ("\xAD" "{\\ng}")
+  ("\xAE" "\\H{o}")
+  ("\xAF" "\\'r")
+  ("\xB0" "\\v{r}")
+  ("\xB1" "\\'s")
+  ("\xB2" "\\v{s}")
+  ("\xB3" "\\c{s}")
+  ("\xB4" "\\v{t}")
+  ("\xB5" "\\c{t}")
+  ("\xB6" "\\H{u}")
+  ("\xB7" "\\r{u}")
+  ("\xB8" "\\\"y")
+  ("\xB9" "\\'z")
+  ("\xBA" "\\v{z}")
+  ("\xBB" "\\.z")
+  ("\xBC" "ij")
+  ("\xBD" "!`")
+  ("\xBE" "?`")
+  ("\xBF" "{\\pounds}")
+  ("\xC0" "\\`A")
+  ("\xC1" "\\'A")
+  ("\xC2" "\\^A")
+  ("\xC3" "\\~A")
+  ("\xC4" "\\\"A")
+  ("\xC5" "{\\AA}")
+  ("\xC6" "{\\AE}")
+  ("\xC7" "\\c{C}")
+  ("\xC8" "\\`E")
+  ("\xC9" "\\'E")
+  ("\xCA" "\\^E")
+  ("\xCB" "\\\"E")
+  ("\xCC" "\\`I")
+  ("\xCD" "\\'I")
+  ("\xCE" "\\^I")
+  ("\xCF" "\\\"I")
+  ("\xD0" "{\\DH}")
+  ("\xD1" "\\~N")
+  ("\xD2" "\\`O")
+  ("\xD3" "\\'O")
+  ("\xD4" "\\^O")
+  ("\xD5" "\\~O")
+  ("\xD6" "\\\"O")
+  ("\xD7" "{\\OE}")
+  ("\xD8" "{\\O}")
+  ("\xD9" "\\`U")
+  ("\xDA" "\\'U")
+  ("\xDB" "\\^U")
+  ("\xDC" "\\\"U")
+  ("\xDD" "\\'Y")
+  ("\xDE" "{\\TH}")
+  ("\xDF" "{\\SS}")
+  ("\xE0" "\\`a")
+  ("\xE1" "\\'a")
+  ("\xE2" "\\^a")
+  ("\xE3" "\\~a")
+  ("\xE4" "\\\"a")
+  ("\xE5" "{\\aa}")
+  ("\xE6" "{\\ae}")
+  ("\xE7" "\\c{c}")
+  ("\xE8" "\\`e")
+  ("\xE9" "\\'e")
+  ("\xEA" "\\^e")
+  ("\xEB" "\\\"e")
+  ("\xEC" "\\`{\\i}")
+  ("\xED" "\\'{\\i}")
+  ("\xEE" "\\^{\\i}")
+  ("\xEF" "\\\"{\\i}")
+  ("\xF0" "{\\dh}")
+  ("\xF1" "\\~n")
+  ("\xF2" "\\`o")
+  ("\xF3" "\\'o")
+  ("\xF4" "\\^o")
+  ("\xF5" "\\~o")
+  ("\xF6" "\\\"o")
+  ("\xF7" "{\\oe}")
+  ("\xF8" "{\\o}")
+  ("\xF9" "\\`u")
+  ("\xFA" "\\'u")
+  ("\xFB" "\\^u")
+  ("\xFC" "\\\"u")
+  ("\xFD" "\\'y")
+  ("\xFE" "{\\th}")
+  ("\xFF" "{\\ss}"))
 
 (drd-table cyrillic-catcodes%
   ("" "\\CYRA")
