@@ -26,8 +26,8 @@
 ;; Global variables
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-(define tmtex-style "generic")
-(define tmtex-packages '())
+(tm-define tmtex-style "generic")
+(tm-define tmtex-packages '())
 (define tmtex-env (make-ahash-table))
 (define tmtex-serial 0)
 (define tmtex-auto-produce 0)
@@ -48,13 +48,12 @@
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (texmacs-modes
-  (elsevier-style% (in? tmtex-style '("elsart")))
+  (elsevier-style% (in? tmtex-style '("elsart" "jsc")))
+  (jsc-style% (in? tmtex-style '("jsc")) elsevier-style%)
   (natbib-package% (in? "cite-author-year" tmtex-packages)))
 
-(define (tmtex-set-style style body)
-  (set! tmtex-style (car style))
-  (set! tmtex-packages (cdr style))
-  (if (elsevier-style?) (init-elsevier body)))
+(tm-define (tmtex-style-init body)
+  (noop))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Language
@@ -426,13 +425,22 @@
 	((== (car l) 'hide-preamble) "")
 	(else (cons (car l) (map tmtex-filter-body (cdr l))))))
 
+(define (tmtex-apply-init body init)
+  ;;(display* "init= " init "\n")
+  (cond ((== (assoc-ref init "language") "verbatim")
+	 (with init* (assoc-remove! init "language")
+	   (tmtex-apply-init `(verbatim ,body) init*)))
+	(else body)))
+
 (define (tmtex-file l)
   (let* ((doc (car l))
 	 (styles (cadr l))
 	 (lang (caddr l))
-	 (init (cadddr l))
+	 (init (or (cadddr l) '(collection)))
+	 (init-bis (map (lambda (x) (cons (cadr x) (caddr x))) (cdr init)))
 	 (doc-preamble (tmtex-filter-preamble doc))
-	 (doc-body (tmtex-filter-body doc)))
+	 (doc-body-pre (tmtex-filter-body doc))
+	 (doc-body (tmtex-apply-init doc-body-pre init-bis)))
     (if (== (get-preference "texmacs->latex:expand-user-macros") "on")
 	(set! doc-preamble '()))
     (if (null? styles) (tmtex doc)
@@ -505,7 +513,7 @@
       (tex-concat (tmtex-math-concat-spaces (tmtex-list l)))
       (tex-concat (tmtex-list (tmtex-rewrite-no-break l)))))
 
-(define (tmtex-group l)
+(define (tmtex-rigid l)
   (tmtex-function '!group l))
 
 (define (tmtex-no-first-indentation l) (tex-apply 'noindent))
@@ -516,12 +524,13 @@
 (define (tmtex-next-line l) (list '!nextline))
 (define (tmtex-no-break l) '(!group (nobreak)))
 
-(define (tmtex-decode-length s)
+(define (tmtex-decode-length len)
   ;; FIXME: should be completed
-  (cond ((string-ends? s "fn") (string-replace s "fn" "em"))
-	((string-ends? s "spc") (string-replace s "spc" "em"))
-	((string-ends? s "par") (string-replace s "par" "\\columnwidth"))
-	(else s)))
+  (with s (force-string len)
+    (cond ((string-ends? s "fn") (string-replace s "fn" "em"))
+	  ((string-ends? s "spc") (string-replace s "spc" "em"))
+	  ((string-ends? s "par") (string-replace s "par" "\\columnwidth"))
+	  (else s))))
 
 (define (tmtex-hspace l)
   (let ((s (if (= (length l) 1) (car l) (cadr l))))
@@ -585,6 +594,15 @@
 ;; Mathematics
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
+(define (tmtex-around l)
+  (tmtex-concat (cdr (downgrade-brackets (cons 'around l)))))
+
+(define (tmtex-around* l)
+  (tmtex-concat (cdr (downgrade-brackets (cons 'around* l)))))
+
+(define (tmtex-big-around l)
+  (tmtex-concat (cdr (downgrade-brackets (cons 'big-around l)))))
+
 (define (tmtex-large-decode s)
   (cond ((nstring? s) ".")
         ((in? s '("(" ")" "[" "]" "|" "/" ".")) s)
@@ -629,6 +647,22 @@
 (define (tmtex-big l)
   (list (string->symbol (tmtex-big-decode (car l)))))
 
+(define (tmtex-decode-long-arrow s)
+  (cond ((nstring? s) 'xrightarrow)
+        ((and (string-starts? s "<rubber-") (string-ends? s ">"))
+         (tmtex-decode-long-arrow (substring s 8 (- (string-length s) 1))))
+        ((in? s '("minus" "leftarrow" "rightarrow" "leftrightarrow"
+                  "equal" "Leftarrow" "Rightarrow" "Leftrightarrow"
+                  "mapsto" "mapsfrom"))
+         (string->symbol (string-append "x" s)))
+        (else 'xrightarrow)))
+
+(define (tmtex-long-arrow l)
+  (with cmd (tmtex-decode-long-arrow (car l))
+    (if (== (length l) 2)
+        (list cmd (tmtex (cadr l)))
+        (list cmd (list '!option (tmtex (caddr l))) (tmtex (cadr l))))))
+
 (define (tmtex-prime-list l)
   (if (null? l) l
       (cond ((== (car l) #\<)
@@ -673,10 +707,14 @@
 	(list which r))))
 
 (define (tmtex-rsub l)
-  (tmtex-script '!sub (car l)))
+  (if (tmtex-math-mode?)
+      (tmtex-script '!sub (car l))
+      (list 'tmrsub (tmtex (car l)))))
 
 (define (tmtex-rsup l)
-  (tmtex-script '!sup (car l)))
+  (if (tmtex-math-mode?)
+      (tmtex-script '!sup (car l))
+      (list 'tmrsup (tmtex (car l)))))
 
 (define (tmtex-frac l)
   (tmtex-function 'frac l))
@@ -1025,14 +1063,36 @@
 	    (convert-to-file u fm "postscript-file" name-url)
 	    (list 'includegraphics name-string))))))
 
-(define (tmtex-postscript l)
+(define (tmtex-image-length len)
+  (let* ((s (force-string len))
+	 (unit (and (tm-length? s) (tm-length-unit len))))
+    (cond ((== s "") "!")
+	  ((string-ends? s "%") "!")
+	  ((in? unit '("w" "h")) "!")
+	  (else (tmtex-decode-length len)))))
+
+(define (tmtex-image-mag len)
+  (let* ((s (force-string len))
+	 (val (and (tm-length? s) (tm-length-value len)))
+	 (unit (and (tm-length? s) (tm-length-unit len))))
+    (cond ((== s "") 0.0)
+	  ((string-ends? s "%")
+	   (with x (string->number (string-drop-right s 1))
+	     (if x (/ x 100.0) 0)))
+	  ((in? unit '("w" "h")) (or val 0))
+	  (else #f))))
+
+(define (tmtex-image l)
   (let* ((fig (tmtex-as-eps (force-string (car l))))
-	 (hor (if (== (cadr l) "") "!" (tmtex-decode-length (cadr l))))
-	 (ver (if (== (caddr l) "") "!" (tmtex-decode-length (caddr l)))))
-    (if (or (string-starts? hor "*") (string-starts? hor "/")) (set! hor "!"))
-    (if (or (string-starts? ver "*") (string-starts? ver "/")) (set! ver "!"))
-    (if (and (== hor "!") (== ver "!")) fig
-	(list 'resizebox hor ver fig))))
+	 (hor (tmtex-image-length (cadr l)))
+	 (ver (tmtex-image-length (caddr l)))
+	 (mhor (tmtex-image-mag (cadr l)))
+	 (mver (tmtex-image-mag (caddr l))))
+    (cond ((or (not mhor) (not mver)) (list 'resizebox hor ver fig))
+	  ((and (== mhor 0.0) (== mver 0.0)) fig)
+	  ((or (== mhor 1.0) (== mver 1.0)) fig)
+	  ((== mhor 0.0) (list 'scalebox (number->string mver) fig))
+	  (else (list 'scalebox (number->string mhor) fig)))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Titles of documents
@@ -1198,6 +1258,54 @@
 (define (tmtex-math s l)
   (tmtex `(with "mode" "math" ,(car l))))
 
+(define (tmtex-textual x)
+  (tmtex-env-set "mode" "text")
+  (with r (tmtex x)
+    (tmtex-env-reset "mode")
+    r))
+
+(define (tmtex-math-up s l)
+  (list 'mathup (tmtex-textual (car l))))
+
+(define (tmtex-math-ss s l)
+  (list 'mathsf (tmtex-textual (car l))))
+
+(define (tmtex-math-tt s l)
+  (list 'mathtt (tmtex-textual (car l))))
+
+(define (tmtex-math-bf s l)
+  (list 'mathbf (tmtex-textual (car l))))
+
+(define (tmtex-math-sl s l)
+  (list 'mathsl (tmtex-textual (car l))))
+
+(define (tmtex-math-it s l)
+  (list 'mathit (tmtex-textual (car l))))
+
+(define (tmtex-mathord s l)
+  (list 'mathord (tmtex (car l))))
+
+(define (tmtex-mathbin s l)
+  (list 'mathbin (tmtex (car l))))
+
+(define (tmtex-mathrel s l)
+  (list 'mathrel (tmtex (car l))))
+
+(define (tmtex-mathopen s l)
+  (list 'mathopen (tmtex (car l))))
+
+(define (tmtex-mathclose s l)
+  (list 'mathclose (tmtex (car l))))
+
+(define (tmtex-mathpunct s l)
+  (list 'mathpunct (tmtex (car l))))
+
+(define (tmtex-mathop s l)
+  (list 'mathop (tmtex (car l))))
+
+(define (tmtex-syntax s l)
+  (tmtex (car l)))
+
 (define (tmtex-dummy s l)
   "")
 
@@ -1253,28 +1361,36 @@
 (define (tmtex-nbsp s l)
   '(!nbsp))
 
+(define (tmtex-nbhyph s l)
+  '(!nbhyph))
+
 (define (tmtex-session s l)
-  (tmtex (car l)))
+  (tmtex (cAr l)))
 
 (define (tmtex-input s l)
   (let ((prompt (car l)) (x (cadr l)))
     (tex-concat
-     (list `(!group (!concat (color "red") (ttfamily ,(tmtex prompt))))
+     (list '(noindent)
+	   `(!group (!concat (color "red") (ttfamily ,(tmtex prompt))))
 	   (cond ((func? x 'math 1)
 		  (tmtex-env-set "mode" "math")
 		  (let ((r (tmtex (cadr x))))
 		    (tmtex-env-reset "mode")
 		    `(!math (!group (!concat (color "blue") ,r)))))
 		 (else `(!group (!concat (color "blue")
-					 (!verb ,(tmtex-tt x))))))))))
+					 (!verb ,(tmtex-tt x))))))
+	   '(smallskip)))))
 
 (define (tmtex-output s l)
-  (list '!group (list 'ttfamily (tmtex (car l)))))
+  (tex-concat
+   (list '(noindent)
+	 (list '!group (list 'ttfamily (tmtex (car l))))
+	 '(medskip))))
 
 (define (tmtex-hlink s l)
   (list 'href (tmtex (cadr l)) (tmtex (car l))))
 
-(define (tmtex-slink s l)
+(define (tmtex-href s l)
   (tmtex-function 'url l))
 
 (define (tmtex-action s l)
@@ -1428,7 +1544,7 @@
   (para tmtex-para)
   (surround tmtex-surround)
   (concat tmtex-concat)
-  (group tmtex-group)
+  (rigid tmtex-rigid)
   (hidden tmtex-noop)
   (hspace tmtex-hspace)
   (vspace* tmtex-noop)
@@ -1459,10 +1575,14 @@
   (new-dpage* tmtex-noop)
   (new-dpage tmtex-noop)
 
+  (around tmtex-around)
+  (around* tmtex-around*)
+  (big-around tmtex-big-around)
   (left tmtex-left)
   (mid tmtex-mid)
   (right tmtex-right)
   (big tmtex-big)
+  (long-arrow tmtex-long-arrow)
   (lprime tmtex-lprime)
   (rprime tmtex-rprime)
   (below tmtex-below)
@@ -1499,6 +1619,7 @@
   ((:or unquote* copy
 	if if* case while for-each
 	extern include use-package) tmtex-noop)
+  (syntax tmtex-syntax)
 
   ((:or or xor and not plus minus times over div mod
 	merge length range number date translate change-case find-file
@@ -1533,7 +1654,7 @@
   (superpose tmtex-noop)
   ((:or gr-group gr-linear-transform
 	text-at cline arc carc spline spine* cspline fill) tmtex-noop)
-  (postscript tmtex-postscript)
+  (image tmtex-image)
   ((:or box-info frame-direct frame-inverse) tmtex-noop)
 
   ((:or format line-sep split delay hold release
@@ -1583,9 +1704,37 @@
   (really-huge (,tmtex-Huge 1))
 
   (math (,tmtex-math 1))
+  (math-up (,tmtex-math-up 1))
+  (math-ss (,tmtex-math-ss 1))
+  (math-tt (,tmtex-math-tt 1))
+  (math-bf (,tmtex-math-bf 1))
+  (math-sl (,tmtex-math-sl 1))
+  (math-it (,tmtex-math-it 1))
+  (math-separator (,tmtex-mathpunct 1))
+  (math-quantifier (,tmtex-mathord 1))
+  (math-imply (,tmtex-mathbin 1))
+  (math-or (,tmtex-mathbin 1))
+  (math-and (,tmtex-mathbin 1))
+  (math-not (,tmtex-mathord 1))
+  (math-relation (,tmtex-mathrel 1))
+  (math-union (,tmtex-mathbin 1))
+  (math-intersection (,tmtex-mathbin 1))
+  (math-exclude (,tmtex-mathbin 1))
+  (math-plus (,tmtex-mathbin 1))
+  (math-minus (,tmtex-mathbin 1))
+  (math-times (,tmtex-mathbin 1))
+  (math-over (,tmtex-mathbin 1))
+  (math-big (,tmtex-mathop 1))
+  (math-prefix (,tmtex-mathord 1))
+  (math-postfix (,tmtex-mathord 1))
+  (math-open (,tmtex-mathopen 1))
+  (math-close (,tmtex-mathclose 1))
+  (math-ordinary (,tmtex-mathord 1))
+  (math-ignore (,tmtex-mathord 1))
   ((:or equation equation*) (,tmtex-equation-wrapper 1))
   ((:or eqnarray eqnarray* leqnarray*) (,tmtex-eqnarray 1))
   (eq-number (,tmtex-default -1))
+
   (the-index (,tmtex-dummy -1))
   (glossary (,tmtex-glossary 1))
   (glossary-explain (,tmtex-glossary 2))
@@ -1601,12 +1750,13 @@
   (item* (,tmtex-item-arg 1))
   (render-proof (,tmtex-render-proof 2))
   (nbsp (,tmtex-nbsp 0))
-  (session (,tmtex-session 1))
+  (nbhyph (,tmtex-nbhyph 0))
+  (session (,tmtex-session -1))
   (input (,tmtex-input 2))
   (output (,tmtex-output 1))
   (hlink (,tmtex-hlink 2))
-  (action (,tmtex-action 2))
-  (slink (,tmtex-slink 1))
+  (action (,tmtex-action -1))
+  (href (,tmtex-href 1))
   (choose (,tmtex-choose 2))
   ((:or strong em tt name samp abbr dfn kbd var acronym person)
    (,tmtex-modifier 1))
@@ -1685,18 +1835,15 @@
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (define (tmtex-get-style sty)
-  (cond ((string? sty) (set! sty (list sty)))
+  (cond ((not sty) (set! sty (list "generic")))
+	((string? sty) (set! sty (list sty)))
 	((func? sty 'tuple) (set! sty (cdr sty)))
 	((null? sty) (set! sty '("letter"))))
   (if (== (car sty) "generic") (set! sty (cons "letter" (cdr sty))))
   sty)
 
-(define (find? x what)
-  (cond ((== x what) #t)
-	((nlist? x) #f)
-	(else (list-or (map (cut find? <> what) x)))))
-
 (tm-define (texmacs->latex x opts)
+  ;;(display* "texmacs->latex [" opts "], " x "\n")
   (if (tmfile? x)
       (let* ((body (tmfile-extract x 'body))
 	     (style (tmtex-get-style (tmfile-extract x 'style)))
@@ -1705,11 +1852,17 @@
 	     (init (tmfile-extract x 'initial))
 	     (doc (list '!file body style lan init (get-texmacs-path))))
 	(latex-set-style main-style)
+	(latex-set-packages '())
 	(latex-set-language lan)
-	(tmtex-set-style style body)
+	(set! tmtex-style (car style))
+	(set! tmtex-packages (cdr style))
+	(when (elsevier-style?)
+	  (import-from (convert latex tmtex-elsevier)))
+	(tmtex-style-init body)
 	(tmtex-set-language lan)
 	(with result (texmacs->latex doc opts)
-	  (tmtex-set-style '("generic") "")
+	  (set! tmtex-style "generic")
+	  (set! tmtex-packages '())
 	  (tmtex-set-language "english")
 	  result))
       (let* ((x2 (tmtm-eqnumber->nonumber x))
