@@ -5,10 +5,9 @@
 ;; DESCRIPTION : DRD properties for MathML
 ;; COPYRIGHT   : (C) 2004  Joris van der Hoeven
 ;;
-;; This software falls under the GNU general public license and comes WITHOUT
-;; ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for details.
-;; If you don't have this file, write to the Free Software Foundation, Inc.,
-;; 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+;; This software falls under the GNU general public license version 3 or later.
+;; It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
+;; in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ;;
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
@@ -112,16 +111,16 @@
   ("odot" "&CircleDot;")
   ("oplus" "&CirclePlus;")
   ("otimes" "&CircleTimes;")
-;;  ("sqcap" "&SquareIntersection;")
-;;  ("sqcup" "&SquareUnion;")
-;;  ("curlywedge" "&CurlyWedge;")
-;;  ("curlyvee" "&CurlyVee;")
-;;  ("triangleup" "&TriangleUp;")
-;;  ("triangledown" "&TriangleDown;")
-;;  ("box" "&Box;")
+  ("sqcap" "&SquareIntersection;")     ;; FIXME: displayed too small
+  ("sqcup" "&SquareUnion;")            ;; FIXME: displayed too small
+  ;;("curlywedge" "&CurlyWedge;")
+  ;;("curlyvee" "&CurlyVee;")
+  ;;("triangleup" "&TriangleUp;")
+  ;;("triangledown" "&TriangleDown;")
+  ;;("box" "&Box;")
   ("pluscup" "&UnionPlus;")
-;;  ("parallel" "&Parallel;")
-;;  ("interleave" "&Interleave;")
+  ;;("parallel" "&Parallel;")
+  ;;("interleave" "&Interleave;")
 )
 
 (drd-table tmtm-big%
