@@ -15,7 +15,8 @@
   (:use (utils library tree)
 	(utils library cursor)
 	(utils plugins plugin-cmd)
-	(convert tools tmconcat)))
+	(convert tools tmconcat)
+	(dynamic scripts-drd)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Some switches
@@ -44,7 +45,7 @@
   (with lan (get-env "prog-scripts")
     (or (connection-defined? lan)
 	(begin
-	  (set-message (string-append "plugin '" lan "' not defined") "")
+	  (set-message `(concat "undefined plugin: " (verbatim ,lan)) "")
 	  #f))))
 
 (tm-define (script-evaluable?)
@@ -55,37 +56,44 @@
   (tm-in? t '(script-eval script-result script-approx)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;; In place asynchroneous plug-in evaluations
+;; In place asynchronous plug-in evaluations
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (tm-define (script-feed lan ses in out opts)
-  (when (not (supports-scripts? lan))
-    (with s (string-append "Error:#" lan "#is not a scripting language")
-      (set-message s "Evaluate")))
-  (when (supports-scripts? lan)
-    (tree-set! out '(script-busy))
-    (with ptr (tree->tree-pointer out)
-      (with ret (lambda (r)
-		  (with check (tree-pointer->tree ptr)
-		    (tree-pointer-detach ptr)
-		    (when (== check out)
-		      (with-cursor (tree->path check :end)
-			(tree-select out)
-			(clipboard-cut "dummy")
-			(if (and (in-var-math?) (tm-func? r 'math 1))
-			    (set! r (cadr r)))
-			(if (in? :declaration opts)
-			    (insert in)
-			    (insert r))))))
-	(silent-feed* lan ses in ret opts)))))
+  ;;(with ok? (supports-scripts? lan)
+  (with ok? (or (supports-scripts? lan) (supports-sessions? lan))
+    (when (not ok?)
+      (with m `(concat "Error: " (verbatim ,lan)
+                       " is not a scripting language")
+        (set-message m "Evaluate")))
+    (when ok?
+      (tree-set! out '(script-busy))
+      (with ptr (tree->tree-pointer out)
+        (with ret (lambda (r)
+                    (with check (tree-pointer->tree ptr)
+                      (tree-pointer-detach ptr)
+                      (when (== check out)
+                        (with-cursor (tree->path check :end)
+                          (tree-select out)
+                          (clipboard-cut "dummy")
+                          (if (and (in-var-math?) (tm-func? r 'math 1))
+                              (set! r (cadr r)))
+                          (if (in? :declaration opts)
+                              (insert in)
+                              (insert r))))))
+          (silent-feed* lan ses in ret opts))))))
 
 (tm-define (script-eval-at where lan session in . opts)
   (script-feed lan session in where opts))
 
-(tm-define (kbd-return)
-  (:inside script-eval)
-  (with-innermost t 'script-eval
-    (script-modified-eval noop)))
+(tm-define (kbd-enter t forwards?)
+  (:require (and (tree-is? t 'script-eval)
+                 (xor (not forwards?)
+                      (tree-is? t 1 'document))))
+  (script-modified-eval noop))
+
+(tm-define (evaluate-context? t)
+  (tree-in? t '(script-input script-output)))
 
 (tm-define (make-script-input)
   (let* ((lan (get-env "prog-scripts"))
@@ -93,22 +101,33 @@
     (insert-go-to `(script-input ,lan ,session "" "")
 		  '(2 0))))
 
-(tm-define (hidden-variant)
-  (:inside script-input)
-  (with-innermost t 'script-input
-    (let* ((lan (tree->string (tree-ref t 0)))
-	   (session (tree->string (tree-ref t 1)))
-	   (in (tree->stree (tree-ref t 2)))
-	   (out (tree-ref t 3)))
-      (script-eval-at out lan session in :math-input :simplify-output)
-      (tree-assign-node! t 'script-output)
-      (tree-go-to t 3 :end))))
+(tm-define (alternate-toggle t)
+  (:require (tree-is? t 'script-input))
+  (let* ((lan (tree->string (tree-ref t 0)))
+         (session (tree->string (tree-ref t 1)))
+         (in (tree->stree (tree-ref t 2)))
+         (out (tree-ref t 3)))
+    (script-eval-at out lan session in :math-input :simplify-output)
+    (tree-assign-node! t 'script-output)
+    (tree-go-to t 3 :end)))
 
-(tm-define (hidden-variant)
-  (:inside script-output)
-  (with-innermost t 'script-output
-    (tree-assign-node! t 'script-input)
-    (tree-go-to t 2 :end)))
+(tm-define (alternate-toggle t)
+  (:require (tree-is? t 'script-output))
+  (tree-assign-node! t 'script-input)
+  (tree-go-to t 2 :end))
+
+(tm-define (kbd-enter t forwards?)
+  (:require (or (tree-is? t 'script-output)
+                (and (tree-is? t 'script-input)
+                     (not (tree-is? t :up 'inactive)))))
+  (cond ((tree-is? t 'script-output)
+         (alternate-toggle t))
+        ((xor (not forwards?) (tree-is? t 2 'document))
+         (alternate-toggle t))
+        (else
+         (if (not (tree-is? t 2 'document))
+             (tree-set t 2 `(document ,(tree-ref t 2))))
+         (insert-return))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Operate on current selection or formula
@@ -234,6 +253,9 @@
 ;; Plots
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
+(tm-define (plot-context? t)
+  (tree-in? t '(plot-curve plot-curve* plot-surface plot-surface*)))
+
 (tm-define (script-plot-command lan t)
   (cond ((== (car t) 'plot-curve)
 	 `(concat "set samples 1000 ~ "
@@ -260,56 +282,72 @@
 		  ", " ,(tm-ref t 1)
 		  ", " ,(tm-ref t 2)))))
 
-(define (activate-plot)
-  (with-innermost t '(plot-curve plot-curve* plot-surface plot-surface*)
-    (let* ((lan "gnuplot")
-	   (session "default")
-	   (in (script-plot-command lan (tree->stree t))))
-      (tree-set! t `(plot-output ,t ""))
-      (script-eval-at (tree-ref t 1) lan session in :math-correct :math-input)
-      (tree-go-to t 1 :end))))
+(define (activate-plot t)
+  (let* ((lan "gnuplot")
+         (session "default")
+         (in (script-plot-command lan (tree->stree t))))
+    (tree-set! t `(plot-output ,t ""))
+    (script-eval-at (tree-ref t 1) lan session in :math-correct :math-input)
+    (tree-go-to t 1 :end)))
 
-(tm-define (kbd-return)
-  (:inside plot-curve plot-curve* plot-surface plot-surface*)
-  (with-innermost t '(plot-curve plot-curve* plot-surface plot-surface*)
-    (if (= (tree-down-index t) (- (tree-arity t) 1))
-	(activate-plot)
-	(tree-go-to t (1+ (tree-down-index t)) :end))))
+(tm-define (alternate-toggle t)
+  (:require (plot-context? t))
+  (activate-plot t))
 
-(tm-define (hidden-variant)
-  (:inside plot-curve plot-curve* plot-surface plot-surface*)
-  (activate-plot))
+(tm-define (alternate-toggle t)
+  (:require (tree-is? t 'plot-output))
+  (tree-remove-node! t 0)
+  (tree-go-to t 0 :end))
 
-(tm-define (hidden-variant)
-  (:inside plot-output)
-  (with-innermost t 'plot-output
-    (tree-remove-node! t 0)
-    (tree-go-to t 0 :end)))
+(tm-define (kbd-enter t forwards?)
+  (:require (plot-context? t))
+  (if (= (tree-down-index t) (- (tree-arity t) 1))
+      (activate-plot t)
+      (tree-go-to t (1+ (tree-down-index t)) :end)))
+
+(tm-define (kbd-enter t forwards?)
+  (:require (tree-is? t 'plot-output))
+  (alternate-toggle t))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Converters
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-(tm-define (kbd-return)
-  (:inside converter-eval)
-  (with-innermost t 'converter-eval
-    (let* ((format (string-append (tree->string (tree-ref t 0)) "-snippet"))
-	   (in (texmacs->verbatim (tree-ref t 1))))
-      (tree-select t)
-      (clipboard-cut "primary")
-      (insert (convert in format "texmacs-tree")))))
+(tm-define (converter-context? t)
+  (tree-in? t '(converter-input converter-output)))
 
-(tm-define (hidden-variant)
-  (:inside converter-input)
-  (with-innermost t 'converter-input
-    (let* ((format (string-append (tree->string (tree-ref t 0)) "-snippet"))
-	   (in (texmacs->verbatim (tree-ref t 1))))
-      (tree-set! t 2 (convert in format "texmacs-tree"))
-      (tree-assign-node! t 'converter-output)
-      (tree-go-to t 2 :end))))
+(tm-define (kbd-enter t forwards?)
+  (:require (and (tree-is? t 'converter-eval)
+                 (xor (not forwards?)
+                      (tree-is? t 1 'document))))
+  (let* ((format (string-append (tree->string (tree-ref t 0)) "-snippet"))
+         (in (texmacs->verbatim (tree-ref t 1))))
+    (tree-select t)
+    (clipboard-cut "primary")
+    (insert (convert in format "texmacs-tree"))))
 
-(tm-define (hidden-variant)
-  (:inside converter-output)
-  (with-innermost t 'converter-output
-    (tree-assign-node! t 'converter-input)
-    (tree-go-to t 1 :end)))
+(tm-define (alternate-toggle t)
+  (:require (tree-is? t 'converter-input))
+  (let* ((format (string-append (tree->string (tree-ref t 0)) "-snippet"))
+         (in (texmacs->verbatim (tree-ref t 1))))
+    (tree-set! t 2 (convert in format "texmacs-tree"))
+    (tree-assign-node! t 'converter-output)
+    (tree-go-to t 2 :end)))
+
+(tm-define (alternate-toggle t)
+  (:require (tree-is? t 'converter-output))
+  (tree-assign-node! t 'converter-input)
+  (tree-go-to t 1 :end))
+
+(tm-define (kbd-enter t forwards?)
+  (:require (or (tree-is? t 'converter-output)
+                (and (tree-is? t 'converter-input)
+                     (not (tree-is? t :up 'inactive)))))
+  (cond ((tree-is? t 'converter-output)
+         (alternate-toggle t))
+        ((xor (not forwards?) (tree-is? t 1 'document))
+         (alternate-toggle t))
+        (else
+         (if (not (tree-is? t 1 'document))
+             (tree-set t 1 `(document ,(tree-ref t 1))))
+         (insert-return))))
