@@ -31,6 +31,12 @@
 (tm-property (project-attach master)
   (:argument master "Master file"))
 
+(tm-define (get-style-list)
+  (with t (tree->stree (get-style-tree))
+    (cond ((string? t) (list t))
+          ((and (pair? t) (== (car t) 'tuple)) (cdr t))
+          (else (texmacs-error "get-style-list ""invalid style ~S" t)))))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Preamble mode
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -83,24 +89,36 @@
 	(chinese-before (in? (tree->stree (get-init-tree "language"))
 			     '("chinese" "taiwanese")))
 	(chinese-after (in? lan '("chinese" "taiwanese")))
-	(oriental-before (in? (tree->stree (get-init-tree "language"))
-			      '("japanese" "korean")))
-	(oriental-after (in? lan '("japanese" "korean"))))
+	(japanese-before (in? (tree->stree (get-init-tree "language"))
+			      '("japanese")))
+	(japanese-after (in? lan '("japanese")))
+	(korean-before (in? (tree->stree (get-init-tree "language"))
+			    '("korean")))
+	(korean-after (in? lan '("korean"))))
     (if (and cyrillic-before (not cyrillic-after)) (init-default "font"))
     (if (and chinese-before (not chinese-after)) (init-default "font"))
-    (if (and oriental-before (not oriental-after)) (init-default "font"))
+    (if (and japanese-before (not japanese-after)) (init-default "font"))
+    (if (and korean-before (not korean-after)) (init-default "font"))
     (init-env "language" lan)
     (if (and cyrillic-after (not cyrillic-before))
 	(init-env "font" "cyrillic"))
     (if (and chinese-after (not chinese-before))
-	(init-env "font" "fireflysung"))
-    (if (and oriental-after (not oriental-before))
-	(init-env "font" "modern"))))
+	(init-env "font" "sys-chinese"))
+    (if (and japanese-after (not japanese-before))
+	(init-env "font" "sys-japanese"))
+    (if (and korean-after (not korean-before))
+	(init-env "font" "sys-korean"))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Main page layout
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
+(define (test-default-page-medium?) (test-default? "page-medium"))
+(tm-define (init-default-page-medium)
+  (:check-mark "*" test-default-page-medium?)
+  (init-default "page-medium")
+  (notify-page-change))
+
 (define (test-page-medium? s) (string=? (get-env "page-medium") s))
 (tm-define (init-page-medium s)
   (:check-mark "*" test-page-medium?)
