@@ -34,6 +34,7 @@
       ("Default" (init-default "info-flag"))
       ---
       ("None" (init-env "info-flag" "none"))
+      ("Minimal" (init-env "info-flag" "minimal"))
       ("Short" (init-env "info-flag" "short"))
       ("Detailed" (init-env "info-flag" "detailed"))
       ("Also on paper" (init-env "info-flag" "paper")))
@@ -180,6 +181,98 @@
      (init-language "ukrainian")
      (set-output-language "ukrainian"))))
 
+(tm-menu (supported-scripts-menu)
+  (let* ((dummy (lazy-plugin-force))
+         (l (list-sort supported-scripts-list string<=?)))
+    (for (name l)
+      ((eval (ahash-ref supported-scripts-table name))
+       (noop) ;; NOTE: inhibit segfault due to property searching?
+       (init-env "prog-scripts" name)))))
+
+(tm-menu (document-page-size-menu)
+  ("Default"
+   (init-default "page-type" "page-width" "page-height")
+   (notify-page-change))
+  ---
+  (group "Common formats")
+  ("A3" (init-page-type "a3"))
+  ("A4" (init-page-type "a4"))
+  ("A5" (init-page-type "a5"))
+  ("B4" (init-page-type "b4"))
+  ("B5" (init-page-type "b5"))
+  ("Letter" (init-page-type "letter"))
+  ("Legal" (init-page-type "legal"))
+  ("Executive" (init-page-type "executive"))
+  ---
+  (group "Standard formats")
+  (-> "A series"
+      ("A0" (init-page-type "a0"))
+      ("A1" (init-page-type "a1"))
+      ("A2" (init-page-type "a2"))
+      ("A3" (init-page-type "a3"))
+      ("A4" (init-page-type "a4"))
+      ("A5" (init-page-type "a5"))
+      ("A6" (init-page-type "a6"))
+      ("A7" (init-page-type "a7"))
+      ("A8" (init-page-type "a8"))
+      ("A9" (init-page-type "a9")))
+  (-> "B series"
+      ("B0" (init-page-type "b0"))
+      ("B1" (init-page-type "b1"))
+      ("B2" (init-page-type "b2"))
+      ("B3" (init-page-type "b3"))
+      ("B4" (init-page-type "b4"))
+      ("B5" (init-page-type "b5"))
+      ("B6" (init-page-type "b6"))
+      ("B7" (init-page-type "b7"))
+      ("B8" (init-page-type "b8"))
+      ("B9" (init-page-type "b9")))
+  (-> "Arch series"
+      ("ArchA" (init-page-type "archA"))
+      ("ArchB" (init-page-type "archB"))
+      ("ArchC" (init-page-type "archC"))
+      ("ArchD" (init-page-type "archD"))
+      ("ArchE" (init-page-type "archE")))
+  (-> "American"
+      ("10x14" (init-page-type "10x14"))
+      ("11x17" (init-page-type "11x17"))
+      ("C5" (init-page-type "C5"))
+      ("Comm10" (init-page-type "Comm10"))
+      ("DL" (init-page-type "DL"))
+      ("Executive" (init-page-type "executive"))
+      ("Half letter" (init-page-type "halfletter"))
+      ("Half executive" (init-page-type "halfexecutive"))
+      ("Ledger" (init-page-type "ledger"))
+      ("Legal" (init-page-type "legal"))
+      ("Letter" (init-page-type "letter"))
+      ("Monarch" (init-page-type "Monarch")))
+  (-> "Miscellaneous"
+      ("C sheet" (init-page-type "csheet"))
+      ("D sheet" (init-page-type "dsheet"))
+      ("E sheet" (init-page-type "esheet"))
+      ("Flsa" (init-page-type "flsa"))
+      ("Flse" (init-page-type "flse"))
+      ("Folio" (init-page-type "folio"))
+      ("Lecture note" (init-page-type "lecture note"))
+      ("Note" (init-page-type "note"))
+      ("Quarto" (init-page-type "quarto"))
+      ("Statement" (init-page-type "statement"))
+      ("Tabloid" (init-page-type "tabloid")))
+  ---
+  ("Other" (interactive init-page-size)))
+
+(tm-menu (document-font-base-size-menu)
+  ("Default" (init-default "font-base-size"))
+  ---
+  ("8" (init-env "font-base-size" "8"))
+  ("9" (init-env "font-base-size" "9"))
+  ("10" (init-env "font-base-size" "10"))
+  ("11" (init-env "font-base-size" "11"))
+  ("12" (init-env "font-base-size" "12"))
+  ("14" (init-env "font-base-size" "14"))
+  ---
+  ("Other" (init-interactive-env "font-base-size")))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; The Document menu
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -203,10 +296,10 @@
       (-> "Text font"
 	  ("Default" (init-default "font"))
 	  ---
-	  ("Roman" (init-env "font" "roman"))
-	  ("Pandora" (init-env "font" "pandora"))
 	  ("Concrete" (init-env "font" "concrete"))
 	  ("Cyrillic" (init-env "font" "cyrillic"))
+	  ("Pandora" (init-env "font" "pandora"))
+	  ("Roman" (init-env "font" "roman"))
 	  ---
 	  ("Bookman" (init-env "font" "bookman"))
 	  ("Courier" (init-env "font" "courier"))
@@ -214,37 +307,100 @@
 	  ("Palatino" (init-env "font" "palatino"))
 	  ("Times" (init-env "font" "times"))
 	  ---
-	  (if (> (get-font-type) 0)
-	      (if (font-exists-in-tt? "luxirr")
-		  ("Luxi" (init-env "font" "luxi"))))
+	  (if (font-exists-in-tt? "DejaVuSerif")
+	      ("Dejavu" (init-env "font" "dejavu")))
 	  ("Lucida" (init-env "font" "x-lucida"))
-	  ("Utopia" (init-env "font" "x-utopia")))
+	  (if (font-exists-in-tt? "luxirr")
+              ("Luxi" (init-env "font" "luxi")))
+	  (if (font-exists-in-tt? "STIXGeneral")
+	      ("Stix" (init-env "font" "stix")))
+	  ("Utopia" (init-env "font" "x-utopia"))
+	  (if (or (supports-chinese?)
+		  (supports-japanese?)
+		  (supports-korean?))
+	      ---
+	      (if (font-exists-in-tt? "Batang")
+		  ("Batang" (init-env "font" "batang")))
+	      (if (font-exists-in-tt? "fireflysung")
+		  ("Fireflysung" (init-env "font" "fireflysung")))
+	      (if (font-exists-in-tt? "AppleGothic")
+		  ("Gothic" (init-env "font" "apple-gothic")))
+	      (if (font-exists-in-tt? "Gulim")
+		  ("Gulim" (init-env "font" "gulim")))
+	      (if (font-exists-in-tt? "华文细黑")
+		  ("HeiTi" (init-env "font" "heiti")))
+	      (if (font-exists-in-tt? "ヒラギノ明朝 ProN W6")
+		  ("Hiragino Kaku" (init-env "font" "kaku")))
+	      (if (font-exists-in-tt? "ipam")
+		  ("Ipa" (init-env "font" "ipa")))
+	      (if (font-exists-in-tt? "ttf-japanese-gothic")
+		  ("Japanese" (init-env "font" "ttf-japanese")))
+	      (if (font-exists-in-tt? "kochi-mincho")
+		  ("Kochi" (init-env "font" "kochi")))
+	      (if (font-exists-in-tt? "儷黑 Pro")
+		  ("LiHei" (init-env "font" "lihei")))
+	      (if (font-exists-in-tt? "wqy-microhei")
+		  ("MicroHei" (init-env "font" "wqy-microhei")))
+	      (if (font-exists-in-tt? "mingliu")
+		  ("MingLiU" (init-env "font" "mingliu")))
+	      (if (font-exists-in-tt? "PMingLiU")
+		  ("MingLiU" (init-env "font" "pmingliu")))	      
+	      (if (font-exists-in-tt? "MS Gothic")
+		  ("MS Gothic" (init-env "font" "ms-gothic")))
+	      (if (font-exists-in-tt? "MS Mincho")
+		  ("MS Mincho" (init-env "font" "ms-mincho")))
+	      (if (font-exists-in-tt? "sazanami-gothic")
+		  ("Sazanami" (init-env "font" "sazanami")))
+	      (if (font-exists-in-tt? "simfang")
+		  ("SimFang" (init-env "font" "simfang")))
+	      (if (font-exists-in-tt? "simhei")
+		  ("SimHei" (init-env "font" "simhei")))
+	      (if (font-exists-in-tt? "simkai")
+		  ("SimKai" (init-env "font" "simkai")))
+	      (if (font-exists-in-tt? "simli")
+		  ("SimLi" (init-env "font" "simli")))
+	      (if (font-exists-in-tt? "simsun")
+		  ("SimSun" (init-env "font" "simsun")))
+	      (if (and (font-exists-in-tt? "SimSun")
+		       (not (font-exists-in-tt? "simsun")))
+		  ("SimSun" (init-env "font" "apple-simsun")))
+	      (if (font-exists-in-tt? "simyou")
+		  ("SimYou" (init-env "font" "simyou")))
+	      (if (font-exists-in-tt? "ukai")
+		  ("UKai" (init-env "font" "ukai")))
+	      (if (font-exists-in-tt? "unbatang")
+		  ("Unbatang" (init-env "font" "unbatang")))
+	      (if (font-exists-in-tt? "uming")
+		  ("UMing" (init-env "font" "uming")))
+	      (if (font-exists-in-tt? "wqy-zenhei")
+		  ("ZenHei" (init-env "font" "wqy-zenhei")))))
       (-> "Mathematical font"
 	  ("Default" (init-default "math-font"))
 	  ---
-	  ("Roman" (init-env "math-font" "roman"))
+	  ("Adobe" (init-env "math-font" "adobe"))
+	  (if (font-exists-in-tt? "Apple Symbols")
+	      ("Apple symbols" (init-env "math-font" "math-apple")))
+	  (if (font-exists-in-tt? "Asana-Math")
+	      ("Asana" (init-env "math-font" "math-asana")))
 	  ("Concrete" (init-env "math-font" "concrete"))
+	  (if (font-exists-in-tt? "DejaVuSerif")
+	      ("Dejavu" (init-env "math-font" "math-dejavu")))
 	  ("Euler new roman" (init-env "math-font" "ENR"))
-	  ("Adobe" (init-env "math-font" "adobe")))
+	  (if (font-exists-in-tt? "LucidaGrande")
+	      ("Lucida" (init-env "math-font" "math-lucida")))
+	  ("Roman" (init-env "math-font" "roman"))
+	  (if (font-exists-in-tt? "STIXGeneral")
+	      ("Stix" (init-env "math-font" "math-stix"))))
       (-> "Program font"
 	  ("Default" (init-default "prog-font"))
 	  ---
-	  ("Roman" (init-env "prog-font" "roman"))
-	  ("Pandora" (init-env "prog-font" "pandora"))
 	  ("Concrete" (init-env "prog-font" "concrete"))
+	  ("Pandora" (init-env "prog-font" "pandora"))
+	  ("Roman" (init-env "prog-font" "roman"))
 	  ("Times" (init-env "prog-font" "times")))
       ---
       (-> "Size"
-	  ("Default" (init-default "font-base-size"))
-	  ---
-	  ("8" (init-env "font-base-size" "8"))
-	  ("9" (init-env "font-base-size" "9"))
-	  ("10" (init-env "font-base-size" "10"))
-	  ("11" (init-env "font-base-size" "11"))
-	  ("12" (init-env "font-base-size" "12"))
-	  ("14" (init-env "font-base-size" "14"))
-	  ---
-	  ("Other" (init-interactive-env "font-base-size")))
+	  (link document-font-base-size-menu))
       (-> "Dpi"
 	  ("Default" (init-default "dpi"))
 	  ---
@@ -273,37 +429,18 @@
       (-> "Foreground"
 	  ("Default" (init-default "color"))
 	  ---
-	  ("Black" (init-env "color" "black"))
-	  ("White" (init-env "color" "white"))
-	  ("Grey" (init-env "color" "grey"))
-	  ("Red" (init-env "color" "red"))
-	  ("Blue" (init-env "color" "blue"))
-	  ("Yellow" (init-env "color" "yellow"))
-	  ("Green" (init-env "color" "green"))
-	  ("Orange" (init-env "color" "orange"))
-	  ("Magenta" (init-env "color" "magenta"))
-	  ("Brown" (init-env "color" "brown"))
-	  ("Pink" (init-env "color" "pink"))
+	  (pick-color (init-env "color" answer))
 	  ---
+          ("Palette" (interactive-color
+                      (lambda (col) (init-env "color" col)) '()))
 	  ("Other" (init-interactive-env "color")))
       (-> "Background"
 	  ("Default" (init-default "bg-color"))
 	  ---
-	  ("Black" (init-env "bg-color" "black"))
-	  ("White" (init-env "bg-color" "white"))
-	  ("Grey" (init-env "bg-color" "grey"))
-	  ("Red" (init-env "bg-color" "red"))
-	  ("Blue" (init-env "bg-color" "blue"))
-	  ("Yellow" (init-env "bg-color" "yellow"))
-	  ("Green" (init-env "bg-color" "green"))
-	  ("Orange" (init-env "bg-color" "orange"))
-	  ("Magenta" (init-env "bg-color" "magenta"))
-	  ("Brown" (init-env "bg-color" "brown"))
-	  ("Pink" (init-env "bg-color" "pink"))
-	  ("Light grey" (init-env "bg-color" "light grey"))
-	  ("Dark grey" (init-env "bg-color" "dark grey"))
-	  ("Broken white" (init-env "bg-color" "broken white"))
+	  (pick-background (init-env-tree "bg-color" answer))
 	  ---
+          ("Palette" (interactive-background
+                      (lambda (col) (init-env "bg-color" col)) '()))
 	  ("Other" (init-interactive-env "bg-color"))))
   (if (detailed-menus?)
       (-> "Language"
@@ -372,85 +509,13 @@
 	  ("3" (init-env "par-columns" "3"))))
   (-> "Page"
       (-> "Type"
-	  ("Default" (check "*" (test-default? "page-medium"))
-	   (init-default "page-medium")
-	   (notify-page-change))
+	  ("Default" (init-default-page-medium))
 	  ---
 	  ("Paper" (init-page-medium "paper"))
 	  ("Papyrus" (init-page-medium "papyrus"))
 	  ("Automatic" (init-page-medium "automatic")))
       (-> "Size"
-	  ("Default"
-	   (check "*" (test-default? "page-type" "page-width" "page-height"))
-	   (init-default "page-type" "page-width" "page-height")
-	   (notify-page-change))
-	  ---
-	  (group "Common formats")
-	  ("A3" (init-page-type "a3"))
-	  ("A4" (init-page-type "a4"))
-	  ("A5" (init-page-type "a5"))
-	  ("B4" (init-page-type "b4"))
-	  ("B5" (init-page-type "b5"))
-	  ("Letter" (init-page-type "letter"))
-	  ("Legal" (init-page-type "legal"))
-	  ("Executive" (init-page-type "executive"))
-	  ---
-	  (group "Standard formats")
-	  (-> "A series"
-	      ("A0" (init-page-type "a0"))
-	      ("A1" (init-page-type "a1"))
-	      ("A2" (init-page-type "a2"))
-	      ("A3" (init-page-type "a3"))
-	      ("A4" (init-page-type "a4"))
-	      ("A5" (init-page-type "a5"))
-	      ("A6" (init-page-type "a6"))
-	      ("A7" (init-page-type "a7"))
-	      ("A8" (init-page-type "a8"))
-	      ("A9" (init-page-type "a9")))
-	  (-> "B series"
-	      ("B0" (init-page-type "b0"))
-	      ("B1" (init-page-type "b1"))
-	      ("B2" (init-page-type "b2"))
-	      ("B3" (init-page-type "b3"))
-	      ("B4" (init-page-type "b4"))
-	      ("B5" (init-page-type "b5"))
-	      ("B6" (init-page-type "b6"))
-	      ("B7" (init-page-type "b7"))
-	      ("B8" (init-page-type "b8"))
-	      ("B9" (init-page-type "b9")))
-	  (-> "Arch series"
-	      ("ArchA" (init-page-type "archA"))
-	      ("ArchB" (init-page-type "archB"))
-	      ("ArchC" (init-page-type "archC"))
-	      ("ArchD" (init-page-type "archD"))
-	      ("ArchE" (init-page-type "archE")))
-	  (-> "American"
-	      ("10x14" (init-page-type "10x14"))
-	      ("11x17" (init-page-type "11x17"))
-	      ("C5" (init-page-type "C5"))
-	      ("Comm10" (init-page-type "Comm10"))
-	      ("DL" (init-page-type "DL"))
-	      ("Executive" (init-page-type "executive"))
-	      ("Half letter" (init-page-type "halfletter"))
-	      ("Half executive" (init-page-type "halfexecutive"))
-	      ("Ledger" (init-page-type "ledger"))
-	      ("Legal" (init-page-type "legal"))
-	      ("Letter" (init-page-type "letter"))
-	      ("Monarch" (init-page-type "Monarch")))
-	  (-> "Miscellaneous"
-	      ("C sheet" (init-page-type "csheet"))
-	      ("D sheet" (init-page-type "dsheet"))
-	      ("E sheet" (init-page-type "esheet"))
-	      ("Flsa" (init-page-type "flsa"))
-	      ("Flse" (init-page-type "flse"))
-	      ("Folio" (init-page-type "folio"))
-	      ("Lecture note" (init-page-type "lecture note"))
-	      ("Note" (init-page-type "note"))
-	      ("Quarto" (init-page-type "quarto"))
-	      ("Statement" (init-page-type "statement"))
-	      ("Tabloid" (init-page-type "tabloid")))
-	  ---
-	  ("Other" (interactive init-page-size)))
+	  (link document-page-size-menu))
       (-> "Orientation"
 	  ("Default" (check "*" (test-default? "page-orientation"))
 	   (init-default "page-orientation")
@@ -513,3 +578,71 @@
       (if (project-attached?)
 	  ---
 	  ("Clear local information" (clear-local-info)))))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;; Document focus menus
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+(tm-menu (focus-document-extra-menu t))
+
+(tm-menu (focus-style-menu t)
+  (group "Style")
+  (let* ((st* (get-style-list))
+         (st (if (null? st*) (list "no style") st*)))
+    (-> (eval (upcase-first (car st)))
+        (link style-menu))
+    (for (pack (cdr st))
+      (-> (eval (upcase-first pack))
+          ("Remove package" (init-remove-package pack)))))
+  (-> "Add style package"
+      (link add-package-menu)))
+
+(tm-menu (focus-document-menu t)
+  (group "Document")
+  (-> (eval (upcase-first (get-init "page-type")))
+      (link document-page-size-menu))
+  (-> (eval (string-append (get-init "font-base-size") " pt"))
+      (link document-font-base-size-menu)))
+
+(tm-menu (standard-focus-menu t)
+  (:require (tree-is-buffer? t))
+  (dynamic (focus-style-menu t))
+  ---
+  (dynamic (focus-document-menu t))
+  (dynamic (focus-document-extra-menu t))
+  ---
+  ("Help" (focus-help)))
+
+(tm-menu (focus-document-extra-icons t))
+
+(tm-menu (focus-style-icons t)
+  (minibar
+   (let* ((st* (get-style-list))
+          (st (if (null? st*) (list "no style") st*)))
+     (=> (balloon (eval (upcase-first (car st))) "Document style")
+	 (link style-menu))
+     (for (pack (cdr st))
+       (=> (eval pack)
+	   ("Remove package" (init-remove-package pack)))))
+   (=> (balloon (icon "tm_add.xpm") "Add style package")
+       (link add-package-menu))
+   (assuming (tree-is-buffer? t)
+     ((balloon (icon "tm_focus_help.xpm") "Describe tag")
+      (focus-help)))))
+
+(tm-menu (focus-document-icons t)
+  (minibar
+   (=> (balloon (eval (upcase-first (get-init "page-type")))
+		"Paper size")
+       (link document-page-size-menu))
+   (=> (balloon (eval (string-append (get-init "font-base-size") "pt"))
+		"Font size")
+       (link document-font-base-size-menu))))
+
+(tm-menu (standard-focus-icons t)
+  (:require (tree-is-buffer? t))
+  (dynamic (focus-style-icons t))  
+  (glue #f #f 5 0)
+  (dynamic (focus-document-icons t))  
+  (glue #f #f 5 0)   
+  (dynamic (focus-document-extra-icons t)))
