@@ -5,6 +5,7 @@
 ;; DESCRIPTION : utilities routines for graphics mode
 ;; COPYRIGHT   : (C) 2001  Joris van der Hoeven
 ;;               (C) 2004, 2005, 2006  Joris van der Hoeven and Henri Lesourd
+;;               (C) 2011  Joris van der Hoeven
 ;;
 ;; This software falls under the GNU general public license version 3 or later.
 ;; It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
@@ -13,7 +14,20 @@
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (texmacs-module (graphics graphics-utils)
-  (:use (utils library cursor) (utils library tree)))
+  (:use (graphics graphics-drd)))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;; Some global definitions
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+(tm-define ShiftMask     256)
+(tm-define LockMask      512)
+(tm-define ControlMask  1024)
+(tm-define Mod1Mask     2048)
+(tm-define Mod2Mask     4096)
+(tm-define Mod3Mask     8192)
+(tm-define Mod4Mask    16384)
+(tm-define Mod5Mask    32768)
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Basic scheme processing
@@ -65,7 +79,7 @@
 
 ;; Error
 (tm-define (graphics-error . msg)
-  (foreach (e msg)
+  (for (e msg)
      (display e))
   (newline) ;(quit-TeXmacs)
 )
@@ -108,9 +122,8 @@
      `(remove-eq0? ,x ,l)))
 
 ;; Iterators
-(define-public foreach for)
 
-(define-public-macro (foreach-number what . body)
+(tm-define-macro (foreach-number what . body)
   (let ((n (length what)))
     (cond ((== n 3)
          ;;(foreach-number (i i0 iN) body[i])
@@ -133,7 +146,7 @@
 		     ,(- 0 (car (cddddr what))) ,(caddr what)) ,@body)))
           (else '(noop)))))
 
-(define-public-macro (foreach-cons i . b)
+(tm-define-macro (foreach-cons i . b)
 ;;(foreach-cons (e l) i   body[elt]) -> for each cons e of the list l
 ;;(foreach-cons (e l1 l2) body[elt]) -> for each cons e in [l1...l2]
   (if (null? (cddr i))
@@ -280,17 +293,6 @@
 ;; Subroutines for accessing the graphics
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-;;NOTE: This section is OK.
-(tm-define gr-tags-all '(point
-			 line cline spline cspline
-			 arc carc
-			 text-at
-			 gr-group))
-(tm-define gr-tags-curves       '(line cline spline cspline arc carc))
-(tm-define gr-tags-point-curves `(point . ,gr-tags-curves))
-(tm-define gr-tags-noncurves    '(point text-at gr-group))
-(tm-define gr-tags-oneshot      '(point arc carc text-at gr-group))
-
 (tm-define (stree-at p)
   (tree->stree (path->tree p)))
 ;; TODO: Put this in kernel/library/tree.scm
@@ -310,7 +312,8 @@
 	 (with o (path->tree p)
 	    (if (and (tree? o) ;; (in? (tree-label o) gr-tags-all)
 		     (not (eq? (tree-label o) 'string))
-		     (tm-upwards-path (cDr p) '(graphics) '(text-at)))
+		     (tm-upwards-path (cDr p) '(graphics)
+                                      (graphical-text-tag-list)))
 		(begin
 		  ;;(display* "gp=" (path->tree (cDr path)) "\n")
 		  (if (eq? (tree-label o) 'graphics) #f p))
@@ -340,178 +343,83 @@
 ;;NOTE: This section is OK.
 (define (graphics-get-raw-property var)
   (with val (get-upwards-tree-property (graphics-graphics-path) var)
-     (if (eq? val nothing)
-	 (get-default-tree-val var)
-	 (if (eq? (tm-car val) 'quote)
-	     (tree-ref val 0)
-	     val))))
-         
-(tm-define (graphics-frozen-property? var)
-  (with val (graphics-get-raw-property var)
-     (and (tree? val) (== (tree-label val) 'freeze))))
-         
-(tm-define (graphics-frozen-property! var b)
-  (if b
-      (if (not (graphics-frozen-property? var))
-          (graphics-set-property var
-            `(quote (freeze ,(tree->stree (get-env-tree var))))))
-      (if (graphics-frozen-property? var)
-          (graphics-set-property
-	     var (tree-ref (graphics-get-raw-property var) 0)))))
-         
+    (if (eq? val nothing)
+	(get-default-tree-val var)
+	(if (eq? (tm-car val) 'quote)
+	    (tree-ref val 0)
+	    val))))
+
 (tm-define (graphics-get-property var)
   (with val (graphics-get-raw-property var)
-     (tree->stree
-        (if (graphics-frozen-property? var)
-            (tree-ref val 0) 
-            val))))
-               
-(tm-define (graphics-change-property var val)
-  (set! val (t2o val))
-  (if (graphics-frozen-property? var)
-      (graphics-set-property var `(quote (freeze ,val)))
-      (graphics-set-property var val)))
+     (tree->stree val)))
 
 (tm-define (graphics-set-property var val)
   (with p (graphics-graphics-path)
-    (if p (path-insert-with p var val))))
+    (cond ((tree? val) (graphics-set-property var (tree->stree val)))
+          ((== val "default") (graphics-remove-property var))
+          ((== val (graphics-attribute-default var))
+           (graphics-remove-property var))
+          (p (path-insert-with p var val)))))
 
 (tm-define (graphics-remove-property var)
   (with p (graphics-graphics-path)
     (if p (path-remove-with p var))))
 
 ;; Magnification
-(tm-define (graphics-eval-magnification)
-  (tree->stree (get-env-tree-at
-		  "magnification" (rcons (graphics-graphics-path) 0))))
+(tm-define (graphics-eval-magnify)
+  (graphical-get-attribute (path->tree (graphics-graphics-path)) "magnify"))
 
-(tm-define (graphics-eval-magnification-at path)
-  (tree->stree (get-env-tree-at "magnification" path)))
+(tm-define (graphics-eval-magnify-at path)
+  (graphical-get-attribute (path->tree (cDr path)) "magnify"))
 
-(define (convert-magn m)
-  (if (== m "default")
-      (set! m 1.0))
-  (if (string? m)
-      (set! m (s2f m)))
-  m)
-  ;; FIXME: Using this function is crappy ; it would be
-  ;;   much better to be sure that a magnification equal
-  ;;   to "default" never enters the functions below.
-  ;;   But this seems to depend on some realtime-dependent
-  ;;   behaviour, thus testing this has not (yet) been
-  ;;   done correctly.
+(tm-define (magnify->number m)
+  (cond ((number? m) m)
+        ((== m "default") 1)
+        ((== m #f) 1)
+        (else (string->number m))))
 
-(tm-define (multiply-magnification magn h)
-  (set! h (convert-magn h))
-  (cond ((equal? h 1.0)
-         #t
-        )
-        ((and (pair? magn) (eq? (car magn) 'times))
-         (set! h (* h (s2f (cadr magn))))
-        )
-        ((or (string? magn) (number? magn))
-	 (set! magn (convert-magn magn))
-         (set! h (* h magn))
-        )
-        (else
-           (set! h 1.0))
-  )
-  (if (equal? h 1.0)
-      #f
-     `(times ,(f2s h) (value "magnification"))))
+(tm-define (number->magnify m)
+  (cond ((and (> m 0.999999) (< m 1.000001)) "default")
+        (else (number->string m))))
 
-(tm-define (local-magnification amagn)
-  (set! amagn (convert-magn amagn))
- `(times ,(f2s (/ amagn (s2f (graphics-eval-magnification))))
-	  (value "magnification")))
+(tm-define (multiply-magnify m1 m2)
+  (number->magnify (* (magnify->number m1) (magnify->number m2))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Enriching graphics with properties like color, line width, etc.
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-;;NOTE: This section is OK.
-(tm-define (graphics-valid-attribute? attr tag)
-  (cond ((== tag 'point)
-	 (in? attr '("color" "fill-color" "point-style")))
-	((not (in? tag gr-tags-noncurves))
-	 (in? attr '("color" "fill-color" "line-width"
-		     "magnification" "dash-style" "dash-style-unit"
-		     "line-arrows")))
-	((== tag 'text-at)
-	 (in? attr '("magnification" "text-at-halign" "text-at-valign")))
-	((== tag 'gr-group)
-	 (in? attr '("color" "fill-color"
-		     "point-style" "line-width"
-		     "magnification" "dash-style" "dash-style-unit"
-		     "line-arrows"
-		     "text-at-halign" "text-at-valign")))
-	(else #f)))
-
 (tm-define (graphics-enrich-filter t l)
   (if (null? l) l
       (let* ((head (car l))
 	     (tail (graphics-enrich-filter t (cdr l))))
-	(if (or (== (cadr head) "default")
+	(if (or (not (cadr head))
+                (== (cadr head) "default")
 		(== (cadr head) (get-default-val (car head)))
-		(and (== (car head) "magnification")
-		     (or (== (cadr head) "1.0")
-			 (== (cadr head) '(times "1.0" (value "magnification")))))
-		(not (graphics-valid-attribute? (car head) t)))
+		(not (graphics-attribute? t (car head))))
 	    tail
 	    (cons* (car head) (cadr head) tail)))))
 
 (tm-define (graphics-enrich-sub t l)
   (with f (graphics-enrich-filter (car t) l)
-    (if (null? f)
-	t
-	`(with ,@f ,t))))
+    (if (null? f) t `(with ,@f ,t))))
 
-(tm-define (graphics-enrich-bis t color ps lw mag st stu lp fc ha va)
-  (let* ((mode (car t)))
-    (cond ((== mode 'point)
-	   (graphics-enrich-sub t
-	    `(("color" ,color)
-	      ("fill-color" ,fc)
-	      ("point-style" ,ps))))
-	  ((not (in? mode gr-tags-noncurves))
-	   (graphics-enrich-sub t
-	    `(("color" ,color)
-	      ("line-width" ,lw)
-	      ("magnification" ,mag)
-	      ("dash-style" ,st) ("dash-style-unit" ,stu)
-	      ("line-arrows" ,lp)
-	      ("fill-color" ,fc))))
-	  ((== mode 'text-at)
-	   (graphics-enrich-sub t
-	    `(("magnification" ,mag)
-	      ("text-at-halign" ,ha)
-	      ("text-at-valign" ,va))))
-	  ((== mode 'gr-group)
-	   (graphics-enrich-sub t
-	    `(("color" ,color)
-	      ("point-style" ,ps)
-	      ("line-width" ,lw)
-	      ("magnification" ,mag)
-	      ("dash-style" ,st) ("dash-style-unit" ,stu)
-	      ("line-arrows" ,lp)
-	      ("fill-color" ,fc)
-	      ("text-at-halign" ,ha)
-	      ("text-at-valign" ,va))))
-	  (else
-	   (graphics-enrich-sub t '())))))
+(tm-define (graphics-enrich-bis t id tab)
+  (set! tab (list->ahash-table (ahash-table->list tab)))
+  (ahash-remove! tab "gid")
+  (let* ((attrs (graphical-relevant-attributes t))
+         (sel (ahash-table-select tab attrs))
+         (l1 (cons (cons "gid" id) (ahash-table->list sel)))
+         (l2 (map (lambda (x) (list (car x) (cdr x))) l1)))
+    ;;(display* "l= " l2 "\n")
+    (graphics-enrich-sub t l2)))
 
 (tm-define (graphics-enrich t)
-  (let* ((color (graphics-get-property "gr-color"))
-	 (ps (graphics-get-property "gr-point-style"))
-	 (lw (graphics-get-property "gr-line-width"))
-	 (mag "1.0")
-	 (st (graphics-get-property "gr-dash-style"))
-	 (stu (graphics-get-property "gr-dash-style-unit"))
-	 (lp (graphics-get-property "gr-line-arrows"))
-	 (fc (graphics-get-property "gr-fill-color"))
-	 (ha (graphics-get-property "gr-text-at-halign"))
-	 (va (graphics-get-property "gr-text-at-valign")))
-    (graphics-enrich-bis t color ps lw mag st stu lp fc ha va)))
+  (let* ((l1 (graphics-all-attributes))
+         (l2 (map gr-prefix l1))
+         (l3 (map graphics-get-property l2))
+         (tab (list->ahash-table (map cons l1 l3))))
+    (graphics-enrich-bis t "default" tab)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Subroutines for modifying the innermost group of graphics
@@ -520,8 +428,7 @@
 (tm-define (graphics-group-insert-bis t go-into)
  ;(display* "t=" t "\n")
   (let* ((p (graphics-group-path))
-	 (p2 #f)
-    )
+	 (p2 #f))
     (if (null? layer-of-last-removed-object)
 	(set! layer-of-last-removed-object #f))
     (if p (with n (if layer-of-last-removed-object
@@ -534,17 +441,15 @@
                       (tree-arity (path->tree p)))
 	    (tree-insert (path->tree p) n (list t))
 	    (if (func? t 'with)
-		(if (and go-into (func? (cAr t) 'text-at))
+		(if (and go-into (graphical-text-context? (cAr t)))
 		    (set! p2 (append p (list n (- (length t) 2) 0 0)))
 		    (set! p2 (append p (list n (- (length t) 2) 1))))
-		(if (and go-into (func? t 'text-at))
+		(if (and go-into (graphical-text-context? t))
 		    (set! p2 (append p (list n 0 0)))
-		    (set! p2 (append p (list n 0))))
-	    )
+		    (set! p2 (append p (list n 0)))))
 	    (go-to p2)
-	    (graphics-path p2)
-	  )
-	  #f)))
+	    (graphics-path p2))
+        #f)))
 
 (tm-define (graphics-group-insert t)
   (graphics-group-insert-bis t #t))
@@ -552,10 +457,9 @@
 (tm-define (graphics-group-enrich-insert t)
   (graphics-group-insert (graphics-enrich t)))
 
-(tm-define (graphics-group-enrich-insert-bis
-	    t color ps lw mag st stu lp fc ha va go-into)
-  (graphics-group-insert-bis
-    (graphics-enrich-bis t color ps lw mag st stu lp fc ha va) go-into))
+(tm-define (graphics-group-enrich-insert-table t tab go-into)
+  (graphics-group-insert-bis (graphics-enrich-bis t "default" tab)
+                             go-into))
 
 (tm-define (graphics-group-start)
   (graphics-finish)
@@ -575,41 +479,16 @@
   (with p (graphics-path path)
     (if p (tree->stree (path->tree p)) #f)))
 
-(tm-define (graphics-active-tree)
-  (with p (graphics-active-path)
-    (if p (path->tree p) #f)))
-
 (tm-define (graphics-active-object)
   (with p (graphics-active-path)
     (if p (tree->stree (path->tree p)) #f)))
 
-(tm-define (graphics-active-type)
-  (with t (graphics-active-tree)
-    (if t (tm-car t) #f)))
-  ;;NOTE: Currently unused.
-  ;;TODO: Test it.
-
-(tm-define (graphics-active-val var)
-  (graphics-active-property var (get-default-val var)))
-  ;;NOTE: Currently unused.
-  ;;TODO: Test it.
-
 (tm-define (get-default-tree-val var)
   (get-init-tree var))
 
 (tm-define (get-default-val var)
   (tree->stree (get-init-tree var)))
 
-(tm-define (graphics-active-color)
-  (graphics-active-val "color"))
-  ;;NOTE: Currently unused.
-  ;;TODO: Test it.
-
-(tm-define (graphics-active-lwidth)
-  (graphics-active-val "line-width"))
-  ;;NOTE: Currently unused.
-  ;;TODO: Test it.
-
 (tm-define (graphics-path-property-bis p var default-val)
   (with c (get-upwards-property p var)
     (if (== c nothing) default-val c)))
@@ -624,54 +503,31 @@
 (tm-define (graphics-path-property-1 p var)
   (graphics-path-property-bis-1 p var "default"))
 
-(tm-define (graphics-active-property var default-val)
-  (graphics-path-property-bis (graphics-active-path) var default-val))
-
-(tm-define (graphics-active-assign t)
-  (with p (graphics-active-path)
-    (if p (begin
-	    (tree-assign (path->tree p) t)
-	    (go-to (rcons p 1))))))
-
-(tm-define (graphics-active-set-tag l)
-  (with t (graphics-active-object)
-    (if t (graphics-active-assign (cons l (cdr t))))))
-
-(tm-define (graphics-active-insert t)
-  (with p (graphics-active-path)
-    (if p (with n (tree-arity (path->tree p))
-	    (tree-insert (path->tree p) n (list t))
-	    (go-to (rcons p 1))))))
-
 (tm-define (graphics-object-root-path p)
   (let* ((q (tm-upwards-path p '(with) '()))
-	 (path (if (and q
-			(== (+ (length q) 1) (length p)))
-		   q p
-	       )))
-	path))
-    
+	 (path (if (and q (== (+ (length q) 1) (length p))) q p)))
+    path))
+
 (tm-define (graphics-remove p . parms)
-  (if p
-  (with p0 (graphics-object-root-path p)
-     (set! layer-of-last-removed-object
-	   (if (and (pair? parms) (eq? (car parms) 'memoize-layer))
-	       (if (list? layer-of-last-removed-object)
-		   (cons (cAr p0) layer-of-last-removed-object)
-		   (cAr p0))
-	       #f))
-     (tree-remove (path->tree (cDr p0)) (cAr p0) 1))))
+  (when p
+    (with p0 (graphics-object-root-path p)
+      (set! layer-of-last-removed-object
+            (if (and (pair? parms) (eq? (car parms) 'memoize-layer))
+                (if (list? layer-of-last-removed-object)
+                    (cons (cAr p0) layer-of-last-removed-object)
+                    (cAr p0))
+                #f))
+      (tree-remove (path->tree (cDr p0)) (cAr p0) 1))))
 
 (tm-define (graphics-assign p t)
-  (if p (begin
-	  (tree-assign (path->tree p) t)
-	  (go-to (rcons p 1)))))
+  (when p
+    (tree-assign (path->tree p) t)
+    (go-to (rcons p 1))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Box info & frame
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-;;NOTE: This section is OK.
 (tm-define (box-info t cmd)
   (tree->stree (texmacs-exec `(box-info ,t ,cmd))))
 
@@ -685,28 +541,24 @@
   (let* ((i1a (car i1))
 	 (i1b (cadr i1))
 	 (i2a (car i2))
-	 (i2b (cadr i2))
-     )
-     (or (and (<= i1a i2a) (>= i1b i2b))
-	 (and (<= i2a i1a) (>= i2b i1b))
-	 (and (>= i1a i2a) (<= i1a i2b))
-	 (and (>= i2a i1a) (<= i2a i1b)))))
+	 (i2b (cadr i2)))
+    (or (and (<= i1a i2a) (>= i1b i2b))
+        (and (<= i2a i1a) (>= i2b i1b))
+        (and (>= i1a i2a) (<= i1a i2b))
+        (and (>= i2a i1a) (<= i2a i1b)))))
 
 (tm-define (box-intersects t1 t2)
   (define (max-box t)
-     (let* ((bx1 (box-info t "lbrt"))
-	    (bx2 (box-info t "LBRT"))
-	)
-	(set! bx1 (map s2f (cdr bx1)))
-	(set! bx2 (map s2f (cdr bx2)))
-       `(,(min (car bx1) (car bx2))
-	 ,(min (cadr bx1) (cadr bx2))
-	 ,(max (caddr bx1) (caddr bx2))
-	 ,(max (cadddr bx1) (cadddr bx2))))
-  )
+    (let* ((bx1 (box-info t "lbrt"))
+           (bx2 (box-info t "LBRT")))
+      (set! bx1 (map s2f (cdr bx1)))
+      (set! bx2 (map s2f (cdr bx2)))
+      `(,(min (car bx1) (car bx2))
+        ,(min (cadr bx1) (cadr bx2))
+        ,(max (caddr bx1) (caddr bx2))
+        ,(max (cadddr bx1) (cadddr bx2)))))
   (let* ((box1 (max-box t1))
-	 (box2 (max-box t2))
-    )
+	 (box2 (max-box t2)))
     (and (interval-intersects `(,(car box1) ,(caddr box1))
 			      `(,(car box2) ,(caddr box2)))
 	 (interval-intersects `(,(cadr box1) ,(cadddr box1))
@@ -724,53 +576,107 @@
       (tree-ref t (- (tree-arity t) 1))
       t))
 
+(tm-define (radical->enhanced-tree r)
+  (with t (tree-up r)
+     (if (enhanced-tree? t) t r)))
+
 (tm-define (stree-radical t)
   (if (and (pair? t) (eq? (car t) 'with) (nnull? (cdr t)))
       (cAr t)
       t))
 
-(tm-define (radical->enhanced-tree r)
-  (with t (tree-up r)
-     (if (enhanced-tree? t) t r)))
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;; New style graphical attributes
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-(tm-define (enhanced-tree-arity t)
-  (tree-arity (enhanced-tree->radical t)))
+(define (group-pairs l)
+  (if (or (null? l) (null? (cdr l))) '()
+      (cons (cons (car l) (cadr l)) (group-pairs (cddr l)))))
 
-(tm-define (enhanced-tree-ref t i)
-  (tree-ref (enhanced-tree->radical t) i))
+(tm-define (with-get-attributes t)
+  (let* ((a1 (cDr (tree-children t)))
+         (a2 (map tree->stree a1))
+         (a3 (group-pairs a2)))
+    (list->ahash-table a3)))
 
-(tm-define (enhanced-tree-set! t i val)
-  (tree-set! (enhanced-tree->radical t) i val))
+(define (ungroup-pairs l)
+  (if (null? l) l
+      (cons (caar l)
+            (cons (cdar l)
+                  (ungroup-pairs (cdr l))))))
 
-(tm-define (enhanced-tree-insert! t i val)
-  (tree-insert! (enhanced-tree->radical t) i `(,val)))
+(tm-define (with-set-attributes t tab)
+  (let* ((a1 (ahash-table->list tab))
+         (a2 (ungroup-pairs a1))
+         (a3 (map tm->tree a2)))
+    (tree-remove! t 0 (- (tree-arity t) 1))
+    (tree-insert! t 0 a3)
+    t))
 
-(tm-define (enhanced-tree-remove! t i n)
-  (tree-remove! (enhanced-tree->radical t) i n))
+(tm-define (graphical-get-attributes t)
+  (cond ((not (tree? t))
+         (graphical-get-attributes (tm->tree t)))
+        ((not (tree-is? t 'with))
+         (if (tree-is? t :up 'with)
+             (graphical-get-attributes (tree-ref t :up))
+             (make-ahash-table)))
+        (else (with-get-attributes t))))
 
-(tm-define (enhanced-tree-property-ref t var)
-  (define res #f)
-  (if (string? var)
-      (set! var (stree->tree var)))
-  (if (enhanced-tree? t)
-      (foreach-number (i 0 < (- (tree-arity r) 1))
-	 (if (equal? (tree-ref t i) var)
-	     (set! res (tree-ref t (+ i 1))))
-	 (set! i (+ 1 i))))
-  res)
+(tm-define (graphical-set-attributes t tab)
+  (cond ((not (tree? t))
+         (tree->stree (graphical-set-attributes (tm->tree t) tab)))
+        ((not (tree-is? t 'with))
+         (if (tree-is? t :up 'with)
+             (graphical-set-attributes (tree-ref t :up) tab)
+             (begin
+               (tree-insert-node! t 0 '(with))
+               (graphical-set-attributes t tab))))
+        (else
+         (set! t (with-set-attributes t tab))
+         (if (and (tree-is? t 'with) (== (tree-arity t) 1))
+             (tree-remove-node! t 0))
+         t)))
 
-(tm-define (enhanced-tree-property-set! t var val)
-  (define found #f)
-  (if (string? var)
-      (set! var (stree->tree var)))
-  (if (enhanced-tree? t)
-      (begin
-	 (foreach-number (i 0 < (- (tree-arity r) 1))
-	    (if (equal? (tree-ref t i) var)
-		(begin
-		   (set! found #t)
-		   (tree-set! t (+ i 1) val)))
-	    (set! i (+ 1 i))
-	 )
-         (if (not found)
-	     (tree-insert! t (tree-arity t) `(,var ,val))))))
+(tm-define (graphical-get-attribute* t var)
+  (ahash-ref (graphical-get-attributes t) var))
+
+(tm-define (graphical-get-attribute t var)
+  (or (graphical-get-attribute* t var)
+      (graphics-attribute-default var)))
+
+(tm-define (graphical-set-attribute t var val)
+  (with new (graphical-get-attributes t)
+    (if val
+        (ahash-set! new var val)
+        (ahash-remove! new var))
+    (graphical-set-attributes t new)))
+
+(tm-define (graphical-get-selected-attributes* t filter-list)
+  (let* ((old (graphical-get-attributes t))
+         (new (make-ahash-table)))
+    (for (var filter-list)
+      (if (ahash-ref old var)
+          (ahash-set! new var (ahash-ref old var))))
+    new))
+
+(tm-define (graphical-get-selected-attributes t filter-list)
+  (with tab (graphical-get-selected-attributes t filter-list)
+    (for (var filter-list)
+      (if (and (not (ahash-ref tab var)) (graphics-attribute-default var))
+          (ahash-set! tab var (graphics-attribute-default var))))
+    tab))
+
+(tm-define (graphical-set-selected-attributes t filter-list tab)
+  (with new (graphical-get-attributes t)
+    (for (var filter-list)
+      (if (ahash-ref tab var)
+          (ahash-set! new var (ahash-ref tab var))
+          (ahash-remove! new var)))
+    (graphical-set-attributes t new)))
+
+(tm-define (graphical-relevant-attributes t)
+  (cond ((tm-is? t 'with)
+         (graphical-relevant-attributes (tm-ref t :last)))
+        ((tm-compound? t)
+         (graphical-relevant-attributes (tm-car t)))
+        (else (graphics-attributes t))))
