@@ -92,6 +92,10 @@
       `(apply (lambda ,var ,@body) ,val)
       `(let ((,var ,val)) ,@body)))
 
+(define-public-macro (with-define fun fun-body . body)
+  `(let ((,(car fun) (lambda ,(cdr fun) ,fun-body)))
+     ,@body))
+
 (define-public-macro (with-global var val . body)
   (let ((old (gensym)) (new (gensym)))
     `(let ((,old ,var))
@@ -145,6 +149,9 @@
   (let ((x (gensym)))
     `(for (,x 0 ,n) ,@body)))
 
+(define-public-macro (twice . body)
+  `(begin ,@body ,@body))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Small rewritings on top of C++ interface
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
