@@ -21,6 +21,8 @@
 (define (guile-c?) (equal? (scheme-dialect) "guile-c"))
 (define (guile-b-c?) (or (guile-b?) (guile-c?)))
 (if (guile-c?) (use-modules (ice-9 rdelim) (ice-9 pretty-print)))
+(define has-look-and-feel? (lambda (x) (== x "emacs")))
+(define (experimental-qt-gui?) #f)
 
 ;; Should be defined 
 (define dialogue-break #f)
@@ -145,17 +147,3 @@
     `(begin
        (define-module ,name)
        ,@l)))
-
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;; Lazy modules for extern mactos in style packages
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-
-(define markup-modules-list '((utils misc markup-funcs)))
-
-(define-macro (lazy-markup-modules . l)
-  `(set! markup-modules-list (append markup-modules-list ',l)))
-
-(define-macro (lazy-markup-modules-force)
-  `(begin
-     (use-modules ,@markup-modules-list)
-     (set! markup-modules-list '())))
