@@ -12,64 +12,75 @@
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (texmacs-module (table table-menu)
-  (:use (table table-edit)))
+  (:use (table table-edit)
+        (generic generic-menu)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Inserting tables
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (menu-bind insert-table-menu
-  (if (style-has? "std-dtd")
-      (when (and (in-text?) (style-has? "env-float-dtd"))
-	    ("Small table" (make 'small-table))
-	    ("Big table" (make 'big-table))
-	    ---))
+  (if (and (style-has? "std-dtd") (in-text?) (style-has? "env-float-dtd"))
+      ("Small table" (make 'small-table))
+      ("Big table" (make 'big-table))
+      ---)
   ("Plain tabular" (make 'tabular))
   ("Centered tabular" (make 'tabular*))
   ("Plain block" (make 'block))
   ("Centered block" (make 'block*))
-  (if (style-has? "std-dtd")
-      (when (in-math?)
-	    ---
-	    ("Matrix" (make 'matrix))
-	    ("Determinant" (make 'det))
-	    ("Choice" (make 'choice))
-	    ("Stack" (make 'stack)))))
+  (if (and (style-has? "std-dtd") (in-math?))
+      ---
+      ("Matrix" (make 'matrix))
+      ("Determinant" (make 'det))
+      ("Choice" (make 'choice))
+      ("Stack" (make 'stack))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Submenus of the Table menu
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
+(menu-bind table-size-menu
+  ("Set number of rows" (interactive table-set-rows))
+  ("Minimal number of rows" (table-interactive-set "table-min-rows"))
+  ("Minimal number of columns" (table-interactive-set "table-min-cols"))
+  ---
+  ("Set number of columns" (interactive table-set-columns))
+  ("Maximal number of rows" (table-interactive-set "table-max-rows"))
+  ("Maximal number of columns" (table-interactive-set "table-max-cols")))
+
 (menu-bind table-width-menu
   ("Automatic" (table-set-automatic-width))
   ("Paragraph" (table-set-exact-width "1par"))
-  ("Exact" (check "o" (table-test-exact-width?))
-   (interactive table-set-exact-width))
-  ("Minimal" (check "o" (table-test-minimal-width?))
-   (interactive table-set-minimal-width))
-  ("Maximal" (check "o" (table-test-maximal-width?))
-   (interactive table-set-maximal-width)))
+  ("Exact" (table-ia-exact-width))
+  ("Minimal" (table-ia-minimal-width))
+  ("Maximal" (table-ia-maximal-width)))
 
 (menu-bind table-height-menu
   ("Automatic" (table-set-automatic-height))
-  ("Exact" (check "o" (table-test-exact-height?))
-   (interactive table-set-exact-height))
-  ("Minimal" (check "o" (table-test-minimal-height?))
-   (interactive table-set-minimal-height))
-  ("Maximal" (check "o" (table-test-maximal-height?))
-   (interactive table-set-maximal-height)))
+  ("Exact" (table-ia-exact-height))
+  ("Minimal" (table-ia-minimal-height))
+  ("Maximal" (table-ia-maximal-height)))
 
 (menu-bind table-halign-menu
   ("Left" (table-set-halign "l"))
   ("Middle" (table-set-halign "c"))
-  ("Right" (table-set-halign "r")))
+  ("Right" (table-set-halign "r"))
+  ---
+  ("Left baseline" (table-set-halign "L"))
+  ("Middle baseline" (table-set-halign "C"))
+  ("Right baseline" (table-set-halign "R"))
+  ("Specific baseline" (interactive table-specific-halign)))
 
 (menu-bind table-valign-menu
-  ("Bottom" (table-set-valign "b"))
-  ("Baseline" (table-set-valign "B"))
-  ("Middle" (table-set-valign "c"))
   ("Axis" (table-set-valign "f"))
-  ("Top" (table-set-valign "t")))
+  ("Bottom" (table-set-valign "b"))
+  ("Middle" (table-set-valign "c"))
+  ("Top" (table-set-valign "t"))
+  ---
+  ("Bottom baseline" (table-set-valign "B"))
+  ("Middle baseline" (table-set-valign "C"))
+  ("Top baseline" (table-set-valign "T"))
+  ("Specific baseline" (interactive table-specific-valign)))
 
 (menu-bind table-border-menu
   ("All" (interactive table-set-border))
@@ -85,21 +96,10 @@
   ("Bottom" (table-interactive-set "table-bsep"))
   ("Top" (table-interactive-set "table-tsep")))
 
-(menu-bind table-limits-menu
-  ("Minimal number of rows" (table-interactive-set "table-min-rows"))
-  ("Minimal number of columns" (table-interactive-set "table-min-cols"))
-  ("Maximal number of rows" (table-interactive-set "table-max-rows"))
-  ("Maximal number of columns" (table-interactive-set "table-max-cols")))
-
 (menu-bind table-special-menu
+  ("Table breaking" (toggle-table-hyphen))
   ("Deactivate" (table-disactivate))
-  ("Extract format" (table-extract-format))
-  ---
-  ("Hyphenation" (toggle-table-hyphen))
-  (-> "Origin"
-      ("Row" (table-interactive-set "table-row-origin"))
-      ("Column" (table-interactive-set "table-col-origin")))
-  (-> "Size limits" (link table-limits-menu)))
+  ("Extract format" (table-extract-format)))
 
 (menu-bind cell-mode-menu
   ("Cells" (set-cell-mode "cell"))
@@ -135,21 +135,15 @@
 
 (menu-bind cell-width-menu
   ("Automatic" (cell-set-automatic-width))
-  ("Exact" (check "o" (cell-test-exact-width?))
-   (interactive cell-set-exact-width))
-  ("Minimal" (check "o" (cell-test-minimal-width?))
-   (interactive cell-set-minimal-width))
-  ("Maximal" (check "o" (cell-test-maximal-width?))
-   (interactive cell-set-maximal-width)))
+  ("Exact" (cell-ia-exact-width))
+  ("Minimal" (cell-ia-minimal-width))
+  ("Maximal" (cell-ia-maximal-width)))
 
 (menu-bind cell-height-menu
   ("Automatic" (cell-set-automatic-height))
-  ("Exact" (check "o" (cell-test-exact-height?))
-   (interactive cell-set-exact-height))
-  ("Minimal" (check "o" (cell-test-minimal-height?))
-   (interactive cell-set-minimal-height))
-  ("Maximal" (check "o" (cell-test-maximal-height?))
-   (interactive cell-set-maximal-height)))
+  ("Exact" (cell-ia-exact-height))
+  ("Minimal" (cell-ia-minimal-height))
+  ("Maximal" (cell-ia-maximal-height)))
 
 (menu-bind cell-border-menu
   ("All" (interactive cell-set-border))
@@ -169,31 +163,27 @@
   ("None" (cell-set-background ""))
   ("Foreground" (cell-set-background "foreground"))
   ---
-  ("Black" (cell-set-background "black"))
-  ("White" (cell-set-background "white"))
-  ("Grey" (cell-set-background "grey"))
-  ("Red" (cell-set-background "red"))
-  ("Blue" (cell-set-background "blue"))
-  ("Yellow" (cell-set-background "yellow"))
-  ("Green" (cell-set-background "green"))
-  ("Orange" (cell-set-background "orange"))
-  ("Magenta" (cell-set-background "magenta"))
-  ("Brown" (cell-set-background "brown"))
-  ("Pink" (cell-set-background "pink"))
+  (pick-background (cell-set-background answer))
   ---
-  ("Other"  (interactive cell-set-background)))
+  ("Palette" (interactive-background
+              (lambda (col) (cell-set-background "color" col)) '()))
+  ("Other" (interactive cell-set-background)))
 
 (menu-bind cell-special-menu
-  ("Set span" (interactive cell-set-span))
-  ("Subtable" (make-subtable))
+  (when (== (get-cell-mode) "cell")
+    ("Insert subtable" (make-subtable)))
   ---
+  (when (== (get-cell-mode) "cell")
+    (-> "Cell span"
+        ("Horizontal" (interactive cell-set-column-span))
+        ("Vertical" (interactive cell-set-row-span))))
   (-> "Text height correction"
       ("Off" (cell-set-vcorrect "n"))
       ---
       ("Bottom" (cell-set-vcorrect "b"))
       ("Top" (cell-set-vcorrect "t"))
       ("Both" (cell-set-vcorrect "a")))
-  (-> "Hyphenation"
+  (-> "Line wrapping"
       ("Off" (cell-set-hyphen "n"))
       ---
       ("Top" (cell-set-hyphen "t"))
@@ -201,7 +191,7 @@
       ("Bottom" (cell-set-hyphen "b")))
   (-> "Block content"
       ("Never" (cell-set-block "no"))
-      ("If hyphenated" (cell-set-block "auto"))
+      ("When line wrapping" (cell-set-block "auto"))
       ("Always" (cell-set-block "yes")))
   (-> "Distribute unused space"
       ("Horizontal part" (cell-interactive-set "cell-hpart"))
@@ -214,29 +204,31 @@
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (menu-bind table-menu
-  (-> "Insert"
-      ("Row above" (table-insert-row #f))
-      ("Row below" (table-insert-row #t))
-      ("Column to the left" (table-insert-column #f))
-      ("Column to the right" (table-insert-column #t))
-      ("Blank row" (interactive table-insert-blank-row))
-      ("Blank column" (interactive table-insert-blank-column)))
-  (-> "Remove"
-      ("This row" (table-remove-row #f))
-      ("This column" (table-remove-column #f)))
-  ---
+  (assuming #f
+    (-> "Insert"
+        ("Row above" (table-insert-row #f))
+        ("Row below" (table-insert-row #t))
+        ("Column to the left" (table-insert-column #f))
+        ("Column to the right" (table-insert-column #t))
+        ("Blank row" (interactive table-insert-blank-row))
+        ("Blank column" (interactive table-insert-blank-column)))
+    (-> "Remove"
+        ("This row" (table-remove-row #f))
+        ("This column" (table-remove-column #f)))
+    ---)
+  (-> "Size" (link table-size-menu))
   (-> "Width" (link table-width-menu))
   (-> "Height" (link table-height-menu))
   (-> "Border" (link table-border-menu))
   (-> "Padding" (link table-padding-menu))
   (-> "Horizontal alignment" (link table-halign-menu))
   (-> "Vertical alignment" (link table-valign-menu))
-  ---
-  (-> "Special properties" (link table-special-menu)))
+  ;;---
+  (-> "Special" (link table-special-menu)))
 
 (menu-bind cell-menu
   (-> "Operation mode" (link cell-mode-menu))
-  ---
+  ;;---
   (-> "Width" (link cell-width-menu))
   (-> "Height" (link cell-height-menu))
   (-> "Border" (link cell-border-menu))
@@ -244,8 +236,8 @@
   (-> "Horizontal alignment" (link cell-halign-menu))
   (-> "Vertical alignment" (link cell-valign-menu))
   (-> "Background color" (link cell-color-menu))
-  ---
-  (-> "Special properties" (link cell-special-menu)))
+  ;;---
+  (-> "Special" (link cell-special-menu)))
 
 (menu-bind vertical-table-cell-menu
   (=> "Table" (link table-menu))
@@ -261,227 +253,248 @@
   (if (== (get-cell-mode) "column") (-> "Column" (link cell-menu)))
   (if (== (get-cell-mode) "table") (-> "Cells" (link cell-menu))))
 
+(tm-menu (standard-focus-menu t)
+  (:require (table-markup-context? t))
+  (dynamic (focus-tag-menu t))
+  (-> "Move" (dynamic (focus-move-menu t)))
+  (-> "Resize" (dynamic (focus-insert-menu t)))
+  ---
+  (group "Table")
+  (link table-menu)
+  ---
+  (if (== (get-cell-mode) "cell") (group "Cell"))
+  (if (== (get-cell-mode) "row") (group "Row"))
+  (if (== (get-cell-mode) "column") (group "Column"))
+  (if (== (get-cell-mode) "table") (group "Cells"))
+  (link cell-menu)
+  (dynamic (focus-extra-menu t))
+  (dynamic (focus-hidden-menu t)))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Icons for manipulation of tables
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (menu-bind table-insert-icons
   ((balloon (icon "tm_table_arow.xpm")
-	    "Insert row above (A-up)")
+	    "Insert row above")
    (table-insert-row #f))
   ((balloon (icon "tm_table_brow.xpm")
-	    "Insert row below (A-down)")
+	    "Insert row below")
    (table-insert-row #t))
   ((balloon (icon "tm_table_lcol.xpm")
-	    "Insert column to the left (A-left)")
+	    "Insert column to the left")
    (table-insert-column #f))
   ((balloon (icon "tm_table_rcol.xpm")
-	    "Insert column to the right (A-right)")
+	    "Insert column to the right")
    (table-insert-column #t))
   ---
   ((balloon (icon "tm_table_drow.xpm")
-	    "Delete row (M-t h delete)")
+	    "Delete row")
    (table-remove-row #t))
   ((balloon (icon "tm_table_dcol.xpm")
-	    "Delete column (M-t v delete)")
+	    "Delete column")
    (table-remove-column #t)))
 
 (menu-bind table-hpos-icons
   ((balloon (icon "tm_table_left.xpm")
-	    "Align table at the left (M-t H l)")
+	    "Align table at the left")
    (table-set-halign "l"))
   ((balloon (icon "tm_table_hcenter.xpm")
-	    "Align table at the horizontal center (M-t H c)")
+	    "Align table at the horizontal center")
    (table-set-halign "c"))
   ((balloon (icon "tm_table_right.xpm")
-	    "Align table to the right (M-t H r)")
+	    "Align table to the right")
    (table-set-halign "r"))
   ((icon "tm_empty.xpm") (noop))
   ((balloon (icon "tm_table_bleft.xpm")
-	    "Align table at base of left column (M-t H L)")
+	    "Align table at base of left column")
    (table-set-halign "L"))
   ((balloon (icon "tm_table_bhcenter.xpm")
-	    "Align table at base of middle column (M-t H C)")
+	    "Align table at base of middle column")
    (table-set-halign "C"))
   ((balloon (icon "tm_table_bright.xpm")
-	    "Align table at base of right column (M-t H R)")
+	    "Align table at base of right column")
    (table-set-halign "R"))
   ((balloon (icon "tm_table_hbase.xpm")
-	    "Align table at base of origin column (M-t H O)")
+	    "Align table at base of origin column")
    (table-set-halign "O")))
 
 (menu-bind table-vpos-icons
   ((balloon (icon "tm_table_bottom.xpm")
-	    "Align table at the bottom (M-t V b)")
+	    "Align table at the bottom")
    (table-set-valign "b"))
   ((balloon (icon "tm_table_vcenter.xpm")
-	    "Align table at the center (M-t V c)")
+	    "Align table at the center")
    (table-set-valign "c"))
   ((balloon (icon "tm_table_top.xpm")
-	    "Align table at the top (M-t V t)")
+	    "Align table at the top")
    (table-set-valign "t"))
   ((balloon (icon "tm_table_vfrac.xpm")
-	    "Align table at axis (M-t V f)")
+	    "Align table at axis")
    (table-set-valign "f"))
   ((balloon (icon "tm_table_bbottom.xpm")
-	    "Align table at the base of bottom row (M-t V B)")
+	    "Align table at the base of bottom row")
    (table-set-valign "B"))
   ((balloon (icon "tm_table_bvcenter.xpm")
-	    "Align table at the base of middle row (M-t V C)")
+	    "Align table at the base of middle row")
    (table-set-valign "C"))
   ((balloon (icon "tm_table_btop.xpm")
-	    "Align table at the base of top row (M-t V T)")
+	    "Align table at the base of top row")
    (table-set-valign "T"))
   ((balloon (icon "tm_table_vbase.xpm")
-	    "Align table at the base of origin row (M-t V O)")
+	    "Align table at the base of origin row")
    (table-set-valign "O")))
 
 (menu-bind cell-mode-icons
   ((balloon (icon "tm_cell_by_cell.xpm")
-	    "Perform operations on cells (M-t m c)")
+	    "Perform operations on cells")
    (set-cell-mode "cell"))
   ((balloon (icon "tm_cell_by_row.xpm")
-	    "Perform operations on rows (M-t m h)")
+	    "Perform operations on rows")
    (set-cell-mode "row"))
   ((balloon (icon "tm_cell_by_column.xpm")
-	    "Perform operations on columns (M-t m v)")
+	    "Perform operations on columns")
    (set-cell-mode "column"))
   ((balloon (icon "tm_cell_by_table.xpm")
-	    "Perform operations on entire table (M-t m t)")
+	    "Perform operations on entire table")
    (set-cell-mode "table")))
 
 (menu-bind cell-pos-icons
   ((balloon (icon "tm_cell_left.xpm")
-	    "Align cell to the left (M-t h l)")
+	    "Align cell to the left")
    (cell-set-halign "l"))
   ((balloon (icon "tm_cell_hcenter.xpm")
-	    "Horizontally center the cell (M-t h c)")
+	    "Horizontally center the cell")
    (cell-set-halign "c"))
   ((balloon (icon "tm_cell_right.xpm")
-	    "Align cell to the right (M-t h r)")
+	    "Align cell to the right")
    (cell-set-halign "r"))
   ((balloon (icon "tm_cell_hbase.xpm")
-	    "Horizontally align cell to the base (M-t h R)")
+	    "Horizontally align cell to the base")
    (cell-set-halign "R"))
   ((balloon (icon "tm_cell_hdot.xpm")
-	    "Align cell to the decimal dot (M-t h .)")
+	    "Align cell to the decimal dot")
    (cell-set-halign "L."))
   ((balloon (icon "tm_cell_hcomma.xpm")
-	    "Align cell to the decimal comma (M-t h ,)")
+	    "Align cell to the decimal comma")
    (cell-set-halign "L,"))
   ((balloon (icon "tm_cell_bottom.xpm")
-	    "Align cell to the bottom (M-t v b)")
+	    "Align cell to the bottom")
    (cell-set-valign "b"))
   ((balloon (icon "tm_cell_vcenter.xpm")
-	    "Vertically center the cell (M-t v c)")
+	    "Vertically center the cell")
    (cell-set-valign "c"))
   ((balloon (icon "tm_cell_top.xpm")
-	    "Align cell to the top (M-t v t)")
+	    "Align cell to the top")
    (cell-set-valign "t"))
   ((balloon (icon "tm_cell_vbase.xpm")
-	    "Vertically align cell to the base (M-t v B)")
+	    "Vertically align cell to the base")
    (cell-set-valign "B")))
 
 (menu-bind cell-size-icons
   ((balloon (icon "tm_cell_width.xpm")
-	    "Set width of cell (M-t h s)")
+	    "Set width of cell")
    (cell-interactive-set "cell-width"))
   ((balloon (icon "tm_cell_hmin.xpm")
-	    "Width is minimum of specified width and box width (M-t h m)")
+	    "Width is minimum of specified width and box width")
    (cell-set-hmode "min"))
   ((balloon (icon "tm_cell_hexact.xpm")
-	    "Width is exactly the specified width (M-t h e)")
+	    "Width is exactly the specified width")
    (cell-set-hmode "exact"))
   ((balloon (icon "tm_cell_hmax.xpm")
-	    "Width is maximum of specified width and box width (M-t h M)")
+	    "Width is maximum of specified width and box width")
    (cell-set-hmode "max"))
   ((balloon (icon "tm_cell_height.xpm")
-	    "Set height of cell (M-t v s)")
+	    "Set height of cell")
    (cell-interactive-set "cell-height"))
   ((balloon (icon "tm_cell_vmin.xpm")
-	    "Height is minimum of specified height and box height (M-t v m)")
+	    "Height is minimum of specified height and box height")
    (cell-set-vmode "min"))
   ((balloon (icon "tm_cell_vexact.xpm")
-	    "Height is exactly the specified height (M-t v e)")
+	    "Height is exactly the specified height")
    (cell-set-vmode "exact"))
   ((balloon (icon "tm_cell_vmax.xpm")
-	    "Height is maximum of specified height and box height (M-t v M)")
+	    "Height is maximum of specified height and box height")
    (cell-set-vmode "max")))
 
 (menu-bind cell-decoration-icons
   ((balloon (icon "tm_decorate_left.xpm")
-	    "Use left hand column as border (M-t b left)")
+	    "Use left hand column as border")
    (table-column-decoration #f))
   ((balloon (icon "tm_decorate_right.xpm")
-	    "Use right hand column as border (M-t b right)")
+	    "Use right hand column as border")
    (table-column-decoration #t))
   ((balloon (icon "tm_decorate_up.xpm")
-	    "Use row above as border (M-t b up)")
+	    "Use row above as border")
    (table-row-decoration #f))
   ((balloon (icon "tm_decorate_down.xpm")
-	    "Use row below as border (M-t b down)")
+	    "Use row below as border")
    (table-row-decoration #t)))
 
-(menu-bind table-icons
-  |
-  (=> (balloon (icon "tm_table_insert.xpm")
-	       "Insert or delete rows or columns")
-      (link table-insert-icons))
-  (=> (balloon (icon "tm_table_size.xpm") "Specify the size of the table")
-      (group "Width")
-      (link table-width-menu)
-      ---
-      (group "Height")
-      (link table-height-menu))
-  (=> (balloon (icon "tm_table_border.xpm") "Change the border of the table")
-      (group "Border")
-      (link table-border-menu)
-      ---
-      (group "Padding")
-      (link table-padding-menu))
-  (=> (balloon (icon "tm_table_pos.xpm") "Position the table")
-      (tile 4 (link table-hpos-icons))
-      ---
-      (tile 4 (link table-vpos-icons)))
-  (=> (balloon (icon "tm_table_special.xpm") "Set special table properties")
-      (link table-special-menu))
-  |
-  (if (== (get-cell-mode) "cell")
-      (=> (balloon (icon "tm_cell_cell.xpm")
-		   "Change cell operation mode")
-	  (link cell-mode-icons)))
-  (if (== (get-cell-mode) "row")
-      (=> (balloon (icon "tm_cell_row.xpm")
-		   "Change cell operation mode")
-	  (link cell-mode-icons)))
-  (if (== (get-cell-mode) "column")
-      (=> (balloon (icon "tm_cell_column.xpm")
-		   "Change cell operation mode")
-	  (link cell-mode-icons)))
-  (if (== (get-cell-mode) "table")
-      (=> (balloon (icon "tm_cell_table.xpm")
-		   "Change cell operation mode")
-	  (link cell-mode-icons)))
-  (=> (balloon (icon "tm_cell_size.xpm") "Modify cell size")
-      (group "Width")
-      (link cell-width-menu)
-      ---
-      (group "Height")
-      (link cell-height-menu))
-  (=> (balloon (icon "tm_cell_border.xpm") "Change border of cell")
-      (group "Border")
-      (link cell-border-menu)
-      ---
-      (group "Padding")
-      (link cell-padding-menu))
-  (=> (balloon (icon "tm_cell_pos.xpm") "Modify cell alignment")
-      (group "Horizontal alignment")
-      (link cell-halign-menu)
-      ---
-      (group "Vertical alignment")
-      (link cell-valign-menu))
-  (=> (balloon (icon "tm_cell_background.xpm")
-	       "Set background color of cell")
-      (link cell-color-menu))
-  (=> (balloon (icon "tm_cell_special.xpm") "Set special cell properties")
-      (link  cell-special-menu)))
+(define (document-like? t)
+  (or (tree-is? t 'document)
+      (and (tree-is? t 'with)
+           (document-like? (tree-up t)))))
+
+(tm-menu (focus-tag-extra-icons t)
+  (:require (table-markup-context? t))
+  (if (document-like? (tree-up t))
+      ((check (balloon (icon "tm_table_parwidth.xpm")
+                       "Extend table to full paragraph width")
+              "v" (table-test-parwidth?))
+       (table-toggle-parwidth)))
+  (=> (balloon (icon "tm_set_properties.xpm") "Table properties")
+      (mini #f (link table-menu))))
+
+(tm-menu (focus-extra-icons t)
+  (:require (table-markup-context? t))
+  (glue #f #f 10 0)
+  (minibar
+    (if (== (get-cell-mode) "cell")
+        (=> (balloon "Cell" "Change cell operation mode")
+            (mini #f
+              (link cell-mode-icons))))
+    (if (== (get-cell-mode) "row")
+        (=> (balloon "Row" "Change cell operation mode")
+            (mini #f
+              (link cell-mode-icons))))
+    (if (== (get-cell-mode) "column")
+        (=> (balloon "Column" "Change cell operation mode")
+            (mini #f
+              (link cell-mode-icons))))
+    (if (== (get-cell-mode) "table")
+        (=> (balloon "All cells" "Change cell operation mode")
+            (mini #f
+              (link cell-mode-icons))))
+    (=> (balloon (icon "tm_cell_size.xpm") "Modify cell size")
+        (mini #f
+          (group "Width")
+          (link cell-width-menu)
+          ---
+          (group "Height")
+          (link cell-height-menu)))
+    (=> (balloon (icon "tm_cell_border.xpm") "Change border of cell")
+        (mini #f
+          (group "Border")
+          (link cell-border-menu)
+          ---
+          (group "Padding")
+          (link cell-padding-menu)))
+    (=> (balloon (icon "tm_cell_pos.xpm") "Modify cell alignment")
+        (mini #f
+          (group "Horizontal alignment")
+          (link cell-halign-menu)
+          ---
+          (group "Vertical alignment")
+          (link cell-valign-menu)))
+    (=> (balloon (icon "tm_cell_background.xpm")
+                 "Set background color of cell")
+        (mini #f
+          (link cell-color-menu)))
+    ((check (balloon (icon "tm_cell_wrap.xpm") "Line wrapping inside cell")
+            "v" (cell-test-wrap?))
+     (cell-toggle-wrap))
+    (=> (balloon (icon "tm_cell_special.xpm")
+                 "Special cell properties and actions")
+        (mini #f
+          (link  cell-special-menu)))))
