@@ -5,10 +5,9 @@
 ;; DESCRIPTION : file handling
 ;; COPYRIGHT   : (C) 2001  Joris van der Hoeven
 ;;
-;; This software falls under the GNU general public license and comes WITHOUT
-;; ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for details.
-;; If you don't have this file, write to the Free Software Foundation, Inc.,
-;; 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+;; This software falls under the GNU general public license version 3 or later.
+;; It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
+;; in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ;;
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
@@ -54,7 +53,7 @@
 (tm-define (save-buffer . l)
   (if (and (pair? l) (url? (car l))) (set! current-save-target (car l)))
   (cond ((= (length l) 0) (save-buffer (get-name-buffer)))
-	((url-scratch? (car l)) (interactive save-buffer))
+	((url-scratch? (car l)) (choose-file save-buffer "Save TeXmacs file" "texmacs"))
 	((= (length l) 1) (texmacs-save-buffer (car l) "generic"))
 	(else (secure-save-buffer (car l) (cadr l)))))
 
@@ -70,17 +69,30 @@
 ;; Loading
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
+(define (more-recent file suffix1 suffix2)
+  (and (url-exists? (url-glue file suffix1))
+       (url-exists? (url-glue file suffix2))
+       (url-newer? (url-glue file suffix1) (url-glue file suffix2))))
+
+(define (most-recent-suffix file)
+  (if (more-recent file "~" "")
+      (if (not (more-recent file "#" "")) "~"
+          (if (more-recent file "#" "~") "#" "~"))
+      (if (more-recent file "#" "") "#" "")))
+
 (define (load-buffer-sub file fm where)
-  (dialogue
-    (if (and (!= fm "help")
-	     (not (url-rooted-web? file))
-	     (url-exists? file)
-	     (url-exists? (url-glue file "~"))
-	     (url-newer? (url-glue file "~") file)
-	     (dialogue-confirm? "Load more recent autosave file?" #t))
-	(texmacs-load-buffer (url-glue file "~") fm where #t)
-	(texmacs-load-buffer file fm where #f))
-    (activate-highlighting)))
+  (let* ((suffix (most-recent-suffix file))
+         (question (if (== suffix "#")
+                       "Rescue file from crash?"
+                       "Load more recent autosave file?")))
+    (dialogue
+      (if (and (!= fm "help")
+               (not (url-rooted-web? file))
+               (!= suffix "")
+               (dialogue-confirm? question #t))
+          (texmacs-load-buffer (url-glue file suffix) fm where #t)
+          (texmacs-load-buffer file fm where #f))
+      (activate-highlighting))))
 
 (tm-define (load-buffer . l)
   (with file (url-append "$TEXMACS_FILE_PATH" (car l))
@@ -138,3 +150,22 @@
 (tm-define (load-in-new-window s) (load-buffer s 1))
 (tm-define (load-browse-buffer s)
   (if (help-buffer?) (load-buffer s "help") (load-buffer s)))
+
+(tm-define (open-buffer)
+  (:synopsis "Open a new file")
+  (choose-file load-buffer "Load file" ""))
+
+(tm-define (print-buffer)
+  (:synopsis "Print the current buffer")
+  (print))
+
+(tm-define (interactive-page-setup)
+  (:synopsis "Specify the page setup")
+  (:interactive #t)
+  (set-message "Not yet implemented" "Printer setup"))
+
+(tm-define (interactive-print-buffer)
+  (:synopsis "Print the current buffer")
+  (:interactive #t)
+  (print-to-file "$TEXMACS_HOME_PATH/system/tmp/tmpprint.ps")
+  (interactive-print '() "$TEXMACS_HOME_PATH/system/tmp/tmpprint.ps"))
