@@ -18,11 +18,11 @@
 ;; Preferences
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-(define (get-default-look-and-feel)
-  (if (os-win32?) "windows" "emacs"))
-
 (define (get-default-interactive-questions)
-  (if (== (get-preference "look and feel") "windows") "popups" "footer"))
+  (if (or (like-gnome?) (like-macos?) (like-windows?)) "popups" "footer"))
+
+(define (get-default-show-table-cells)
+  (if (qt-gui?) "on" "off"))
 
 (define (notify-look-and-feel var val)
   (set-message "Restart in order to let the new look and feel take effect"
@@ -49,17 +49,45 @@
 (define (notify-bibtex-command var val)
   (set-bibtex-command val))
 
+(define (notify-tool var val)
+  ;; FIXME: the menus sometimes don't get updated,
+  ;; but the fix below does not work
+  (if (has-view?) (notify-change 1)))
+
+(define (notify-fast-environments var val)
+  (set-fast-environments (== val "on")))
+
 (define-preferences
   ("profile" "beginner" (lambda args (noop)))
-  ("look and feel" (get-default-look-and-feel) notify-look-and-feel)
+  ("look and feel" "default" notify-look-and-feel)
   ("detailed menus" "detailed" noop)
   ("interactive questions" (get-default-interactive-questions) noop)
   ("language" (get-locale-language) notify-language)
+  ("fast environments" "on" notify-fast-environments)
+  ("show full context" "on" (lambda args (noop)))
+  ("show table cells" (get-default-show-table-cells) (lambda args (noop)))
+  ("show focus" "on" (lambda args (noop)))
+  ("show only semantic focus" "on" (lambda args (noop)))
+  ("semantic editing" "off" (lambda args (noop)))
+  ("semantic selections" "on" (lambda args (noop)))
+  ("remove superfluous invisible" "off" (lambda args (noop)))
+  ("insert missing invisible" "off" (lambda args (noop)))
+  ("zealous invisible correct" "off" (lambda args (noop)))
+  ("homoglyph correct" "off" (lambda args (noop)))
+  ("manual remove superfluous invisible" "on" (lambda args (noop)))
+  ("manual insert missing invisible" "on" (lambda args (noop)))
+  ("manual zealous invisible correct" "off" (lambda args (noop)))
+  ("manual homoglyph correct" "on" (lambda args (noop)))
   ("security" "prompt on scripts" notify-security)
   ("bibtex command" "bibtex" notify-bibtex-command)
   ("scripting language" "none" notify-scripting-language)
-  ("remote connections" "off" noop)
-  ("linking tool" "off" noop))
+  ("debugging tool" "off" notify-tool)
+  ("linking tool" "off" notify-tool)
+  ("presentation tool" "off" notify-tool)
+  ("source tool" "off" notify-tool)
+  ("versioning tool" "off" notify-tool)
+  ("remote connections" "off" notify-tool)
+  ("experimental alpha" "off" notify-tool))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Properties of some built-in routines
