@@ -22,14 +22,22 @@
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (define (maxima-prompt? t)
-  (match? t '(with "mode" "text" "font-family" "tt" "color" "red" :*)))
+  (or (match? t '(text (with "font-family" "tt" "color" "red" :*)))
+      (match? t '(with "font-family" "tt" "color" "red" :*))
+      (match? t '(with "mode" "text" "font-family" "tt" "color" "red" :*))))
 
 (define (maxima-output-simplify t)
   ;;(display* "Simplify " t "\n")
   (cond ((and (func? t 'concat) (> (length t) 2) (maxima-prompt? (cadr t)))
 	 (plugin-output-std-simplify "maxima" (cons 'concat (cddr t))))
+	((match? t '(with "math-display" "true" :%1))
+	 (maxima-output-simplify (cAr t)))
 	((match? t '(with "mode" "math" "math-display" "true" :%1))
 	 `(math ,(maxima-output-simplify (cAr t))))
+	((func? t 'text 1)
+	 `(text ,(maxima-output-simplify (cAr t))))
+	((func? t 'math 1)
+	 `(math ,(maxima-output-simplify (cAr t))))
 	((func? t 'with 1)
 	 (maxima-output-simplify (cAr t)))
 	((func? t 'with)
@@ -44,6 +52,8 @@
 	 (list-or (map maxima-contains-prompt? (cdr t))))
 	((and (func? t 'with) (nnull? (cdr t)))
 	 (maxima-contains-prompt? (cAr t)))
+	((or (func? t 'text 1) (func? t 'math 1))
+	 (maxima-contains-prompt? (cAr t)))
 	(else #f)))
 
 (tm-define (plugin-output-simplify name t)
@@ -72,7 +82,8 @@
       ("Simplify" (maxima-apply "fullratsimp"))
       ("Factor" (maxima-apply "factor"))
       ("Expand" (maxima-apply "expand"))
-      ("Expand#w.r.t." (maxima-apply "expandwrt" 2)))
+      ((eval '(concat "Expand " "w.r.t."))
+       (maxima-apply "expandwrt" 2)))
   (-> "Solving equations"
       ("Solve" (maxima-apply "solve"))
       ("Solve in" (maxima-apply "solve" 2)))
@@ -96,7 +107,8 @@
       ("Arc tangent" (maxima-apply "atan"))
       ---
       ("Reduce trigonometric functions" (maxima-apply "trigreduce"))
-      ("Reduce trigonometric functions#w.r.t." (maxima-apply "trigreduce" 2))
+      ((eval '(concat "Reduce trigonometric functions " "w.r.t."))
+       (maxima-apply "trigreduce" 2))
       ("Expand trigonometric functions" (maxima-apply "trigexpand")))
   (-> "Special functions"
       ("Airy" (maxima-apply "Airy"))
