@@ -41,6 +41,14 @@ check_tree (tree t) {
 }
 */
 
+static bool
+is_var_compound (tree t, string s, int n) {
+  return
+    is_compound (t, s, n) ||
+    (is_func (t, APPLY, n+1) && t[0] == s) ||
+    (is_func (t, EXPAND, n+1) && t[0] == s);
+}
+
 /******************************************************************************
 * Preprocess preamble
 ******************************************************************************/
@@ -50,33 +58,68 @@ filter_preamble (tree t) {
   int i, n=N(t);
   bool in_preamble= true;
   tree r (CONCAT);
+  tree doc (CONCAT);
   tree preamble (CONCAT);
   tree title_info (CONCAT);
 
   for (i=0; i<n; i++) {
     tree u= t[i];
-    if (in_preamble) {
+    if (is_tuple (u, "\\title") ||
+        is_tuple (u, "\\author") ||
+        is_tuple (u, "\\address"))
+      title_info << u;
+    else if (is_tuple (u, "\\affiliation")) {
+      tree v= copy (u);
+      v[0]= "\\address";
+      title_info << v;
+    }
+    else if (is_tuple (u, "\\thanks")) {
+      tree v= copy (u);
+      v[0]= "\\title-thanks";
+      title_info << v;
+    }
+    else if (is_tuple (u, "\\email")) {
+      tree v= copy (u);
+      v[0]= "\\title-email";
+      title_info << v;
+    }
+    else if (is_tuple (u, "\\urladdr"))
+      title_info << u;
+    else if (is_tuple (u, "\\keywords"))
+      title_info << u;
+    else if (is_tuple (u, "\\classification"))
+      title_info << u;
+    else if (is_tuple (u, "\\subjclass"))
+      title_info << u;
+    else if (is_tuple (u, "\\subjclass*"))
+      title_info << u;
+    else if (in_preamble) {
       if (u == tuple ("\\begin-document")) {
 	r << u;
 	if (N(preamble) > 0)
 	  r << tuple ("\\begin-hide-preamble") << A(preamble)
 	    << tuple ("\\end-hide-preamble");
-	r << A(title_info);
 	in_preamble= false;
       }
       else if (is_tuple (u, "\\documentclass") ||
 	       is_tuple (u, "\\documentclass*") ||
 	       is_tuple (u, "\\documentstyle") ||
 	       is_tuple (u, "\\documentstyle*"))
-	r << u;
-      else if (is_tuple (u, "\\def") || is_tuple (u, "\\def*"))
+	doc << u;
+      else if (is_tuple (u, "\\def") ||
+	       is_tuple (u, "\\def*"))
+	preamble << u << "\n" << "\n";
+      else if (is_tuple (u, "\\newtheorem") ||
+	       is_tuple (u, "\\newtheorem*"))
+	preamble << u << "\n" << "\n";
+      else if (is_tuple (u, "\\newenvironment") ||
+	       is_tuple (u, "\\newenvironment*"))
 	preamble << u << "\n" << "\n";
-      else if (is_tuple (u, "\\title") || is_tuple (u, "\\author") ||
-	       is_tuple (u, "\\address"))
-	title_info << u;
     }
-    else r << u;
+    else doc << u;
   }
+  r << A(title_info);
+  r << A(doc);
   if (in_preamble) return t;
   return r;
 }
@@ -129,6 +172,7 @@ latex_symbol_to_tree (string s) {
       if (s == "\\noindent")  return tree (FORMAT, "no first indentation");
       if (s == "\\linebreak")  return tree (FORMAT, "line break");
       if (s == "\\newline")  return tree (FORMAT, "new line");
+      if (s == "\\nobreak")  return tree (FORMAT, "no line break");
       if (s == "\\nolinebreak")  return tree (FORMAT, "no line break");
       if (s == "\\pagebreak")  return tree (FORMAT, "page break");
       if (s == "\\nopagebreak")  return tree (FORMAT, "no page break after");
@@ -151,6 +195,7 @@ latex_symbol_to_tree (string s) {
       if (s == "\\hfill")  return tree (HTAB, "1fn");
       if (s == "\\hline")  return tree (APPLY, "hline");
       if (s == "\\appendix") { textm_appendices= true; return ""; }
+      if (s == "\\limits") return ""; // tree (FORMAT, "with limits");
       if (s == "\\nolimits") return ""; // temporarily
       if (s == "\\vert") return "|";
       if (s == "\\Vert") return "<||>";
@@ -161,6 +206,22 @@ latex_symbol_to_tree (string s) {
       if (s == "\\rang") return "<rangle>";
       if (s == "\\tableofcontents")
 	return compound ("table-of-contents", "toc", tree (DOCUMENT, ""));
+      if (s == "\\bgroup") return "";
+      if (s == "\\egroup") return "";
+      if (s == "\\colon") return ":";
+      if (s == "\\dotsc") return "<ldots>";
+      if (s == "\\dotsb") return "<cdots>";
+      if (s == "\\dotsm") return "<cdots>";
+      if (s == "\\dotsi") return "<cdots>";
+      if (s == "\\dotso") return "<ldots>";
+      if (s == "\\lvert") return "|";
+      if (s == "\\rvert") return "|";
+      if (s == "\\lVert") return "<||>";
+      if (s == "\\rVert") return "<||>";
+      if (s == "\\qed") return compound ("math", "<Box>");
+      if (s == "\\implies") return "<Longrightarrow>";
+      if (s == "\\iff") return "<Longleftrightarrow>";
+      if (s == "\\gets") return "<leftarrow>";
     }
 
     if (latex_type (s) == "texmacs") {
@@ -238,10 +299,10 @@ latex_symbol_to_tree (string s) {
       return s(1,N(s));
     if (latex_type (s) == "control") return s(1,N(s));
     if ((s == "\\ldots") && (command_type ("!mode") != "math")) return "...";
+    if (s == "\\bignone") return tree (BIG, ".");
     if (latex_type (s) == "symbol")  return "<" * s(1,N(s)) * ">";
     if (latex_type (s) == "big-symbol") {
-      if (s == "\\bignone") return tree (BIG, ".");
-      else if (s(0,4)=="\\big") return tree (BIG, s(4,N(s)));
+      if (s(0,4)=="\\big") return tree (BIG, s(4,N(s)));
       else return tree (BIG, s(1,N(s)));
     }
 
@@ -298,7 +359,7 @@ string_arg (tree t) {
   else if (is_func (t, APPLY, 1) && t[0] == "nbsp")
     return " ";
   else {
-    cout << "t= " << t << "\n";
+    //cout << "t= " << t << "\n";
     return "";
   }
 }
@@ -349,12 +410,22 @@ latex_concat_to_tree (tree t, bool& new_flag) {
 	    continue;
 	}
 	else {
-	  if ((t[i] != tree (TUPLE, "\\ ")) && (i>0) && (is_tuple (t[i-1]))) {
-	    string s= t[i-1][0]->label;
-	    if ((s[0] == '\\') && (latex_type (s) == "command") &&
-		(s!="\\end-math") && (s!="\\end-displaymath"))
-	      if ((arity(t[i-1])==1) || (s=="\\label"))
+	  if ((t[i] != tree (TUPLE, "\\ "))) {
+	    if (i>0 && is_tuple (t[i-1])) {
+	      string s= t[i-1][0]->label;
+	      if ((s[0] == '\\') && (latex_type (s) == "command") &&
+		  (s!="\\end-math") && (s!="\\end-displaymath"))
+		if ((arity(t[i-1])==1) || (s=="\\label")) continue;
+	      if (starts (s, "\\begin-") &&
+		  (command_type["!verbatim"] != "true"))
 		continue;
+	    }
+	    if (i+1<N(t) && is_tuple (t[i+1])) {
+	      string s= t[i+1][0]->label;
+	      if (starts (s, "\\end-") &&
+		  (command_type["!verbatim"] != "true"))
+		continue;
+	    }
 	  }
 	}
       }
@@ -488,6 +559,31 @@ latex_accent (tree t, string acc) {
 	       tree (WIDE, tree (WITH, MODE, "text", l2e (t)), acc));
 }
 
+tree
+latex_eps_get (tree t, string var) {
+  if (!is_atomic (t)) return "";
+  string s= t->label;
+  int start=0, i, n=N(s);
+  for (i=0; i <= n; i++)
+    if (i == n || s[i] == ',') {
+      string ss= s (start, i);
+      while (starts (ss, " ")) ss= ss (1, N(ss));
+      while (ends (ss, " ")) ss= ss (0, N(ss) - 1);
+      int j, k= N(ss);
+      for (j=0; j<k; j++)
+	if (ss[j] == '=') break;
+      string v= ss (0, j);
+      while (ends (v, " ")) v= v (0, N(v) - 1);
+      if (j < k && v == var) {
+	string val= ss (j+1, N(ss));
+	while (starts (val, " ")) val= val (1, N(val));
+	return val;
+      }
+      start= i+1;
+    }
+  return "";
+}
+
 tree
 latex_command_to_tree (tree t) {
   if (is_tuple (t, "\\def", 2)) {
@@ -505,6 +601,12 @@ latex_command_to_tree (tree t) {
     return tree (ASSIGN, var, f);
   }
 
+  if (is_tuple (t, "\\newtheorem", 2) || is_tuple (t, "\\newtheorem*", 2)) {
+    string var= l2e(t[1])->label;
+    string val= l2e(t[2])->label;
+    return compound ("new-theorem", var, val);
+  }
+
   if (is_tuple (t, "\\newenvironment", 3)) {
     string var= l2e(t[1])->label;
     return tree (ASSIGN, var, tree (ENV, l2e (t[2]), l2e (t[3])));
@@ -519,11 +621,6 @@ latex_command_to_tree (tree t) {
     return tree (ASSIGN, var, e);
   }
 
-  if (is_tuple (t, "\\setcounter", 2)) {
-    tree u= l2e (t[1]);
-    if (is_compound (u)) return "";
-    else return tree (ASSIGN, u->label * "nr", l2e (t[2]));
-  }
   if (is_tuple (t, "\\arabic", 1)) {
     tree u= l2e (t[1]);
     if (is_compound (u)) return "";
@@ -557,6 +654,10 @@ latex_command_to_tree (tree t) {
   if (is_tuple (t, "\\textit", 1)) return m2e (t, FONT_SHAPE, "italic");
   if (is_tuple (t, "\\textsl", 1)) return m2e (t, FONT_SHAPE, "slanted");
   if (is_tuple (t, "\\textsc", 1)) return m2e (t, FONT_SHAPE, "small-caps");
+  if (is_tuple (t, "\\tmrsub", 1)) return tree (RSUB, l2e (t[1]));
+  if (is_tuple (t, "\\tmrsup", 1)) return tree (RSUP, l2e (t[1]));
+  if (is_tuple (t, "\\textsubscript", 1)) return tree (RSUB, l2e (t[1]));
+  if (is_tuple (t, "\\textsuperscript", 1)) return tree (RSUP, l2e (t[1]));
   if (is_tuple (t, "\\tmtextrm", 1)) return m2e (t, FONT_FAMILY, "rm");
   if (is_tuple (t, "\\tmtexttt", 1)) return m2e (t, FONT_FAMILY, "tt");
   if (is_tuple (t, "\\tmtextsf", 1)) return m2e (t, FONT_FAMILY, "ss");
@@ -583,6 +684,7 @@ latex_command_to_tree (tree t) {
   if (is_tuple (t, "\\mathfrak", 1)) return m2e (t, MATH_FONT, "Euler");
   if (is_tuple (t, "\\mathbb", 1)) return m2e (t, MATH_FONT, "Bbb");
   if (is_tuple (t, "\\mathbbm", 1)) return m2e (t, MATH_FONT, "Bbb*");
+  if (is_tuple (t, "\\mathscr", 1)) return m2e (t, MATH_FONT, "cal*");
 
   if (is_tuple (t, "\\prime", 1)) return tree (RPRIME, string_arg (t[1]));
   if (is_tuple (t, "\\frac", 2)) return tree (FRAC, l2e (t[1]), l2e (t[2]));
@@ -593,6 +695,10 @@ latex_command_to_tree (tree t) {
   if (is_tuple (t, "\\<sub>", 1)) return tree (RSUB, l2e (t[1]));
   if (is_tuple (t, "\\not", 1)) return tree (NEG, l2e (t[1]));
   if (is_tuple (t, "\\bar", 1)) return tree (WIDE, l2e (t[1]), "<bar>");
+  if (is_tuple (t, "\\overline", 1))
+    return tree (WIDE, l2e (t[1]), "<bar>");
+  if (is_tuple (t, "\\underline", 1))
+    return tree (VAR_WIDE, l2e (t[1]), "<bar>");
   if (is_tuple (t, "\\hat", 1)) return tree (WIDE, l2e (t[1]), "^");
   if (is_tuple (t, "\\tilde", 1)) return tree (WIDE, l2e (t[1]), "~");
   if (is_tuple (t, "\\widehat", 1)) return tree (WIDE, l2e (t[1]), "^");
@@ -606,17 +712,32 @@ latex_command_to_tree (tree t) {
   if (is_tuple (t, "\\acute", 1)) return tree (WIDE, l2e (t[1]), "<acute>");
   if (is_tuple (t, "\\vec", 1)) return tree (WIDE, l2e (t[1]), "<vect>");
   if (is_tuple (t, "\\breve", 1)) return tree (WIDE, l2e (t[1]), "<breve>");
-  if (is_tuple (t, "\\abovering", 1))
+  if (is_tuple (t, "\\abovering", 1) || is_tuple (t, "\\mathring", 1))
     return tree (WIDE, l2e (t[1]), "<abovering>");
   if (is_tuple (t, "\\hspace", 1) || is_tuple (t, "\\hspace*", 1)) {
     tree r= t2e (t[1]);
-    if (is_compound (r, "fill", 0)) return tree (HTAB, "1fn");
+    if (is_var_compound (r, "fill", 0)) return tree (HTAB, "1fn");
     return tree (SPACE, r);
   }
   if (is_tuple (t, "\\vspace", 1) || is_tuple (t, "\\vspace*", 1))
     return tree (VSPACE, t2e (t[1]));
   if (is_tuple (t, "\\label", 1)) return tree (LABEL, t2e (t[1]));
   if (is_tuple (t, "\\ref", 1)) return tree (REFERENCE, t2e (t[1]));
+  if (is_tuple (t, "\\newcounter", 1))
+    return compound ("new-counter", v2e (t[1]));
+  if (is_tuple (t, "\\value", 1))
+    return compound ("value-counter", v2e (t[1]));
+  if (is_tuple (t, "\\stepcounter", 1))
+    return compound ("inc-counter", v2e (t[1]));
+  if (is_tuple (t, "\\refstepcounter", 1))
+    return compound ("next-counter", v2e (t[1]));
+  if (is_tuple (t, "\\setcounter", 2)) // FIXME: only reset works
+    return compound ("reset-counter", v2e (t[1]));
+  if (is_tuple (t, "\\addtocounter", 2)) // FIXME: only inc works
+    return compound ("inc-counter", v2e (t[1]));
+  if (is_tuple (t, "\\setlength")) return "";
+  if (is_tuple (t, "\\addtolength")) return "";
+  if (is_tuple (t, "\\enlargethispage")) return "";
   if (is_tuple (t, "\\mathop", 1)) return l2e (t[1]);
   if (is_tuple (t, "\\mathrel", 1)) return l2e (t[1]);
   if (is_tuple (t, "\\overbrace", 1))
@@ -647,6 +768,10 @@ latex_command_to_tree (tree t) {
     return tree (ABOVE, l2e (t[2]), l2e (t[1]));
   if (is_tuple (t, "\\underset", 2))
     return tree (BELOW, l2e (t[2]), l2e (t[1]));
+  if (is_tuple (t, "\\parbox", 2))
+    return compound ("mini-paragraph", v2e (t[1]), l2e (t[2]));
+  if (is_tuple (t, "\\parbox*", 3))
+    return compound ("mini-paragraph", v2e (t[2]), l2e (t[3]));
 
   int dtype= 0;
   if (is_large_delimiter (t, dtype)) {
@@ -664,7 +789,7 @@ latex_command_to_tree (tree t) {
     return latex_cite_to_tree (cite_type, s);
   }
   if (is_tuple (t, "\\cite*", 2)) {
-    tree   ot= t2e(t[1])->label;
+    tree   ot= l2e (t[1]);
     string s = v2e (t[2]);
     tree   ct= latex_cite_to_tree ("cite", s);
     if (N(ct) == 2) return compound ("cite-detail", ct[1], ot);
@@ -716,25 +841,102 @@ latex_command_to_tree (tree t) {
       << tree (END, "array") << tree (RIGHT, ".");
     return r;
   }
-  if (is_tuple (t, "\\includegraphics", 1)) {
+  if (is_tuple (t, "\\includegraphics", 1) ||
+      is_tuple (t, "\\includegraphics*", 1)) {
     tree name= v2e (t[1]);
     if (name == "") return "";
     else {
-      tree g (POSTSCRIPT, 7);
+      tree g (IMAGE, 7);
       g[0]= name;
       return g;
     }
   }
+  if (is_tuple (t, "\\includegraphics*", 2) ||
+      is_tuple (t, "\\includegraphics**", 2)) {
+    tree name= v2e (t[2]);
+    tree data= v2e (t[1]);
+    if (data == "" || name == "") return "";
+    else {
+      tree g (IMAGE, 7);
+      g[0]= name;
+      tree width = latex_eps_get (data, "width");
+      tree height= latex_eps_get (data, "height");
+      g[1]= width;
+      g[2]= height;
+      return g;
+    }
+  }
+  if (is_tuple (t, "\\epsfig", 1)) {
+    tree data  = v2e (t[1]);
+    tree name  = latex_eps_get (data, "file");
+    tree width = latex_eps_get (data, "width");
+    tree height= latex_eps_get (data, "height");
+    if (name == "") return "";
+    else {
+      tree g (IMAGE, 5);
+      g[0]= name;
+      g[1]= width;
+      g[2]= height;
+      return g;
+
+    }
+  }
+  if (is_tuple (t, "\\fbox", 1)) return compound ("frame", l2e (t[1]));
+  if (is_tuple (t, "\\framebox", 1)) return compound ("frame", l2e (t[1]));
+  if (is_tuple (t, "\\centerline", 1)) return compound ("center", l2e (t[1]));
   if (is_tuple (t, "\\noalign", 1))
     return ""; // FIXME: for larger space in maple matrices
   if (is_tuple (t, "\\etalchar", 1)) return t2e (t[1]);
   if (is_tuple (t, "\\natexlab", 1)) return t2e (t[1]);
   if (is_tuple (t, "\\penalty", 1)) return "";
   if (is_tuple (t, "\\url", 1))
-    return tree (APPLY, "slink", t2e (t[1]));
+    return tree (APPLY, "href", t2e (t[1]));
   if (is_tuple (t, "\\href", 2))
     return tree (APPLY, "hlink", l2e (t[2]), t2e (t[1]));
 
+  if (is_tuple (t, "\\xminus", 1))
+    return tree (LONG_ARROW, "<rubber-minus>", l2e (t[1]));
+  if (is_tuple (t, "\\xleftarrow", 1))
+    return tree (LONG_ARROW, "<rubber-leftarrow>", l2e (t[1]));
+  if (is_tuple (t, "\\xrightarrow", 1))
+    return tree (LONG_ARROW, "<rubber-rightarrow>", l2e (t[1]));
+  if (is_tuple (t, "\\xleftrightarrow", 1))
+    return tree (LONG_ARROW, "<rubber-leftrightarrow>", l2e (t[1]));
+  if (is_tuple (t, "\\xmapsto", 1))
+    return tree (LONG_ARROW, "<rubber-mapsto>", l2e (t[1]));
+  if (is_tuple (t, "\\xmapsfrom", 1))
+    return tree (LONG_ARROW, "<rubber-mapsfrom>", l2e (t[1]));
+  if (is_tuple (t, "\\xequal", 1))
+    return tree (LONG_ARROW, "<rubber-equal>", l2e (t[1]));
+  if (is_tuple (t, "\\xLeftarrow", 1))
+    return tree (LONG_ARROW, "<rubber-Leftarrow>", l2e (t[1]));
+  if (is_tuple (t, "\\xRightarrow", 1))
+    return tree (LONG_ARROW, "<rubber-Rightarrow>", l2e (t[1]));
+  if (is_tuple (t, "\\xLeftrightarrow", 1))
+    return tree (LONG_ARROW, "<rubber-Leftrightarrow>", l2e (t[1]));
+  if (is_tuple (t, "\\xminus*", 2))
+    return tree (LONG_ARROW, "<rubber-minus>", l2e (t[1]), l2e (t[2]));
+  if (is_tuple (t, "\\xleftarrow*", 2))
+    return tree (LONG_ARROW, "<rubber-leftarrow>", l2e (t[1]), l2e (t[2]));
+  if (is_tuple (t, "\\xrightarrow*", 2))
+    return tree (LONG_ARROW, "<rubber-rightarrow>", l2e (t[1]), l2e (t[2]));
+  if (is_tuple (t, "\\xleftrightarrow*", 2))
+    return tree (LONG_ARROW, "<rubber-leftrightarrow>",
+                 l2e (t[1]), l2e (t[2]));
+  if (is_tuple (t, "\\xmapsto*", 2))
+    return tree (LONG_ARROW, "<rubber-mapsto>", l2e (t[1]), l2e (t[2]));
+  if (is_tuple (t, "\\xmapsfrom*", 2))
+    return tree (LONG_ARROW, "<rubber-mapsfrom>", l2e (t[1]), l2e (t[2]));
+  if (is_tuple (t, "\\xequal*", 2))
+    return tree (LONG_ARROW, "<rubber-equal>", l2e (t[1]), l2e (t[2]));
+  if (is_tuple (t, "\\xLeftarrow*", 2))
+    return tree (LONG_ARROW, "<rubber-Leftarrow>", l2e (t[1]), l2e (t[2]));
+  if (is_tuple (t, "\\xRightarrow*", 2))
+    return tree (LONG_ARROW, "<rubber-Rightarrow>", l2e (t[1]), l2e (t[2]));
+  if (is_tuple (t, "\\xLeftrightarrow*", 2))
+    return tree (LONG_ARROW, "<rubber-Leftrightarrow>",
+                 l2e (t[1]), l2e (t[2]));
+
   // Start TeXmacs specific markup
   if (is_tuple (t, "\\tmmathbf", 1))
     return tree (CONCAT,
@@ -864,7 +1066,7 @@ parse_matrix_params (tree t) {
 }
 
 static void
-parse_pmatrix (tree& r, tree t, int& i, string lb, string rb) {
+parse_pmatrix (tree& r, tree t, int& i, string lb, string rb, string fm) {
   tree tformat= parse_matrix_params (t[i]);
   if (lb != "") r << tree (LEFT, lb);
 
@@ -887,37 +1089,55 @@ parse_pmatrix (tree& r, tree t, int& i, string lb, string rb) {
       E= tree (CONCAT);
       continue;
     }
-    else if (is_func (v, BEGIN) && ((v[0] == "array") ||(v[0] == "tabular"))) {
-      parse_pmatrix (E, t, i, "", "");
+    else if (is_func (v, BEGIN) && (v[0] == "array" || v[0] == "tabular")) {
+      parse_pmatrix (E, t, i, "", "", "tabular*");
+      if (i<N(t)) continue;
+      break;
+    }
+    else if (v == tree (BEGIN, "stack")) {
+      parse_pmatrix (E, t, i, "", "", "stack");
+      if (i<N(t)) continue;
+      break;
+    }
+    else if (v == tree (BEGIN, "cases")) {
+      parse_pmatrix (E, t, i, "", "", "choice");
       if (i<N(t)) continue;
       break;
     }
     else if (v == tree (BEGIN, "matrix")) {
-      parse_pmatrix (E, t, i, "", "");
+      parse_pmatrix (E, t, i, "", "", "tabular*");
       if (i<N(t)) continue;
       break;
     }
     else if (v == tree (BEGIN, "pmatrix")) {
-      parse_pmatrix (E, t, i, "(", ")");
+      parse_pmatrix (E, t, i, "", "", "matrix");
       if (i<N(t)) continue;
       break;
     }
     else if (v == tree (BEGIN, "bmatrix")) {
-      parse_pmatrix (E, t, i, "{", "}");
+      parse_pmatrix (E, t, i, "[", "]", "tabular*");
       if (i<N(t)) continue;
       break;
     }
     else if (v == tree (BEGIN, "vmatrix")) {
-      parse_pmatrix (E, t, i, "|", "|");
+      parse_pmatrix (E, t, i, "", "", "det");
+      if (i<N(t)) continue;
+      break;
+    }
+    else if (v == tree (BEGIN, "smallmatrix")) {
+      parse_pmatrix (E, t, i, "", "", "matrix*");
       if (i<N(t)) continue;
       break;
     }
     else if (v == tree (END, "array")) break;
     else if (v == tree (END, "tabular")) break;
+    else if (v == tree (END, "cases")) break;
+    else if (v == tree (END, "stack")) break;
     else if (v == tree (END, "matrix")) break;
     else if (v == tree (END, "pmatrix")) break;
     else if (v == tree (END, "bmatrix")) break;
     else if (v == tree (END, "vmatrix")) break;
+    else if (v == tree (END, "smallmatrix")) break;
     else if (v == tree (APPLY, "hline")) {
       int    row  = N(V)+ (N(L)==0? 0: 1);
       string row_s= row==0? as_string (row+1): as_string (row);
@@ -939,17 +1159,15 @@ parse_pmatrix (tree& r, tree t, int& i, string lb, string rb) {
   rows= N(V);
 
   int x, y;
-  tree M (OLD_MATRIX);
-  for (y=0; y<rows; y++)
+  tree M (TABLE);
+  for (y=0; y<rows; y++) {
+    tree R (ROW);
     for (x=0; x<cols; x++)
-      if (x<N(V[y])) M << V[y][x];
-      else M << "";
-  M << as_string (cols) << as_string (rows);
-  if (tformat != tree (TFORMAT)) {
-    tformat << M;
-    M= tformat;
+      if (x<N(V[y])) R << tree (CELL, V[y][x]);
+      else R << tree (CELL, "");
+    M << R;
   }
-  r << M;
+  r << compound (fm, tree (TFORMAT, M));
   if (rb != "") r << tree (RIGHT, rb);
 }
 
@@ -963,12 +1181,24 @@ finalize_pmatrix (tree t) {
     tree r (CONCAT);
     for (i=0; i<n; i++)
       if (is_func (u[i], BEGIN)) {
-	if (u[i][0] == "array") parse_pmatrix (r, u, i, "", "");
-	else if (u[i][0] == "tabular") parse_pmatrix (r, u, i, "", "");
-	else if (u[i][0] == "matrix") parse_pmatrix (r, u, i, "", "");
-	else if (u[i][0] == "pmatrix") parse_pmatrix (r, u, i, "(", ")");
-	else if (u[i][0] == "bmatrix") parse_pmatrix (r, u, i, "[", "]");
-	else if (u[i][0] == "vmatrix") parse_pmatrix (r, u, i, "|", "|");
+	if (u[i][0] == "array")
+	  parse_pmatrix (r, u, i, "", "", "tabular*");
+	else if (u[i][0] == "tabular")
+	  parse_pmatrix (r, u, i, "", "", "tabular*");
+	else if (u[i][0] == "cases")
+	  parse_pmatrix (r, u, i, "", "", "choice");
+	else if (u[i][0] == "stack")
+	  parse_pmatrix (r, u, i, "", "", "stack");
+	else if (u[i][0] == "matrix")
+	  parse_pmatrix (r, u, i, "", "", "tabular*");
+	else if (u[i][0] == "pmatrix")
+	  parse_pmatrix (r, u, i, "", "", "matrix");
+	else if (u[i][0] == "bmatrix")
+	  parse_pmatrix (r, u, i, "[", "]", "tabular*");
+	else if (u[i][0] == "vmatrix")
+	  parse_pmatrix (r, u, i, "", "", "det");
+	else if (u[i][0] == "smallmatrix")
+	  parse_pmatrix (r, u, i, "", "", "matrix*");
 	else r << u[i];
       }
       else r << u[i];
@@ -976,6 +1206,16 @@ finalize_pmatrix (tree t) {
   }
   else if (is_func (u, APPLY, 2) && (u[0] == "matrix"))
     return tree (APPLY, "tabular*", u[1]);
+  else if (is_func (u, APPLY, 2) && (u[0] == "smallmatrix"))
+    return tree (APPLY, "matrix*", u[1]);
+  else if (is_func (u, APPLY, 2) && (u[0] == "substack")) {
+    tree cc (CONCAT);
+    cc << tree (BEGIN, "stack");
+    if (is_func (u[1], CONCAT)) cc << A(u[1]);
+    else cc << u[1];
+    cc << tree (END, "stack");
+    return finalize_pmatrix (cc);
+  }
   else return u;
 }
 
@@ -1003,6 +1243,7 @@ space_eater (tree t) {
 static bool
 admissible_env (tree t) {
   string s= t[0]->label;
+  if (ends (s, "*")) s= s (0, N(s)-1);
   if (latex_type ("\\begin-" * s) == "list") return true;
   if (latex_type ("\\begin-" * s) == "environment") return true;
   if (latex_type ("\\begin-" * s) == "math-environment") return true;
@@ -1019,6 +1260,20 @@ translate_list (string s) {
   if (s == "enumerateromancap") return "enumerate-romancap";
   if (s == "enumeratealpha") return "enumerate-alpha";
   if (s == "enumeratealphacap") return "enumerate-alphacap";
+  if (s == "asparaitem") return "itemize";
+  if (s == "inparaitem") return "itemize";
+  if (s == "compactitem") return "itemize";
+  if (s == "asparaenum") return "enumerate";
+  if (s == "inparaenum") return "enumerate";
+  if (s == "compactenum") return "enumerate";
+  if (s == "itemize*") return "itemize";
+  if (s == "enumerate*") return "enumerate";
+  if (s == "asparaitem*") return "itemize";
+  if (s == "inparaitem*") return "itemize";
+  if (s == "compactitem*") return "itemize";
+  if (s == "asparaenum*") return "enumerate";
+  if (s == "inparaenum*") return "enumerate";
+  if (s == "compactenum*") return "enumerate";
   return s;
 }
 
@@ -1045,7 +1300,7 @@ finalize_layout (tree t) {
 
       if (is_func (v, BEGIN, 1) && (v[0] == "picture")) {
 	for (; i<n; i++)
-	  if (is_func (u[i], POSTSCRIPT)) r << u[i];
+	  if (is_func (u[i], IMAGE)) r << u[i];
 	  else if (is_func (u[i], END, 1) && (u[i][0] == "picture"))
 	    break;
 	continue;
@@ -1083,7 +1338,8 @@ finalize_layout (tree t) {
       }
       */
 
-      if (is_func (v, BEGIN, 1) && admissible_env (v)) {
+      if ((is_func (v, BEGIN, 1) || is_func (v, BEGIN, 2))
+          && admissible_env (v) && v[0] != "thebibliography") {
 	if (v == tree (BEGIN, "verbatim")) {
 	  r << v; i++;
 	  if ((i<n) && (t[i] == tree (FORMAT, "new line"))) {
@@ -1160,6 +1416,49 @@ finalize_layout (tree t) {
   else return u;
 }
 
+static tree
+finalize_sections (tree t) {
+  tree r (DOCUMENT);
+  for (int i=0; i<N(t); i++) {
+    tree u= t[i];
+    if (is_concat (u) && N(u) >= 2 &&
+	(is_var_compound (u[0], "part", 1) ||
+	 is_var_compound (u[0], "part*", 1) ||
+	 is_var_compound (u[0], "chapter", 1) ||
+	 is_var_compound (u[0], "chapter*", 1) ||
+	 is_var_compound (u[0], "section", 1) ||
+	 is_var_compound (u[0], "section*", 1) ||
+	 is_var_compound (u[0], "subsection", 1) ||
+	 is_var_compound (u[0], "subsection*", 1) ||
+	 is_var_compound (u[0], "subsubsection", 1) ||
+	 is_var_compound (u[0], "subsubsection*", 1)))
+      {
+	if (N(u) > 2 && u[1] == " ")
+	  u= u (0, 1) * u (2, N(u));
+	if (!is_func (u[1], LABEL) || (N(u) >= 3 && !is_func (u[2], LABEL))) {
+	  if (!is_func (u[1], LABEL)) {
+	    r << u[0];
+	    if (N(u) == 2) u= u[1];
+	    else u= u (1, N(u));
+	  }
+	  else {
+	    r << u (0, 2);
+	    if (N(u) == 3) u= u[2];
+	    else u= u (2, N(u));
+	  }
+	  if (is_atomic (u) && starts (u->label, " "))
+	    u= u->label (1, N(u->label));
+	  if (is_concat (u) && is_atomic (u[0]) && starts (u[0]->label, " "))
+	    u= tree (CONCAT, u[0]->label (1, N(u[0]->label))) * u (1, N(u));
+	  r << u;
+	}
+	else r << u;
+      }
+    else r << u;
+  }
+  return r;
+}
+
 static tree
 finalize_document (tree t) {
   if (is_atomic (t)) t= tree (CONCAT, t);
@@ -1178,7 +1477,7 @@ finalize_document (tree t) {
     else r << t(start,i);
     if (i==(N(t)-1)) r << "";
   }
-  return r;
+  return finalize_sections (r);
 }
 
 bool
@@ -1267,6 +1566,75 @@ handle_improper_matches (tree& r, tree t, int& pos) {
   }
 }
 
+tree
+remove_env_spaces (tree t, bool& done) {
+  if (is_atomic (t)) {
+    done= done || (t != "" && t != " ");
+    return t;
+  }
+  else if (is_func (t, CONCAT)) {
+    array<tree> r;
+    for (int i=0; i<N(t); i++)
+      if (!done && t[i] == "");
+      else if (!done && t[i] == " ");
+      else r << remove_env_spaces (t[i], done);
+    return simplify_concat (tree (CONCAT, r));
+  }
+  else if (is_func (t, WITH)) {
+    tree r= t (0, N(t));
+    r[N(t)-1]= remove_env_spaces (r[N(t)-1], done);
+    return r;
+  }
+  else return t;
+}
+
+tree
+env_hacks (tree t) {
+  int i, n= N(t);
+  bool done= false;
+  tree beg= remove_env_spaces (t[n-2], done);
+  tree end= remove_env_spaces (t[n-1], done);
+  if (beg == "") beg= tree (CONCAT);
+  else if (!is_concat (beg)) beg= tree (CONCAT, beg);
+  if (end == "") end= tree (CONCAT);
+  else if (!is_concat (end)) end= tree (CONCAT, end);
+  array<tree> ba;
+  array<tree> ea;
+  for (i=0; i<N(beg); i++)
+    if (is_func (beg[i], VSPACE))
+      ba << tree (VAR_VSPACE, A(beg[i]));
+    else if (is_func (beg[i], PAGE_BREAK))
+      ba << tree (VAR_PAGE_BREAK, A(beg[i]));
+    else if (is_func (beg[i], NO_PAGE_BREAK))
+      ba << tree (VAR_NO_PAGE_BREAK, A(beg[i]));
+    else if (is_func (beg[i], NEW_PAGE))
+      ba << tree (VAR_NEW_PAGE, A(beg[i]));
+    else ba << beg[i];
+  for (i=0; i<N(end); i++)
+    if (is_func (end[i], NO_INDENT))
+      ea << tree (VAR_NO_INDENT, A(end[i]));
+    else if (is_func (end[i], YES_INDENT))
+      ea << tree (VAR_YES_INDENT, A(end[i]));
+    else ea << end[i];
+  beg= tree (CONCAT, ba);
+  end= tree (CONCAT, ea);
+  for (i=N(beg); i>0 && is_func (beg[i-1], RESET, 1); i--) ;
+  bool ok= (i<<1) >= N(beg);
+  for (int k=0; k<N(beg)-i; k++) {
+    ok= ok && is_func (beg[i-k-1], SET, 2) && beg[i-k-1][0] == beg[i+k][0];
+    //cout << "Matched " << beg[i-k-1] << " and " << beg[i+k] << "\n";
+  }
+  if (ok && i<N(beg)) {
+    tree r= t (0, n);
+    r[n-2]= simplify_concat (beg (0, i));
+    r[n-1]= simplify_concat (beg (i, N(beg)) * end);
+    //cout << "<< " << t << "\n";
+    //cout << ">> " << r << "\n";
+    t= r;
+  }
+  return t;
+}
+
 tree
 handle_improper_matches (tree t) {
   if (is_atomic (t)) return t;
@@ -1284,6 +1652,8 @@ handle_improper_matches (tree t) {
       if (N(u)==1) return u[0];
       return u;
     }
+    else if (is_func (r, ENV) && N(r) >= 2)
+      return env_hacks (r);
     return r;
   }
 }
@@ -1295,8 +1665,8 @@ handle_improper_matches (tree t) {
 tree
 float_body (tree t) {
   if (is_atomic (t)) return t;
-  else if (is_compound (t, "caption", 1)) return "";
-  else if (is_compound (t, "center", 1)) return float_body (t[0]);
+  else if (is_var_compound (t, "caption", 1)) return "";
+  else if (is_var_compound (t, "center", 1)) return float_body (t[N(t)-1]);
   else {
     int i, n= N(t);
     tree r (t, n);
@@ -1310,7 +1680,7 @@ float_body (tree t) {
 tree
 find_caption (tree t) {
   if (is_atomic (t)) return "";
-  else if (is_compound (t, "caption", 1)) return t[0];
+  else if (is_var_compound (t, "caption", 1)) return t[N(t)-1];
   else {
     int i, n= N(t);
     for (i=0; i<n; i++) {
@@ -1324,12 +1694,12 @@ find_caption (tree t) {
 tree
 finalize_floats (tree t) {
   if (is_atomic (t)) return t;
-  else if (is_compound (t, "bigfigure", 1)) {
+  else if (is_var_compound (t, "bigfigure", 1)) {
     tree body= float_body (t[N(t)-1]);
     tree capt= find_caption (t[N(t)-1]);
     return tree (make_tree_label ("big-figure"), body, capt);
   }
-  else if (is_compound (t, "bigtable", 1)) {
+  else if (is_var_compound (t, "bigtable", 1)) {
     tree body= float_body (t[N(t)-1]);
     tree capt= find_caption (t[N(t)-1]);
     return tree (make_tree_label ("big-table"), body, capt);
@@ -1343,6 +1713,44 @@ finalize_floats (tree t) {
   }
 }
 
+static bool
+is_hyper_link (string s) {
+  return starts (s, "http://") || starts (s, "ftp://");
+}
+
+tree
+finalize_misc (tree t) {
+  if (is_atomic (t)) return t;
+  else if (is_compound (t, "verbatim", 1) &&
+           is_atomic (t[0]) && is_hyper_link (t[0]->label)) {
+    return compound ("href", finalize_misc (t[0]));
+  }
+  else if (is_func (t, WITH, 3) && t[0] == FONT_FAMILY && t[1] == "tt" &&
+           is_atomic (t[2]) && is_hyper_link (t[2]->label)) {
+    return compound ("href", finalize_misc (t[2]));
+  }
+  else if (is_compound (t, "flushleft", 1) ||
+           is_compound (t, "leftaligned", 1))
+    return compound ("left-aligned", finalize_misc (t[0]));
+  else if (is_compound (t, "flushright", 1) ||
+           is_compound (t, "rightaligned", 1))
+    return compound ("right-aligned", finalize_misc (t[0]));
+  else if (is_compound (t, "acknowledgments", 1))
+    return compound ("acknowledgments*", finalize_misc (t[0]));
+  else if (is_compound (t, "text", 1) &&
+           is_func (t[0], WITH, 3) &&
+           t[0][0] == "font-family" &&
+           t[0][1] == "rm")
+    return compound ("math-up", finalize_misc (t[0][2]));
+  else {
+    int i, n= N(t);
+    tree r (t, n);
+    for (i=0; i<n; i++)
+      r[i]= finalize_misc (t[i]);
+    return r;
+  }
+}
+
 /******************************************************************************
 * Final changes programmed in Guile
 ******************************************************************************/
@@ -1370,9 +1778,9 @@ latex_to_tree (tree t1) {
   textm_unicode   = false;
   textm_natbib    = false;
   command_type ("!em") = "false";
-  // cout << "\n\nt1= " << t1 << "\n\n";
+  //cout << "\n\nt1= " << t1 << "\n\n";
   tree t2= is_document? filter_preamble (t1): t1;
-  // cout << "\n\nt2= " << t2 << "\n\n";
+  //cout << "\n\nt2= " << t2 << "\n\n";
   tree t3= parsed_latex_to_tree (t2);
   // cout << "\n\nt3= " << t3 << "\n\n";
   tree t4= finalize_document (t3);
@@ -1380,17 +1788,17 @@ latex_to_tree (tree t1) {
   tree t5= is_document? finalize_preamble (t4, style): t4;
   // cout << "\n\nt5= " << t5 << "\n\n";
   tree t6= handle_improper_matches (t5);
-  // cout << "\n\nt6= " << t6 << "\n\n";
+  //cout << "\n\nt6= " << t6 << "\n\n";
   if ((!is_document) && is_func (t6, DOCUMENT, 1)) t6= t6[0];
   tree t7= upgrade_tex (t6);
-  // cout << "\n\nt7= " << t7 << "\n\n";
+  //cout << "\n\nt7= " << t7 << "\n\n";
   tree t8= finalize_floats (t7);
   // cout << "\n\nt8= " << t8 << "\n\n";
-  tree t9= finalize_textm (t8);
+  tree t9= finalize_misc (t8);
   // cout << "\n\nt9= " << t9 << "\n\n";
-  tree t10= drd_correct (std_drd, t9);
+  tree t10= finalize_textm (t9);
   // cout << "\n\nt10= " << t10 << "\n\n";
-  tree t11= simplify_correct (t10);
+  tree t11= drd_correct (std_drd, t10);
   // cout << "\n\nt11= " << t11 << "\n\n";
 
   if (!exists (url ("$TEXMACS_STYLE_PATH", style * ".ts")))
@@ -1409,8 +1817,16 @@ latex_to_tree (tree t1) {
     initial << tree (ASSOCIATE, LANGUAGE, lan);
     mods << tree (LANGUAGE) << tree (lan);
   }
+
+  tree t12= t11;
+  if (is_document) t12= simplify_correct (t11);
+  else if (N (mods) > 0) { t12= mods; t12 << t11; }
+  // cout << "\n\nt12= " << t12 << "\n\n";
+  tree t13= latex_correct (t12);
+  // cout << "\n\nt13= " << t13 << "\n\n";
+
   if (is_document) {
-    tree the_body   = compound ("body", t11);
+    tree the_body   = compound ("body", t13);
     tree the_style  = compound ("style", style);
     tree the_initial= compound ("initial", initial);
     if (textm_natbib)
@@ -1418,8 +1834,18 @@ latex_to_tree (tree t1) {
     if (N (initial) == 0) return tree (DOCUMENT, the_style, the_body);
     else return tree (DOCUMENT, the_style, the_body, the_initial);
   }
-  else {
-    if (N (mods) > 0) { mods << t10; return mods; }
-    return t10;
-  }
+  else return t13;
+}
+
+tree
+latex_document_to_tree (string s) {
+  command_type ->extend ();
+  command_arity->extend ();
+  command_def  ->extend ();
+  tree t= parse_latex_document (s, true);
+  tree r= latex_to_tree (t);
+  command_type ->shorten ();
+  command_arity->shorten ();
+  command_def  ->shorten ();
+  return r;
 }
