@@ -12,6 +12,8 @@
 #include "Tex/convert_tex.hpp"
 #include "converter.hpp"
 
+string string_arg (tree t);
+
 /******************************************************************************
 * The latex_parser structure
 *******************************************************************************
@@ -50,7 +52,7 @@ struct latex_parser {
   tree parse_unknown   (string s, int& i, string which);
   tree parse_verbatim  (string s, int& i, string end);
 
-  tree parse           (string s);
+  tree parse           (string s, bool change);
 };
 
 /******************************************************************************
@@ -98,8 +100,7 @@ latex_parser::parse (string s, int& i, string stop, bool change) {
 	   (i+4>n || s(i,i+4) != "\\end")))) {
     switch (s[i]) {
     case '~':
-      if (command_type ["!mode"] == "math") t << tuple ("\\sim");
-      else t << tuple ("\\nbsp");
+      t << tuple ("\\nbsp");
       i++;
       break;
     case ' ':
@@ -232,7 +233,8 @@ latex_parser::parse (string s, int& i, string stop, bool change) {
       if ((i<n) && (!is_space (s[i]))) break;
       while ((i<n) && is_space (s[i]))
 	if (s[i++]=='\n') ln++;
-      if (ln >= 2) t << "\n"; else t << tree (TUPLE, "\\ ");
+      if (ln >= 2) t << "\n";
+      else if (i<n) t << tree (TUPLE, "\\ ");
       break;
     }
     case '$': {
@@ -329,7 +331,7 @@ latex_parser::parse_backslash (string s, int& i) {
     i+=16;
     return parse_verbatim (s, i, "\\end{verbatim}");
   }
-  if (((i+5)<n) && (s(i,i+4)=="\\url")) {
+  if (((i+5)<n) && (s(i,i+4)=="\\url") && !is_alpha (s[i+5])) {
     i+=4;
     while (i<n && (s[i] == ' ' || s[i] == '\n' || s[i] == '\t')) i++;
     string ss;
@@ -399,7 +401,7 @@ sharp_to_arg (string s, tree args) {
   for (i=0; i<N(s); i++)
     if ((s[i]=='#') && ((i+1)<N(s)) && (s[i+1]>='1') && (s[i+1]<='9')) {
       int nr= ((int) s[++i]) - ((int) '0');
-      if (N(args)>nr) r << as_string (args[nr]);
+      if (N(args)>nr) r << string_arg (args[nr]);
     }
     else r << s[i];
   return r;
@@ -421,12 +423,46 @@ latex_parser::parse_symbol (string s, int& i) {
   return s(start,i);
 }
 
+static bool
+is_math_environment (tree t) {
+  //cout << "t= " << t << "\n";
+  tree b= t[N(t)-2];
+  tree e= t[N(t)-1];
+  if (!is_concat (b)) b= tree (CONCAT, b);
+  if (!is_concat (e)) e= tree (CONCAT, e);
+  int i, j;
+  for (i=N(b)-1; i>=0; i--)
+    if (is_tuple (b[i]) && N(b[i])>0 && is_atomic (b[i][0]))
+      if (latex_type (b[i][0]->label) == "math-environment")
+	break;
+  for (j=0; j<N(e); j++)
+    if (is_tuple (e[j]) && N(e[j])>0 && is_atomic (e[j][0]))
+      if (latex_type (e[j][0]->label) == "math-environment")
+	break;
+  if (i >= 0 && j < N(e)) {
+    string bs= b[i][0]->label;
+    string es= e[j][0]->label;
+    bool ok=
+      starts (bs, "\\begin-") &&
+      starts (es, "\\end-") &&
+      bs (7, N(bs)) == es (5, N(es));
+    //cout << t[1] << " -> " << ok << "\n";
+    return ok;
+  }
+  return false;
+}
+
+static bool
+is_text_argument (string cmd, int remaining_arity) {
+  // FIXME: this test should be improved using DRD properties
+  (void) remaining_arity;
+  return cmd == "\\label" || cmd == "\\ref";
+}
+
 tree
 latex_parser::parse_command (string s, int& i, string cmd) {
-  /*
-  cout << cmd << " [" << latex_type (cmd) << ", "
-       << command_type ["!mode"] << "]" << LF;
-  */
+  //cout << cmd << " [" << latex_type (cmd) << ", "
+  //<< command_type ["!mode"] << ", " << latex_arity (cmd) << "]" << LF;
   if (cmd == "\\newcommand") cmd= "\\def";
   if (cmd == "\\renewcommand") cmd= "\\def";
   if (cmd == "\\renewenvironment") cmd= "\\newenvironment";
@@ -462,25 +498,33 @@ latex_parser::parse_command (string s, int& i, string cmd) {
     if (option && (s[j]=='[')) {
       j++;
       i=j;
-      t << parse (s, i, "]");
+      tree opt= parse (s, i, "]");
+      if (cmd != "\\newtheorem" && cmd != "\\newtheorem*")
+	t << opt;
       u << s (j, i);
       if ((i<n) && (s[i]==']')) i++;
-      t[0]->label= t[0]->label * "*";
+      if (cmd != "\\newtheorem" && cmd != "\\newtheorem*")
+	t[0]->label= t[0]->label * "*";
       option= false;
     }
     else if ((arity>0) && (s[j]=='{')) {
+      bool text_arg=
+	(command_type["!mode"] == "math") && is_text_argument (cmd, arity);
       j++;
       i=j;
+      if (text_arg) command_type ("!mode")= "text";
       if ((N(t)==1) && (cmd == "\\def")) {
 	while ((i<n) && (s[i]!='}')) i++;
 	t << s (j, i);
       }
       else t << parse (s, i, "}");
+      if (text_arg) command_type ("!mode")= "math";
       u << s (j, i);
       if ((i<n) && (s[i]=='}')) i++;
       arity--;
       if (arity == 0) option= false;
     }
+    else if (s[j] == '}') break;
     else if (option && (s[j]=='#') && (cmd == "\\def")) {
       while ((j+3 <= n) && is_numeric (s[j+1]) && (s[j+2] == '#')) j+=2;
       if (j+2<=n) {
@@ -507,36 +551,48 @@ latex_parser::parse_command (string s, int& i, string cmd) {
 
   /******************** new commands and environments ************************/
   if (is_tuple (t, "\\def", 2)) {
-    string var= as_string (t[1]);
+    string var= string_arg (t[1]);
     command_type  (var)= "user";
     command_arity (var)= 0;
     command_def   (var)= as_string (u[2]);
   }
   if (is_tuple (t, "\\def*", 3)) {
-    string var= as_string (t[1]);
+    string var= string_arg (t[1]);
     command_type  (var)= "user";
     command_arity (var)= as_int (t[2]);
     command_def   (var)= as_string (u[3]);
   }
+  if (is_tuple (t, "\\newtheorem", 2) || is_tuple (t, "\\newtheorem*", 2)) {
+    string var= "\\begin-" * string_arg (t[1]);
+    command_type  (var)= "environment";
+    command_arity (var)= 0;
+    var= "\\end-" * string_arg (t[1]);
+    command_type  (var)= "environment";
+    command_arity (var)= 0;
+  }
   if (is_tuple (t, "\\newenvironment", 3)) {
-    string var= "\\begin-" * as_string (t[1]);
+    string var= "\\begin-" * string_arg (t[1]);
     command_type  (var)= "user";
     command_arity (var)= 0;
     command_def   (var)= as_string (u[2]);
-    var= "\\end-" * as_string (t[1]);
+    if (is_math_environment (t)) command_type (var)= "math-environment";
+    var= "\\end-" * string_arg (t[1]);
     command_type  (var)= "user";
     command_arity (var)= 0;
     command_def   (var)= as_string (u[3]);
+    if (is_math_environment (t)) command_type (var)= "math-environment";
   }
   if (is_tuple (t, "\\newenvironment*", 4)) {
-    string var= "\\begin-" * as_string (t[1]);
+    string var= "\\begin-" * string_arg (t[1]);
     command_type  (var)= "user";
     command_arity (var)= as_int (t[2]);
     command_def   (var)= as_string (u[3]);
-    var= "\\end-" * as_string (t[1]);
+    if (is_math_environment (t)) command_type (var)= "math-environment";
+    var= "\\end-" * string_arg (t[1]);
     command_type  (var)= "user";
     command_arity (var)= 0;
     command_def   (var)= as_string (u[4]);
+    if (is_math_environment (t)) command_type (var)= "math-environment";
   }
 
   /***************** environment changes for user commands  ******************/
@@ -688,7 +744,7 @@ accented_to_Cork (tree t) {
 ******************************************************************************/
 
 tree
-latex_parser::parse (string s) {
+latex_parser::parse (string s, bool change) {
   command_type ->extend ();
   command_arity->extend ();
   command_def  ->extend ();
@@ -698,7 +754,7 @@ latex_parser::parse (string s) {
   array<string> a;
   int i, start=0, n= N(s);
   for (i=0; i<n; i++)
-    if (s[i]=='\n') {
+    if (s[i]=='\n' || (s[i] == '\\' && test (s, i, "\\nextbib"))) {
       while ((i<n) && is_space (s[i])) i++;
       if (test (s, i, "%%%%%%%%%% Start TeXmacs macros\n")) {
 	a << s (start, i);
@@ -716,11 +772,36 @@ latex_parser::parse (string s) {
 	  test (s, i, "\\subsubsection") ||
 	  test (s, i, "\\paragraph") ||
 	  test (s, i, "\\subparagraph") ||
+	  test (s, i, "\\nextbib") ||
 	  test (s, i, "\\newcommand") ||
-	  test (s, i, "\\def"))
+	  test (s, i, "\\def") ||
+	  test (s, i, "\\input{") ||
+	  test (s, i, "\\include{"))
 	{
 	  a << s (start, i);
 	  start= i;
+          if (test (s, i, "\\input{") || test (s, i, "\\include{")) {
+	    while (i<N(s) && s[i] != '{') i++;
+	    int start_name= i+1;
+            while (i<N(s) && s[i] != '}') i++;
+            string name= s (start_name, i);
+            if (!ends (name, ".tex")) name= name * ".tex";
+            url incl= relative (get_file_focus (), name);
+            string body;
+            if (!exists (incl) || load_string (incl, body, false)) i++;
+            else {
+              //cout << "Include " << name << " -> " << incl << "\n";
+              s= s (0, start) * "\n" * body * "\n" * s (i+1, N(s));
+              n= N(s);
+              i= start + 1;
+            }
+            start= i;
+          }
+          while (i < n && test (s, i, "\\nextbib{}")) {
+            i += 10;
+            a << s (start, i);
+            start= i;
+          }
 	}
       if (i == n) break;
     }
@@ -742,9 +823,17 @@ latex_parser::parse (string s) {
     }
   }
 
-  command_type ->shorten ();
-  command_arity->shorten ();
-  command_def  ->shorten ();
+  if (change) {
+    command_type ->merge ();
+    command_arity->merge ();
+    command_def  ->merge ();
+  }
+  else {
+    command_type ->shorten ();
+    command_arity->shorten ();
+    command_def  ->shorten ();
+  }
+  //cout << "Parsed " << t << "\n";
   return t;
 }
 
@@ -849,7 +938,7 @@ taiwanese_tex (string& s) {
 }
 
 tree
-parse_latex (string s) {
+parse_latex (string s, bool change) {
   s= dos_to_better (s);
   string lan= "";
   if (japanese_tex (s)) lan= "japanese";
@@ -859,12 +948,12 @@ parse_latex (string s) {
   bool unicode= (lan == "chinese" || lan == "japanese" ||
 		 lan == "korean" || lan == "taiwanese");
   latex_parser ltx (unicode);
-  tree r= accented_to_Cork (ltx.parse (s));
+  tree r= accented_to_Cork (ltx.parse (s, change));
   if (lan == "") return r;
   return compound ("!language", r, lan);
 }
 
 tree
-parse_latex_document (string s) {
-  return compound ("!file", parse_latex (s));
+parse_latex_document (string s, bool change) {
+  return compound ("!file", parse_latex (s, change));
 }
