@@ -14,10 +14,13 @@
 #include "analyze.hpp"
 #include "hashmap.hpp"
 typedef tree scheme_tree;
+class url;
 
 /*** Miscellaneous ***/
 bool   is_snippet (tree doc);
 string get_texmacs_path ();
+void   set_file_focus (url u);
+url    get_file_focus ();
 
 /*** Generic ***/
 string suffix_to_format (string suffix);
@@ -55,13 +58,18 @@ tree   verbatim_to_tree (string s, bool wrap= false, string enc= "default");
 tree   verbatim_document_to_tree (string s, bool w= false, string e= "default");
 
 /*** Latex ***/
-tree   parse_latex (string s);
-tree   parse_latex_document (string s);
+tree   parse_latex (string s, bool change= false);
+tree   parse_latex_document (string s, bool change= false);
 tree   latex_to_tree (tree t);
+tree   latex_document_to_tree (string s);
 
-/*** Xml / Html ***/
+/*** Xml / Html / Mathml ***/
 tree   parse_xml (string s);
 tree   parse_html (string s);
 tree   tmml_upgrade (scheme_tree t);
+tree   upgrade_mathml (tree t);
+
+/*** BibTeX ***/
+tree   parse_bib (string s);
 
 #endif // defined CONVERT_H
