@@ -41,6 +41,7 @@ string FONT_SIZE ("font-size");
 string FONT_BASE_SIZE ("font-base-size");
 string MAGNIFICATION ("magnification");
 string COLOR ("color");
+string OPACITY ("opacity");
 string BG_COLOR ("bg-color");
 string LOCUS_COLOR ("locus-color");
 string VISITED_COLOR ("visited-color");
@@ -60,6 +61,8 @@ string MATH_LEVEL ("math-level");
 string MATH_DISPLAY ("math-display");
 string MATH_CONDENSED ("math-condensed");
 string MATH_VPOS ("math-vpos");
+string MATH_NESTING_MODE ("math-nesting-mode");
+string MATH_NESTING_LEVEL ("math-nesting-level");
 
 string PROG_LANGUAGE ("prog-language");
 string PROG_SCRIPTS ("prog-scripts");
@@ -74,6 +77,7 @@ string PROG_SESSION ("prog-session");
 ******************************************************************************/
 
 string PAR_MODE ("par-mode");
+string PAR_FLEXIBILITY ("par-flexibility");
 string PAR_HYPHEN ("par-hyphen");
 string PAR_WIDTH ("par-width");
 string PAR_LEFT ("par-left");
@@ -199,35 +203,55 @@ string CELL_COL_NR ("cell-col-nr");
 * Environment variables for graphics
 ******************************************************************************/
 
-string POINT_STYLE ("point-style");
-
-string LINE_WIDTH ("line-width");
-string DASH_STYLE ("dash-style");
-string DASH_STYLE_UNIT ("dash-style-unit");
-string LINE_ARROWS ("line-arrows");
-string LINE_CAPS ("line-caps");
-string FILL_COLOR ("fill-color");
-string FILL_STYLE ("fill-style");
-string TEXTAT_HALIGN ("text-at-halign");
-string TEXTAT_VALIGN ("text-at-valign");
-
 string GR_GEOMETRY ("gr-geometry");
 string GR_FRAME ("gr-frame");
 string GR_MODE ("gr-mode");
-string GR_COLOR ("gr-color");
-string GR_POINT_STYLE ("gr-point-style");
-string GR_LINE_WIDTH ("gr-line-width");
-string GR_DASH_STYLE ("gr-dash-style");
-string GR_DASH_STYLE_UNIT ("gr-dash-style-unit");
-string GR_LINE_ARROWS ("gr-line-arrows");
-string GR_FILL_COLOR ("gr-fill-color");
-string GR_TEXTAT_HALIGN ("gr-text-at-halign");
-string GR_TEXTAT_VALIGN ("gr-text-at-valign");
+string GR_AUTO_CROP ("gr-auto-crop");
+string GR_CROP_PADDING ("gr-crop-padding");
 string GR_GRID ("gr-grid");
 string GR_GRID_ASPECT ("gr-grid-aspect");
 string GR_EDIT_GRID ("gr-edit-grid");
 string GR_EDIT_GRID_ASPECT ("gr-edit-grid-aspect");
 
+string GR_MAGNIFY ("gr-magnify");
+string GR_OPACITY ("gr-opacity");
+string GR_COLOR ("gr-color");
+string GR_POINT_STYLE ("gr-point-style");
+string GR_LINE_WIDTH ("gr-line-width");
+string GR_LINE_JOIN ("gr-line-join");
+string GR_LINE_CAPS ("gr-line-caps");
+string GR_LINE_EFFECTS ("gr-line-effects");
+string GR_DASH_STYLE ("gr-dash-style");
+string GR_DASH_STYLE_UNIT ("gr-dash-style-unit");
+string GR_ARROW_BEGIN ("gr-arrow-begin");
+string GR_ARROW_END ("gr-arrow-end");
+string GR_ARROW_LENGTH ("gr-arrow-length");
+string GR_ARROW_HEIGHT ("gr-arrow-height");
+string GR_FILL_COLOR ("gr-fill-color");
+string GR_FILL_STYLE ("gr-fill-style");
+string GR_TEXT_AT_HALIGN ("gr-text-at-halign");
+string GR_TEXT_AT_VALIGN ("gr-text-at-valign");
+string GR_TEXT_AT_MARGIN ("gr-text-at-margin");
+
+string GID ("gid");
+string MAGNIFY ("magnify");
+string POINT_STYLE ("point-style");
+string LINE_WIDTH ("line-width");
+string LINE_JOIN ("line-join");
+string LINE_CAPS ("line-caps");
+string LINE_EFFECTS ("line-effects");
+string DASH_STYLE ("dash-style");
+string DASH_STYLE_UNIT ("dash-style-unit");
+string ARROW_BEGIN ("arrow-begin");
+string ARROW_END ("arrow-end");
+string ARROW_LENGTH ("arrow-length");
+string ARROW_HEIGHT ("arrow-height");
+string FILL_COLOR ("fill-color");
+string FILL_STYLE ("fill-style");
+string TEXT_AT_HALIGN ("text-at-halign");
+string TEXT_AT_VALIGN ("text-at-valign");
+string TEXT_AT_MARGIN ("text-at-margin");
+
 /******************************************************************************
 * Environment variables for preamble mode
 ******************************************************************************/
