@@ -4,10 +4,9 @@
 * DESCRIPTION: Properties of characters and strings
 * COPYRIGHT  : (C) 1999  Joris van der Hoeven
 *******************************************************************************
-* This software falls under the GNU general public license and comes WITHOUT
-* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
-* If you don't have this file, write to the Free Software Foundation, Inc.,
-* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+* This software falls under the GNU general public license version 3 or later.
+* It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
+* in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ******************************************************************************/
 
 #include "analyze.hpp"
@@ -694,6 +693,7 @@ tm_correct (string s) {
 
 void
 tm_char_forwards (string s, int& pos) {
+  ASSERT (pos >= 0 && pos <= N(s), "out of range");
   int n= N(s);
   if (pos == n);
   else if (s[pos] != '<') pos++;
@@ -705,6 +705,7 @@ tm_char_forwards (string s, int& pos) {
 
 void
 tm_char_backwards (string s, int& pos) {
+  ASSERT (pos >= 0 && pos <= N(s), "out of range");
   if (pos == 0);
   else if (s[pos-1] != '>') pos--;
   else {
@@ -713,6 +714,68 @@ tm_char_backwards (string s, int& pos) {
   }
 }
 
+int
+tm_char_next (string s, int pos) {
+  tm_char_forwards (s, pos);
+  return pos;
+}
+
+int
+tm_char_previous (string s, int pos) {
+  tm_char_backwards (s, pos);
+  return pos;
+}
+
+string
+tm_forward_access (string s, int k) {
+  int pos= 0;
+  for (int i=0; i<k; i++)
+    tm_char_forwards (s, pos);
+  int start= pos;
+  tm_char_forwards (s, pos);
+  return s (start, pos);
+}
+
+string
+tm_backward_access (string s, int k) {
+  int pos= N(s);
+  for (int i=0; i<k; i++)
+    tm_char_backwards (s, pos);
+  int end= pos;
+  tm_char_backwards (s, pos);
+  return s (pos, end);
+}
+
+int
+tm_string_length (string s) {
+  int i= 0, pos= 0;
+  while (pos < N(s)) {
+    tm_char_forwards (s, pos);
+    i++;
+  }
+  return i;
+}
+
+array<string>
+tm_tokenize (string s) {
+  array<string> r;
+  int pos= 0;
+  while (pos < N(s)) {
+    int start= pos;
+    tm_char_forwards (s, pos);
+    r << s (start, pos);
+  }
+  return r;
+}
+
+string
+tm_recompose (array<string> a) {
+  string r;
+  for (int i=0; i<N(a); i++)
+    r << a[i];
+  return r;
+}
+
 /******************************************************************************
 * Quoting
 ******************************************************************************/
@@ -769,7 +832,7 @@ raw_unquote (string s) {
 
 string
 escape_sh (string s) {
-#ifdef OS_WIN32
+#if defined (__MINGW__) || defined (__MINGW32__) || defined (OS_WIN32)
   return raw_quote (s);
 #else
   int i, n= N(s);
@@ -1028,7 +1091,7 @@ parse (string s, int& pos, SI& ret) {
 void
 parse (string s, int& pos, SI*& a, int len) {
   int i;
-  a= new int[len];
+  a= tm_new_array<int> (len);
   for (i=0; i<len; i++) parse (s, pos, a[i]);
 }
 
@@ -1053,6 +1116,11 @@ search_forwards (string s, string in) {
   return search_forwards (s, 0, in);
 }
 
+bool
+occurs (string what, string in) {
+  return search_forwards (what, 0, in) >= 0;
+}
+
 int
 search_backwards (string s, int pos, string in) {
   while (pos >= 0) {
@@ -1115,6 +1183,46 @@ match_wildcard (string s, string w) {
   return match_wildcard (s, 0, w, 0);
 }
 
+array<string>
+tokenize (string s, string sep) {
+  int start=0;
+  array<string> a;
+  for (int i=0; i<N(s); )
+    if (test (s, i, sep)) {
+      a << s (start, i);
+      i += N(sep);
+      start= i;
+    }
+    else i++;
+  return a;
+}
+
+string
+recompose (array<string> a, string sep) {
+  string r;
+  for (int i=0; i<N(a); i++) {
+    if (i != 0) r << sep;
+    r << a[i];
+  }
+  return r;
+}
+
+string
+trim_spaces (string s) {
+  int start, end;
+  for (start=0; start<N(s) && is_space (s[start]); start++);
+  for (end=N(s); end>start && is_space (s[end]); end--);
+  return s (start, end);
+}
+
+array<string>
+trim_spaces (array<string> a) {
+  array<string> b (N(a));
+  for (int i=0; i<N(a); i++)
+    b[i]= trim_spaces (a[i]);
+  return b;
+}
+
 /******************************************************************************
 * Computations with completions
 ******************************************************************************/
