@@ -24,10 +24,18 @@
 #include "../../Style/Memorizer/clean_copy.hpp"
 #endif
 
+#ifdef PDF_RENDERER
+#include "Pdf/pdf_renderer.hpp"
+#endif
+
 #ifdef USE_GS
 #include "Ghostscript/gs_utilities.hpp"
 #endif
 
+#ifdef QTTEXMACS
+#include "Qt/qt_utilities.hpp"
+#endif
+
 /******************************************************************************
 * Constructors and destructor
 ******************************************************************************/
@@ -157,10 +165,15 @@ void
 edit_main_rep::print (url name, bool conform, int first, int last) {
   bool pdf= (suffix (name) == "pdf");
   url orig= resolve (name, "");
+
+#ifndef PDF_RENDERER
 #ifdef USE_GS
   if (pdf) name= url_temp (".ps");
 #endif
-
+#else
+  (void) pdf;  // avoid annoying warning
+#endif
+  
   string medium = env->get_string (PAGE_MEDIUM);
   if (conform && (medium != "paper")) conform= false;
   // FIXME: better command for conform printing
@@ -204,8 +217,15 @@ edit_main_rep::print (url name, bool conform, int first, int last) {
   // Print pages
 
   int i;
-  renderer ren=
-    printer (name, dpi, pages, page_type, landsc, w/cm, h/cm);
+  
+#ifdef PDF_RENDERER
+  renderer ren = (pdf ? 
+    pdf_renderer (name, dpi, pages, page_type, landsc, w/cm, h/cm) :
+    printer (name, dpi, pages, page_type, landsc, w/cm, h/cm) );
+#else
+  renderer ren = printer (name, dpi, pages, page_type, landsc, w/cm, h/cm);
+#endif
+  
   for (i=start; i<end; i++) {
     tree bg= env->read (BG_COLOR);
     ren->set_background_pattern (bg);
@@ -220,12 +240,14 @@ edit_main_rep::print (url name, bool conform, int first, int last) {
   }
   tm_delete (ren);
 
+#ifndef PDF_RENDERER
 #ifdef USE_GS
   if (pdf) {
     gs_to_pdf (name, orig);
     ::remove (name);
   }
 #endif
+#endif
 }
 
 void
@@ -233,14 +255,37 @@ edit_main_rep::print_to_file (url name, string first, string last) {
   print (name, false, as_int (first), as_int (last));
 }
 
+
 void
 edit_main_rep::print_buffer (string first, string last) {
-  url temp= url_temp (".ps");
-  print (temp, false, as_int (first), as_int (last));
-  system (printing_cmd, temp);
-  ::remove (temp);
+  url target= url_temp (".ps"); 
+  print (target, false, as_int (first), as_int (last));
+  system (printing_cmd, target);  // Send the document to the printer
+  ::remove (target);
 }
 
+#ifdef THISISTHEPREVIOUSCODE_IJUSTLEFTITHEREINCASE
+void
+edit_main_rep::print_buffer (string first, string last) {
+  // in Qt this is the main entry point to the printing subsystem.
+  // the other routines (print_to_file, ...) are overriden since all fine tuning 
+  // is made here via the Qt print dialog
+  bool to_file, landscape;
+  url name = url_none();
+  string printer;
+  string paper_type;
+  if (qt_print (to_file, landscape, printer, name, first, last, paper_type)) {
+      if (!to_file) name = url_temp (".ps");
+      print (name, false, as_int (first), as_int (last));
+      if (!to_file) {
+        string cmd = printing_cmd * " -P" * printer;
+        system (cmd, name);  
+        ::remove (name);
+      }
+  }
+}
+#endif
+
 void
 edit_main_rep::export_ps (url name, string first, string last) {
   print (name, true, as_int (first), as_int (last));
@@ -257,9 +302,12 @@ edit_main_rep::print_snippet (url name, tree t) {
   if (b->x4 - b->x3 >= 5*PIXEL && b->y4 - b->y3 >= 5*PIXEL) {
     if (ps) make_eps (name, b, dpi);
     else {
-      url temp= url_temp ("eps");
+      url temp= url_temp (".eps");
       make_eps (temp, b, dpi);
+      ::remove (name);
       system ("convert", temp, name);
+      if (!exists (name))
+        cout << "TeXmacs] warning, failed to create image " << name << "\n";
       ::remove (temp);
     }
   }
@@ -276,7 +324,7 @@ void
 edit_main_rep::footer_eval (string s) {
   // s= unslash (s); // FIXME: dirty fix; should not be necessary
   string r= object_to_string (eval (s));
-  set_message (r, "evaluate expression");
+  set_message (verbatim (r), "evaluate expression");
 }
 
 tree
@@ -310,6 +358,11 @@ edit_main_rep::the_path () {
   return copy (tp);
 }
 
+path
+edit_main_rep::the_shifted_path () {
+  return shift (et, tp, 1);
+}
+
 /******************************************************************************
 * Miscellaneous
 ******************************************************************************/
