@@ -87,19 +87,31 @@ edit_interface_rep::try_shortcut (string comb) {
   //cout << "Try " << comb << " -> " << shorth << ", " << help
   //<< "; " << sh_mark << ", " << status << "\n";
   if (status != 0) {
-    if (sh_mark != 0 && !mark_cancel (sh_mark)) {
-      sh_mark= 0;
-      return false;
+    if (status >= 3) {
+      interrupt_shortcut ();
+      status -= 3;
+      if (status == 0) return false;
+    }
+    else {
+      if (sh_mark != 0 && !mark_cancel (sh_mark)) {
+        sh_mark= 0;
+        return false;
+      }
     }
     sh_s= comb;
     sh_mark= new_marker ();
     mark_start (sh_mark);
-    string rew= sv->kbd_post_rewrite (sh_s);
+    archive_state ();
+    string rew_s= sv->kbd_post_rewrite (sh_s);
+    tree rew= sv->kbd_system_rewrite (rew_s);
     if (N(help)>0) set_message (help, rew);
-    string rhs= (shorth == rew? string (""): shorth);
+    tree rhs= (shorth == rew_s? tree (""): sv->kbd_system_rewrite (shorth));
+    //cout << "Shortcut: " << sh_s << " -> " << rew << "\n";
     if ((search_forwards (" ", comb) >= 0 && comb != " ") ||
 	(search_forwards ("-", comb) >= 0 && comb != "-"))
-      call ("set-temporary-message", "keyboard shortcut: " * rew, rhs, 3000);
+      call ("set-temporary-message",
+	    tree (CONCAT, "keyboard shortcut: ", rew), rhs,
+	    shorth == ""? 1: 3000);
     if ((status & 1) == 1) cmd ();
     else if (N(shorth) > 0) insert_tree (shorth);
     //cout << "Mark= " << sh_mark << "\n";
@@ -109,40 +121,36 @@ edit_interface_rep::try_shortcut (string comb) {
   return false;
 }
 
-static string
-simplify_key_press (string key) {
-  while ((N(key) >= 5) && (key(0,3) == "Mod") && (key[4] == '-') &&
-	 (key[3] >= '1') && (key[3] <= '5')) key= key (5, N(key));
-  if (key == "space") key= " ";
-  return key;
-}
-
 void
-edit_interface_rep::key_press (string key) {
-  if (contains_unicode_char (key)) {
-    if (input_mode == INPUT_NORMAL) insert_tree (key);
-    return;
+edit_interface_rep::key_press (string gkey) {
+  string zero= "a"; zero[0]= '\0';
+  string key= replace (gkey, "<#0>", zero);
+  if (pre_edit_mark != 0) {
+    ASSERT (sh_mark == 0, "invalid shortcut during pre-edit");
+    mark_cancel (pre_edit_mark);
+    pre_edit_s= "";
+    pre_edit_mark= 0;
   }
-
-  set_message ("", "");
-  if (input_mode != INPUT_NORMAL)
-    key= simplify_key_press (key);
-  switch (input_mode) {
-  case INPUT_NORMAL:
-    break;
-  case INPUT_SEARCH:
-    if (search_keypress (key)) return;
-    break;
-  case INPUT_REPLACE:
-    if (replace_keypress (key)) return;
-    break;
-  case INPUT_SPELL:
-    if (spell_keypress (key)) return;
-    break;
-  case INPUT_COMPLETE:
-    if (complete_keypress (key)) return;
-    set_input_normal ();
-    break;
+  if (starts (key, "pre-edit:") ) {
+    interrupt_shortcut ();
+    string s= key (9, N(key));
+    if (s == "") return;
+    int i, n= N(s), pos= N(s);
+    for (i=0; i<n; i++)
+      if (s[i] == ':' && is_int (s (0, i))) {
+	int k= as_int (s (0, i));
+	s= s (i+1, n);
+	pos= 0;
+	for (int j=0; j<k && pos<N(s); j++)
+	  tm_char_forwards (s, pos);
+	break;
+      }
+    pre_edit_s= s;
+    pre_edit_mark= new_marker ();
+    mark_start (pre_edit_mark);
+    archive_state ();
+    insert_tree (compound ("pre-edit", s), path (0, pos));
+    return;
   }
 
   string new_sh= N(sh_s)==0? key: sh_s * " " * key;
@@ -153,13 +161,20 @@ edit_interface_rep::key_press (string key) {
   }
 
   string rew= sv->kbd_post_rewrite (key);
-  if (N(rew)==1) {
+  if (N(rew) == 1) {
     int i ((unsigned char) rew[0]);
-    if ((((i >= 32) && (i <= 127)) || ((i >= 128) && (i <= 255))) &&
-	!inside_active_graphics ())
-      insert_tree (rew);
+    if ((i >= 32 && i <= 127) || (i >= 128 && i <= 255))
+      if (!inside_active_graphics ()) {
+        archive_state ();
+	insert_tree (rew);
+      }
     interrupt_shortcut ();
   }
+  else if (contains_unicode_char (rew)) {
+    archive_state ();
+    insert_tree (key);
+    interrupt_shortcut ();    
+  }
 }
 
 void
@@ -169,13 +184,28 @@ edit_interface_rep::emulate_keyboard (string keys, string action) {
     int i;
     for (i=1; i<N(s); i++)
       if (s[i]==' ') break;
-    key_press (s (0, i));
+    call ("keyboard-press", object (s (0, i)), object ((double) 0));
     if (i<N(s)) i++;
     s= s (i, N(s));
   }
-  if (N(action) != 0)
-    set_message ("You can also obtain#" * action *
-		 "#by typing#" * keys, action);
+  if (N (action) != 0)
+    set_message (concat ("You can also obtain ", action, " by typing ", keys),
+		 action);
+}
+
+/******************************************************************************
+* Retrieving keyboard shortcuts
+******************************************************************************/
+
+tree
+edit_interface_rep::kbd (string s) {
+  return sv->kbd_system_rewrite (s);
+}
+
+tree
+edit_interface_rep::kbd_shortcut (string cmd) {
+  string s= as_string (eval ("(kbd-find-inv-binding '" * cmd * ")"));
+  return kbd (s);
 }
 
 /******************************************************************************
@@ -184,22 +214,38 @@ edit_interface_rep::emulate_keyboard (string keys, string action) {
 
 void
 edit_interface_rep::handle_keypress (string key, time_t t) {
+  if (DEBUG_KEYBOARD)
+    cout << "Keyboard] Pressed " << key << " at " << t << "\n";
   //time_t t1= texmacs_time ();
   if (is_nil (eb)) apply_changes ();
   start_editing ();
-  key_press (key); (void) t;
+  string zero= "a"; zero[0]= '\0';
+  string gkey= replace (key, zero, "<#0>");
+  call ("keyboard-press", object (gkey), object ((double) t));
   notify_change (THE_DECORATIONS);
   end_editing ();
   //time_t t2= texmacs_time ();
   //if (t2 - t1 >= 10) cout << "handle_keypress took " << t2-t1 << "ms\n";
 }
 
+void drag_left_reset ();
+void drag_right_reset ();
+
 void
 edit_interface_rep::handle_keyboard_focus (bool has_focus, time_t t) {
+  if (DEBUG_KEYBOARD) {
+    if (has_focus) cout << "Keyboard] Got focus at " << t << "\n";
+    else cout << "Keyboard] Lost focus at " << t << "\n";
+  }
+  if (got_focus != has_focus) {
+    drag_left_reset ();
+    drag_right_reset ();
+  }
   got_focus= has_focus; (void) t;
   notify_change (THE_FOCUS);
   if (got_focus) {
     focus_on_this_editor ();
     notify_change (THE_DECORATIONS);
   }
+  call ("keyboard-focus", object (has_focus), object ((double) t));
 }
