@@ -25,7 +25,7 @@ edit_text_rep::get_deletion_point (
   p= tp;
   if (forward) {
     //cout << HRULE;
-    if ((rp < p) && (N(p-rp) >= 2) &&
+    if ((rp < p) && (N (p / rp) >= 2) &&
 	is_concat (subtree (et, path_up (p, 2))) &&
 	(last_item (p) == right_index (subtree (et, path_up (p)))) &&
 	(last_item (path_up (p)) < (N (subtree (et, path_up (p, 2))) - 1)))
@@ -164,6 +164,13 @@ edit_text_rep::remove_text (bool forward) {
     case VSPACE:
     case SPACE:
     case HTAB:
+      back_monolithic (p);
+      return;
+    case AROUND:
+    case VAR_AROUND:
+    case BIG_AROUND:
+      back_around (t, p, forward);
+      return;
     case LEFT:
     case MID:
     case RIGHT:
@@ -199,13 +206,22 @@ edit_text_rep::remove_text (bool forward) {
       else back_general (p, forward);
       return;
     default:
-      back_general (p, forward);
+      if (is_compound (t, "separating-space", 1)) back_monolithic (p);
+      else back_general (p, forward);
       break;
     }
 
   // deletion depends on children u
   if (last == (forward? rix: 0)) {
     switch (L (u)) {
+    case AROUND:
+    case VAR_AROUND:
+    case BIG_AROUND:
+      back_in_around (u, p, forward);
+      return;
+    case LONG_ARROW:
+      back_in_long_arrow (u, p, forward);
+      return;
     case WIDE:
     case VAR_WIDE:
       back_in_wide (u, p, forward);
@@ -227,7 +243,10 @@ edit_text_rep::remove_text (bool forward) {
       back_in_with (u, p, forward);
       return;
     default:
-      back_in_general (u, p, forward);
+      if (is_graphical_text (u))
+        back_in_text_at (u, p, forward);
+      else
+        back_in_general (u, p, forward);
       break;
     }
   }
@@ -265,12 +284,12 @@ edit_text_rep::remove_structure (bool forward) {
     while (true) {
       if (forward) {
 	pos= start;
-	(void) lan->advance (s, pos);
+	(void) lan->advance (t, pos);
 	if (pos <= last) break;
       }
       else {
 	int pos= max (start-1, 0);
-	(void) lan->advance (s, pos);
+	(void) lan->advance (t, pos);
 	if (pos < last) break;
       }
       end= pos;
@@ -323,9 +342,14 @@ edit_text_rep::remove_structure_upwards () {
   int last= last_item (p);
   p= path_up (p);
   tree st= subtree (et, p);
+  if (is_func (st, AROUND, 3) ||
+      is_func (st, VAR_AROUND, 3) ||
+      is_func (st, BIG_AROUND, 2))
+    pre_remove_around (p);
   bool recurse=
     is_func (st, TFORMAT) || is_func (st, TABLE) ||
     is_func (st, ROW) || is_func (st, CELL) ||
+    is_compound (st, "shown") ||
     drd->var_without_border (L(st));
   remove (p * (last+1), N(st)-(last+1));
   remove (p * 0, last);
