@@ -58,6 +58,7 @@ protected:
 
   // Main routines for manipulating the entire table
   void table_get_extents (path fp, int& nr_rows, int& nr_cols);
+  void table_set_extents (path fp, int nr_rows, int nr_cols);
   void table_get_limits (path fp, int& i1, int& j1, int& i2, int& j2);
   void table_insert (path fp, int row, int col, int nr_rows, int nr_cols);
   void table_remove (path fp, int row, int col, int nr_rows, int nr_cols);
@@ -94,11 +95,12 @@ public:
   void   table_remove_column (bool forward, bool flag= false);
   int    table_nr_rows ();
   int    table_nr_columns ();
+  void   table_set_extents (int rows, int cols);
   int    table_which_row ();
   int    table_which_column ();
   path   table_search_cell (int row, int col);
   void   table_go_to (int row, int col);
-  void   table_set_format (string var, string val);
+  void   table_set_format (string var, tree val);
   string table_get_format (string var);
   void   table_del_format (string var);
   void   table_format_center ();
@@ -107,7 +109,7 @@ public:
   void   table_correct_block_content ();
   void   set_cell_mode (string mode);
   string get_cell_mode ();
-  void   cell_set_format (string var, string val);
+  void   cell_set_format (string var, tree val);
   string cell_get_format (string var);
   void   cell_del_format (string var);
 
