@@ -42,6 +42,8 @@
 class tm_buffer_rep;
 typedef tm_buffer_rep* tm_buffer;
 class server_rep;
+class modification;
+extern bool enable_fastenv;
 
 class editor_rep: public simple_widget_rep {
 public:
@@ -56,6 +58,7 @@ protected:
   path        tp;   // path of cursor in tree
 #ifdef EXPERIMENTAL
   environment ste;  // environment for style rewriting
+  tree        cct;  // clean copy of the document tree
   memorizer   mem;  // style converted document tree
 #endif
 
@@ -90,21 +93,27 @@ protected:
   virtual void   typeset (SI& x1, SI& y1, SI& x2, SI& y2) = 0;
 
   /* protected subroutines for deletion of content */
+  virtual void back_around (tree t, path p, bool forward) = 0;
   virtual void back_prime (tree t, path p, bool forward) = 0;
+  virtual void back_in_around (tree t, path p, bool forward) = 0;
+  virtual void back_in_long_arrow (tree t, path p, bool forward) = 0;
   virtual void back_in_wide (tree t, path p, bool forward) = 0;
   virtual void back_in_tree (tree t, path p, bool forward) = 0;
+  virtual void pre_remove_around (path p) = 0;
   virtual void back_table (path p, bool forward) = 0;
   virtual void back_in_table (tree t, path p, bool forward) = 0;
   virtual void back_monolithic (path p) = 0;
   virtual void back_general (path p, bool forward) = 0;
   virtual void back_in_with (tree t, path p, bool forward) = 0;
   virtual void back_in_general (tree t, path p, bool forward) = 0;
+  virtual void back_in_text_at (tree t, path p, bool forward) = 0;
 
   /* other protected subroutines */
+  virtual tree kbd (string s) = 0;
+  virtual tree kbd_shortcut (string s) = 0;
   virtual path tree_path (path sp, SI x, SI y, SI delta) = 0;
   virtual void apply_changes () = 0;
   virtual void animate () = 0;
-  virtual void correct_concat (path p, int done=0) = 0;
   virtual path search_format (int& row, int& col) = 0;
   virtual void table_bound (path fp, int& i1, int& j1, int& i2, int& j2) = 0;
   virtual tree table_get_subtable (path p, int i1, int j1, int i2, int j2) = 0;
@@ -137,25 +146,27 @@ public:
   virtual bool in_search_mode () = 0;
   virtual bool in_replace_mode () = 0;
   virtual bool in_spell_mode () = 0;
+  virtual void interrupt_shortcut () = 0;
   virtual bool kbd_get_command (string cmd_s, string& help, command& cmd) = 0;
   virtual void key_press (string key) = 0;
   virtual void emulate_keyboard (string keys, string action= "") = 0;
   virtual bool complete_try () = 0;
   virtual void complete_start (string prefix, array<string> compls) = 0;
   virtual bool complete_keypress (string key) = 0;
-  virtual bool session_complete_try (tree t) = 0;
+  virtual string session_complete_command (tree t) = 0;
+  virtual void custom_complete (tree t) = 0;
   virtual void mouse_any (string s, SI x, SI y, int mods, time_t t) = 0;
   virtual void mouse_click (SI x, SI y) = 0;
   virtual bool mouse_extra_click (SI x, SI y) = 0;
   virtual void mouse_drag (SI x, SI y) = 0;
-  virtual void mouse_select (SI x, SI y, int mods) = 0;
+  virtual void mouse_select (SI x, SI y, int mods, bool drag) = 0;
   virtual void mouse_paste (SI x, SI y) = 0;
   virtual void mouse_adjust (SI x, SI y) = 0;
   virtual void mouse_scroll (SI x, SI y, bool up) = 0;
   virtual cursor get_cursor () = 0;
   virtual void set_pointer (string name) = 0;
   virtual void set_pointer (string curs_name, string mask_name) = 0;
-  virtual void set_message (string l, string r= "", bool temp= false) = 0;
+  virtual void set_message (tree l, tree r= "", bool temp= false) = 0;
   virtual void recall_message () = 0;
 
   /* public routines from edit_cursor */
@@ -237,6 +248,8 @@ public:
   virtual language get_env_language () = 0;
   virtual tree     exec_texmacs (tree t, path p) = 0;
   virtual tree     exec_texmacs (tree t) = 0;
+  virtual tree     exec_verbatim (tree t, path p) = 0;
+  virtual tree     exec_verbatim (tree t) = 0;
   virtual tree     exec_html (tree t, path p) = 0;
   virtual tree     exec_html (tree t) = 0;
   virtual tree     exec_latex (tree t, path p) = 0;
@@ -262,14 +275,28 @@ public:
   virtual void notify_assign_node (path p, tree_label op) = 0;
   virtual void notify_insert_node (path p, tree t) = 0;
   virtual void notify_remove_node (path p) = 0;
+  virtual void notify_set_cursor (path p, tree data) = 0;
   virtual void post_notify (path p) = 0;
-  virtual void mark_undo_blocks () = 0;
-  virtual void remove_undo_mark () = 0;
+  virtual void clear_undo_history () = 0;
+  virtual double this_author () = 0;
+  virtual void archive_state () = 0;
+  virtual void start_editing () = 0;
+  virtual void end_editing () = 0;
+  virtual void start_slave (double a) = 0;
+  virtual void mark_start (double a) = 0;
+  virtual bool mark_cancel (double a) = 0;
+  virtual void mark_end (double a) = 0;
   virtual void add_undo_mark () = 0;
+  virtual void remove_undo_mark () = 0;
+  virtual int  undo_possibilities () = 0;
   virtual void unredoable_undo () = 0;
-  virtual void forget_undo () = 0;
-  virtual void undo () = 0;
-  virtual void redo () = 0;
+  virtual void undo (int i=0) = 0;
+  virtual int  redo_possibilities () = 0;
+  virtual void redo (int i=0) = 0;
+  virtual void require_save () = 0;
+  virtual void notify_save (bool real_save= true) = 0;
+  virtual bool need_save (bool real_save= true) = 0;
+  virtual void show_history () = 0;
   virtual observer position_new (path p) = 0;
   virtual void position_delete (observer o) = 0;
   virtual void position_set (observer o, path p) = 0;
@@ -277,6 +304,7 @@ public:
 
   /* public routines from edit_text */
   virtual void correct (path p) = 0;
+  virtual void correct_concat (path p, int done=0) = 0;
   virtual bool insert_return () = 0;
   virtual void remove_return (path p) = 0;
   virtual void insert_tree (tree t, path p_in_t) = 0;
@@ -296,15 +324,11 @@ public:
   virtual void make_vspace_after (string s) = 0;
   virtual void make_vspace_after (string smin, string sdef, string smax) = 0;
   virtual void make_htab (string spc) = 0;
-  virtual void make_move (string x, string y) = 0;
-  virtual void make_resize (string x1, string y1, string x2, string y2) = 0;
-  virtual void make_postscript (string file_name, bool link,
-				string w, string h,
-				string x1, string y1,
-				string x2, string y2) = 0;
+  virtual void make_image (string file_name, bool link,
+			   string w, string h, string x, string y) = 0;
 
   /* public routines from edit_math */
-  virtual void make_group () = 0;
+  virtual void make_rigid () = 0;
   virtual void make_lprime (string s) = 0;
   virtual void make_rprime (string s) = 0;
   virtual void make_below () = 0;
@@ -329,11 +353,12 @@ public:
   virtual void   table_remove_column (bool forward, bool flag= false) = 0;
   virtual int    table_nr_rows () = 0;
   virtual int    table_nr_columns () = 0;
+  virtual void   table_set_extents (int rows, int cols) = 0;
   virtual int    table_which_row () = 0;
   virtual int    table_which_column () = 0;
   virtual path   table_search_cell (int row, int col) = 0;
   virtual void   table_go_to (int row, int col) = 0;
-  virtual void   table_set_format (string var, string val) = 0;
+  virtual void   table_set_format (string var, tree val) = 0;
   virtual string table_get_format (string var) = 0;
   virtual void   table_del_format (string var= "") = 0;
   virtual void   table_format_center () = 0;
@@ -342,7 +367,7 @@ public:
   virtual void   table_correct_block_content () = 0;
   virtual void   set_cell_mode (string mode) = 0;
   virtual string get_cell_mode () = 0;
-  virtual void   cell_set_format (string var, string val) = 0;
+  virtual void   cell_set_format (string var, tree val) = 0;
   virtual string cell_get_format (string var) = 0;
   virtual void   cell_del_format (string var= "") = 0;
   virtual void   table_test () = 0;
@@ -355,6 +380,7 @@ public:
   virtual void go_to_argument (path p, bool start_flag) = 0;
   virtual void insert_argument (path p, bool forward) = 0;
   virtual void insert_argument (bool forward) = 0;
+  virtual void remove_empty_argument (path p, bool forward) = 0;
   virtual void remove_argument (path p, bool forward) = 0;
   virtual void remove_argument (bool forward) = 0;
   virtual void make_with (string var, string val) = 0;
@@ -366,7 +392,6 @@ public:
   virtual bool activate_latex () = 0;
   virtual void activate_hybrid (bool with_args_hint) = 0;
   virtual void activate_symbol () = 0;
-  virtual void activate_compound () = 0;
   virtual bool make_return_before () = 0;
   virtual bool make_return_after () = 0;
   virtual void temp_proof_fix () = 0;
@@ -380,6 +405,10 @@ public:
   virtual bool get_save_aux () = 0;
 
   /* public routines from edit_select */
+  virtual path semantic_root (path p) = 0;
+  virtual bool semantic_active (path p) = 0;
+  virtual bool semantic_select (path p, path& q1, path& q2, int mode) = 0;
+
   virtual void select (path p) = 0;
   virtual void select (path start, path end) = 0;
   virtual void select_all () = 0;
@@ -394,22 +423,26 @@ public:
 
   virtual bool selection_active_any () = 0;
   virtual bool selection_active_normal () = 0;
-  virtual bool selection_active_table () = 0;
+  virtual bool selection_active_table (bool strict= true) = 0;
   virtual bool selection_active_small () = 0;
   virtual bool selection_active_enlarging () = 0;
 
   virtual void selection_raw_set (string key, tree t) = 0;
   virtual tree selection_raw_get (string key) = 0;
+  virtual void selection_correct (path i1, path i2, path& o1, path& o2) = 0;
   virtual path selection_get_subtable (int& i1, int& j1, int& i2, int& j2) = 0;
   virtual void selection_get (selection& sel) = 0;
   virtual void selection_get (path& start, path& end) = 0;
   virtual path selection_get_start () = 0;
   virtual path selection_get_end () = 0;
   virtual path selection_get_path () = 0;
+  virtual path selection_get_cursor_path () = 0;
+  virtual tree selection_get_env_value (string var) = 0;
   virtual void selection_set (string key, tree t, bool persistant= false) = 0;
   virtual void selection_set (tree t) = 0;
   virtual void selection_set_start (path p= path()) = 0;
   virtual void selection_set_end (path p= path()) = 0;
+  virtual void selection_set_paths (path start, path end) = 0;
   virtual void selection_copy (string key= "primary") = 0;
   virtual void selection_paste (string key= "primary") = 0;
   virtual void selection_clear (string key= "primary") = 0;
@@ -425,6 +458,10 @@ public:
   virtual void selection_move () = 0;
   virtual void cut (path p) = 0;
   virtual void cut (path start, path end) = 0;
+  virtual path manual_focus_get () = 0;
+  virtual void manual_focus_set (path p, bool force= true) = 0;
+  virtual void manual_focus_release () = 0;
+  virtual path focus_get (bool skip_flag= true) = 0;
 
   /* public routines from edit_replace */
   virtual bool inside (string what) = 0;
@@ -471,6 +508,7 @@ public:
   virtual tree the_buffer () = 0;
   virtual tree the_subtree (path p) = 0;
   virtual path the_path () = 0;
+  virtual path the_shifted_path () = 0;
   virtual path the_buffer_path () = 0;
   virtual void show_tree () = 0;
   virtual void show_env () = 0;
@@ -485,6 +523,10 @@ public:
   friend class tm_data_rep;
   friend class tm_server_rep;
   friend class server_command_rep;
+  friend void   edit_announce (editor_rep* ed, modification mod);
+  friend void   edit_done (editor_rep* ed, modification mod);
+  friend string get_editor_status_report ();
+  friend void   tm_failure (const char* msg);
 };
 
 class editor {
