@@ -12,6 +12,7 @@
 #include <tm_configure.hpp>
 #include "Glue/glue.hpp"
 #include "server.hpp"
+#include "tm_window.hpp"
 #include "boot.hpp"
 #include "connect.hpp"
 #include "convert.hpp"
@@ -22,6 +23,9 @@
 #include "client_server.hpp"
 #include "analyze.hpp"
 #include "tree_traverse.hpp"
+#include "tree_analyze.hpp"
+#include "tree_correct.hpp"
+#include "tree_modify.hpp"
 #include "tm_frame.hpp"
 #include "Concat/concater.hpp"
 #include "converter.hpp"
@@ -30,10 +34,11 @@
 #include "Freetype/free_type.hpp"
 #include "Freetype/tt_file.hpp"
 #include "Bibtex/bibtex.hpp"
+#include "Bibtex/bibtex_functions.hpp"
 #include "link.hpp"
 #include "dictionary.hpp"
 #include "patch.hpp"
-#include "grammar.hpp"
+#include "packrat.hpp"
 #include <string.h>
 #include <libguile.h>
 
@@ -73,48 +78,20 @@ scheme_dialect () {
 
 string
 texmacs_version (string which) {
-  if (which == "tgz") return TEXMACS_TGZ;
-  if (which == "rpm") return TEXMACS_RPM;
-  if (which == "stgz") return TEXMACS_STGZ;
-  if (which == "srpm") return TEXMACS_SRPM;
-  if (which == "win") return TEXMACS_WIN;
+  if (which == "tgz") return TM_DEVEL;
+  if (which == "rpm") return TM_DEVEL_RELEASE;
+  if (which == "stgz") return TM_STABLE;
+  if (which == "srpm") return TM_STABLE_RELEASE;
+  if (which == "devel") return TM_DEVEL;
+  if (which == "stable") return TM_STABLE;
+  if (which == "devel-release") return TM_DEVEL_RELEASE;
+  if (which == "stable-release") return TM_STABLE_RELEASE;
   return TEXMACS_VERSION;
 }
 
-bool
-gui_is_x () {
-#ifdef QTTEXMACS
-  return false;
-#else
-  return true;
-#endif
-}
-
-bool
-gui_is_qt () {
-#ifdef QTTEXMACS
-  return true;
-#else
-  return false;
-#endif
-}
-
-bool
-os_win32 () {
-#if defined (OS_WIN32)
-  return true;
-#else
-  return false;
-#endif
-}
-
-bool
-os_mingw () {
-#if defined (__MINGW__) || defined (__MINGW32__)
-  return true;
-#else
-  return false;
-#endif
+void
+set_fast_environments (bool b) {
+  enable_fastenv= b;
 }
 
 void
@@ -145,6 +122,26 @@ tm_errput (string s) {
   SCM_SETCDR (smob, (SCM) (data));
 #endif
 
+void
+cpp_error () {
+  //char *np= 0; *np= 1;
+  FAILED ("an error occurred");
+}
+
+/******************************************************************************
+* Redirections
+******************************************************************************/
+
+void
+cout_buffer () {
+  cout.buffer ();
+}
+
+string
+cout_unbuffer () {
+  return cout.unbuffer ();
+}
+
 /******************************************************************************
 * Direct access to scheme objects
 ******************************************************************************/
@@ -206,13 +203,13 @@ scm_to_int (SCM i) {
 #define SCM_ASSERT_DOUBLE(i,arg,rout) \
   SCM_ASSERT (SCM_REALP (i), i, arg, rout);
 
-static SCM
+SCM
 double_to_scm (double i) {
   return scm_double2scm (i);
 }
 
 #ifndef GUILE_C
-static double
+double
 scm_to_double (SCM i) {
   return scm_scm2double (i);
 }
@@ -392,8 +389,8 @@ tree_active (tree t) {
 
 tree
 tree_child_insert (tree t, int pos, tree x) {
-  cout << "t= " << t << "\n";
-  cout << "x= " << x << "\n";
+  //cout << "t= " << t << "\n";
+  //cout << "x= " << x << "\n";
   int i, n= N(t);
   tree r (t, n+1);
   for (i=0; i<pos; i++) r[i]= t[i];
