@@ -21,6 +21,7 @@
 #define PATTERN WIN_PATTERN
 #define STRING WIN_STRING
 #define GROUP WIN_GROUP
+#define mouse_event win_mouse_event
 #ifdef IN
 #define MY_IN IN
 #undef IN
@@ -38,6 +39,7 @@
 #undef ERROR
 #undef PATTERN
 #undef GROUP
+#undef mouse_event
 #undef IN
 #undef OUT
 #undef MENU_EVENT
@@ -73,6 +75,7 @@ struct object {
   object ();
   object (bool b);
   object (int i);
+  object (double x);
   object (const char* s);
   object (string s);
   object (tree t);
@@ -107,6 +110,7 @@ bool is_null (object obj);
 bool is_list (object obj);
 bool is_bool (object obj);
 bool is_int (object obj);
+bool is_double (object obj);
 bool is_string (object obj);
 bool is_symbol (object obj);
 bool is_tree (object obj);
@@ -115,6 +119,7 @@ bool is_url (object obj);
 
 bool as_bool (object obj);
 int as_int (object obj);
+double as_double (object obj);
 string as_string (object obj);
 string as_symbol (object obj);
 tree as_tree (object obj);
@@ -122,6 +127,7 @@ scheme_tree as_scheme_tree (object obj);
 list<string> as_list_string (object obj);
 list<tree> as_list_tree (object obj);
 path as_path (object obj);
+array<object> as_array_object (object obj);
 url as_url (object obj);
 command as_command (object obj);
 #ifdef WIDGET_H // FIXME: dirty hack
@@ -168,4 +174,7 @@ object call (object fun, object a1, object a2, object a3);
 object call (object fun, object a1, object a2, object a3, object a4);
 object call (object fun, array<object> a);
 
+void notify_preferences_loaded ();
+string get_preference (string var);
+
 #endif // defined OBJECT_H
