@@ -49,6 +49,10 @@ public:
   tree (tree_label l, tree t1, tree t2, tree t3, tree t4);
   tree (tree_label l, tree t1, tree t2, tree t3, tree t4, tree t5);
   tree (tree_label l, tree t1, tree t2, tree t3, tree t4, tree t5, tree t6);
+  tree (tree_label l, tree t1, tree t2, tree t3, tree t4,
+	              tree t5, tree t6, tree t7);
+  tree (tree_label l, tree t1, tree t2, tree t3, tree t4,
+	              tree t5, tree t6, tree t7, tree t8);
   inline tree& operator [] (int i);
   tree operator () (int start, int end);
 
@@ -232,7 +236,7 @@ inline double as_double (tree t) {
 inline string as_string (tree t) {
   if (is_atomic (t)) return t->label;
   else return ""; }
-string var_as_string (tree t);
+string tree_as_string (tree t);
 template<class T> inline tree as_tree(T x) { return (tree) x; }
 template<> inline tree as_tree(int x) { return as_string (x); }
 template<> inline tree as_tree(double x) { return as_string (x); }
@@ -251,11 +255,14 @@ bool is_formatting (tree t);
 bool is_table (tree t);
 bool is_table_format (tree t);
 bool is_multi_paragraph (tree t);
+bool is_around (tree t);
 bool is_script (tree t);
 bool is_script (tree t, bool& right);
 bool is_prime (tree t);
+bool is_right_script_prime (tree t);
 bool is_mod_active (tree t);
 bool is_mod_active_once (tree t);
+bool is_graphical_text (tree t);
 bool is_empty (tree t);
 
 inline bool
@@ -308,6 +315,39 @@ inline bool is_tuple (tree t, string s, int n) {
 inline bool is_tuple (tree t, const char* s, int n) {
   return (L(t) == TUPLE) && (N(t) == (n+1)) && (t[0] == s); }
 
+/******************************************************************************
+* Other frequent markup
+******************************************************************************/
+
+inline tree concat () {
+  return tree (CONCAT); }
+inline tree concat (tree t1) {
+  return tree (CONCAT, t1); }
+inline tree concat (tree t1, tree t2) {
+  return tree (CONCAT, t1, t2); }
+inline tree concat (tree t1, tree t2, tree t3) {
+  return tree (CONCAT, t1, t2, t3); }
+inline tree concat (tree t1, tree t2, tree t3, tree t4) {
+  return tree (CONCAT, t1, t2, t3, t4); }
+inline tree concat (tree t1, tree t2, tree t3, tree t4, tree t5) {
+  return tree (CONCAT, t1, t2, t3, t4, t5); }
+
+inline tree document () {
+  return tree (DOCUMENT); }
+inline tree document (tree t1) {
+  return tree (DOCUMENT, t1); }
+inline tree document (tree t1, tree t2) {
+  return tree (DOCUMENT, t1, t2); }
+inline tree document (tree t1, tree t2, tree t3) {
+  return tree (DOCUMENT, t1, t2, t3); }
+inline tree document (tree t1, tree t2, tree t3, tree t4) {
+  return tree (DOCUMENT, t1, t2, t3, t4); }
+inline tree document (tree t1, tree t2, tree t3, tree t4, tree t5) {
+  return tree (DOCUMENT, t1, t2, t3, t4, t5); }
+
+inline tree verbatim (tree t1) {
+  return compound ("verbatim", t1); }
+
 /******************************************************************************
 * Miscellaneous
 ******************************************************************************/
