@@ -78,7 +78,7 @@ public:
   void  polygon (array<SI> x, array<SI> y, bool convex=true);
   void  xpm (url file_name, SI x, SI y);
   void  image (url u, SI w, SI h, SI x, SI y,
-               double cx1, double cy1, double cx2, double cy2);
+               double cx1, double cy1, double cx2, double cy2, int alpha);
   
   void next_page ();
   bool is_printer();
@@ -129,9 +129,9 @@ cairo_renderer_rep::next_page () {
 
 void
 tm_cairo_set_source_color(cairo_t *context, color c) {
-  int r,g,b;
-  get_rgb_color(c, r, g, b);
-  tm_cairo_set_source_rgba(context, r/255.0, g/255.0, b/255.0, 1.0);
+  int r, g, b, a;
+  get_rgb_color(c, r, g, b, a);
+  tm_cairo_set_source_rgba(context, r/255.0, g/255.0, b/255.0, a/255.0);
 }
 
 void
@@ -268,13 +268,15 @@ struct cairo_cache_image_rep: cache_image_element_rep {
 
 void
 cairo_renderer_rep::image (url u, SI w, SI h, SI x, SI y,
-			   double cx1, double cy1, double cx2, double cy2) 
+			   double cx1, double cy1, double cx2, double cy2,
+                           int alpha)
 {
   // Given an image of original size (W, H),
   // we display the part (cx1 * W, xy1 * H, cx2 * W, cy2 * H)
   // at position (x, y) in a rectangle of size (w, h)
 
   // if (DEBUG_EVENTS) cout << "cairo_renderer_rep::image " << as_string(u) << LF;
+  (void) alpha; // FIXME
 
   w= w/pixel; h= h/pixel;
   decode (x, y);
@@ -449,8 +451,8 @@ cairo_renderer_rep::draw (int c, font_glyphs fng, SI x, SI y) {
   basic_character xc (c, fng, sfactor, 0, 0);
   cairo_image mi = character_image [xc];
   if (is_nil(mi)) {
-    int r, g, b;
-    get_rgb (cur_fg, r, g, b);
+    int r, g, b, a;
+    get_rgb (cur_fg, r, g, b, a);
     SI xo, yo;
     glyph pre_gl= fng->get (c); if (is_nil (pre_gl)) return;
     glyph gl= shrink (pre_gl, sfactor, sfactor, xo, yo);
