@@ -23,6 +23,7 @@
 #include "widget.hpp" 
 #include "message.hpp"
 #include "promise.hpp"
+#include "analyze.hpp"
 
 #include "aqua_basic_widgets.h"
 
@@ -63,6 +64,12 @@ aqua_widget_rep::popup_window_widget (string s) {
 }
 
 
+
+TMMenuItem *
+aqua_widget_rep::as_menuitem() { 
+  return [TMMenuItem alloc];  
+}
+
 /******************************************************************************
 * aqua_view_widget_rep
 ******************************************************************************/
@@ -126,6 +133,7 @@ aqua_view_widget_rep::send (slot s, blackbox val) {
     break;
 	
   default:
+    cout << "slot type= " << slot_name (s) << "\n";
     FAILED ("cannot handle slot type");
   }
 }
@@ -425,7 +433,7 @@ aqua_tm_widget_rep::send (slot s, blackbox val) {
       updateVisibility();
     }
     break;
-  case SLOT_CONTEXT_ICONS_VISIBILITY:
+  case SLOT_MODE_ICONS_VISIBILITY:
     {
       TYPE_CHECK (type_box (val) == type_helper<bool>::id);
       bool f= open_box<bool> (val);
@@ -454,7 +462,7 @@ aqua_tm_widget_rep::send (slot s, blackbox val) {
     {
       TYPE_CHECK (type_box (val) == type_helper<string>::id);
       string msg = open_box<string> (val);
-      [leftField setStringValue:to_nsstring_utf8(aqua_translate(msg))];
+      [leftField setStringValue:to_nsstring_utf8 (tm_var_encode (msg))];
       [leftField displayIfNeeded];
     }
     break;
@@ -462,7 +470,7 @@ aqua_tm_widget_rep::send (slot s, blackbox val) {
     {
       TYPE_CHECK (type_box (val) == type_helper<string>::id);
       string msg = open_box<string> (val);
-      [rightField setStringValue:to_nsstring_utf8(aqua_translate(msg))];
+      [rightField setStringValue:to_nsstring_utf8 (tm_var_encode (msg))];
       [rightField displayIfNeeded];
     }
     break;
@@ -495,14 +503,16 @@ aqua_tm_widget_rep::send (slot s, blackbox val) {
     break;
     
   case SLOT_SHRINKING_FACTOR:
-    TYPE_CHECK (type_box (val) == type_helper<int>::id);
-    simple_widget_rep *w = (simple_widget_rep *)[(TMView*)[sv documentView] widget];
-    if (w) {
-      int new_sf = open_box<int> (val);
-      if (DEBUG_EVENTS) cout << "New shrinking factor :" << new_sf << LF;
-      w->handle_set_shrinking_factor (new_sf);
+    {
+      TYPE_CHECK (type_box (val) == type_helper<int>::id);
+      simple_widget_rep *w = (simple_widget_rep *)[(TMView*)[sv documentView] widget];
+      if (w) {
+        int new_sf = open_box<int> (val);
+        if (DEBUG_EVENTS) cout << "New shrinking factor :" << new_sf << LF;
+        w->handle_set_shrinking_factor (new_sf);
+      }
+      break;
     }
-    break;
 
   case SLOT_FILE:
     {
@@ -563,7 +573,7 @@ aqua_tm_widget_rep::query (slot s, int type_id) {
     TYPE_CHECK (type_id == type_helper<bool>::id);
     return close_box<bool> (visibility[3]);
         
-  case SLOT_CONTEXT_ICONS_VISIBILITY:
+  case SLOT_MODE_ICONS_VISIBILITY:
     TYPE_CHECK (type_id == type_helper<bool>::id);
     return close_box<bool> (visibility[2]);
     
@@ -681,14 +691,19 @@ aqua_tm_widget_rep::write (slot s, blackbox index, widget w) {
     [bc setMenu:to_nsmenu(w) forRow:0];
     layout();
     break;
-  case SLOT_CONTEXT_ICONS:
-    check_type_void (index, "SLOT_CONTEXT_ICONS");
+  case SLOT_MODE_ICONS:
+    check_type_void (index, "SLOT_MODE_ICONS");
     [bc setMenu:to_nsmenu(w) forRow:1];
     layout();
     break;
+  case SLOT_FOCUS_ICONS:
+    check_type_void (index, "SLOT_FOCUS_ICONS");
+    [bc setMenu:to_nsmenu(w) forRow:2];
+    layout();
+    break;
   case SLOT_USER_ICONS:
     check_type_void (index, "SLOT_USER_ICONS");
-    [bc setMenu:to_nsmenu(w) forRow:2];
+    [bc setMenu:to_nsmenu(w) forRow:3];
     layout();
     break;
   case SLOT_INTERACTIVE_PROMPT:
@@ -812,7 +827,7 @@ aqua_window_widget_rep::query (slot s, int type_id) {
   switch (s) {
   case SLOT_IDENTIFIER:
     TYPE_CHECK (type_id == type_helper<int>::id);
-    return close_box<int> ((int)[wc window] ? 1 : 0);
+    return close_box<int> ((intptr_t) [wc window] ? 1 : 0);
   case SLOT_POSITION:  
     {
       typedef pair<SI,SI> coord2;
@@ -923,10 +938,51 @@ simple_widget_rep::handle_repaint (SI x1, SI y1, SI x2, SI y2) {
 
 void
 simple_widget_rep::send (slot s, blackbox val) {
-  aqua_view_widget_rep::send(s, val);
+  if (DEBUG_QT) cout << "aqua_simple_widget_rep::send " << slot_name(s) << LF;
+  switch (s) {
+    case SLOT_INVALIDATE:
+    {
+      TYPE_CHECK (type_box (val) == type_helper<coord4>::id);
+      coord4 p= open_box<coord4> (val);
+      if (DEBUG_QT)
+        cout << "Invalidating rect " << rectangle(p.x1,p.x2,p.x3,p.x4) << LF;
+      aqua_renderer_rep* ren = (aqua_renderer_rep*)get_renderer (this);
+      if (ren) {
+        SI x1 = p.x1, y1 = p.x2, x2 = p.x3, y2 = p.x4;    
+        ren->outer_round (x1, y1, x2, y2);
+        ren->decode (x1, y1);
+        ren->decode (x2, y2);
+        //tm_canvas()->invalidate_rect (x1,y2,x2,y1);
+      }
+    }
+      break;
+    case SLOT_INVALIDATE_ALL:
+    {
+      ASSERT (is_nil (val), "type mismatch");
+      if (DEBUG_QT)
+        cout << "Invalidating all"<<  LF;
+      //tm_canvas()->invalidate_all ();
+    }
+      break;
+    case SLOT_CURSOR:
+    {
+      TYPE_CHECK (type_box (val) == type_helper<coord2>::id);
+      coord2 p= open_box<coord2> (val);
+      //QPoint pt = to_qpoint(p);
+      //tm_canvas() -> setCursorPos(pt);
+    }
+      break;
+      
+    default:
+      if (DEBUG_QT) cout << "[aqua_simple_widget_rep] ";
+      aqua_view_widget_rep::send (s, val);
+      //      FAILED ("unhandled slot type");
+  }
+  
 }
 
 
+
 blackbox
 simple_widget_rep::query (slot s, int type_id) {
   return aqua_view_widget_rep::query(s,type_id);
@@ -1029,6 +1085,18 @@ glue_widget (bool hx, bool vx, SI w, SI h) {
   return tm_new <aqua_view_widget_rep> ([[[NSView alloc] initWithFrame:NSMakeRect(0, 0, 50, 50)] autorelease]);
 }
 
+widget
+glue_widget (tree col, bool hx, bool vx, SI w, SI h) {
+  (void) col;
+  return glue_widget (hx, vx, w, h);
+}
+
+widget
+extend (widget w, array<widget> a) {
+  (void) a;
+  return w;
+}
+
 widget
 wait_widget (SI width, SI height, string message) { 
   // a widget of a specified width and height, displaying a wait message
