@@ -12,6 +12,7 @@
 #include "iterator.hpp"
 #include "dictionary.hpp"
 #include "qt_gui.hpp"
+#include "qt_utilities.hpp"
 #include "analyze.hpp"
 #include <locale.h>
 #include "language.hpp"
@@ -19,29 +20,38 @@
 #include <QDesktopWidget>
 #include <QClipboard>
 #include <QFileOpenEvent>
+#include <QLabel>
 #include <QSocketNotifier>
 #include <QSetIterator>
 #include "QTMGuiHelper.hpp"
 #include "QTMWidget.hpp"
+#include "QTMWindow.hpp"
 #include "qt_renderer.hpp" // for the_qt_renderer
 
+#ifdef MACOSX_EXTENSIONS
+#include "MacOS/mac_utilities.h"
+#endif
+
+
 #include "tm_link.hpp" // for number_of_servers
 
 #include "Scheme/object.hpp"
 //#include "TeXmacs/server.hpp" // for get_server
 
 #include "qt_simple_widget.hpp"
+#include "qt_window_widget.hpp"
 
 extern window (*get_current_window) (void);
 
 qt_gui_rep* the_gui= NULL;
 int nr_windows = 0; // FIXME: fake variable, referenced in tm_server
 
-time_t time_credit;  // interval to interrupt long redrawings
+time_t time_credit = 100;  // interval to interrupt long redrawings
 time_t timeout_time; // new redraw interruption
 time_t lapse = 0; // optimization for delayed commands
 
 // marshalling flags between update, needs_update and check_event.
+bool disable_check_event = false;
 bool updating = false;
 bool needing_update = false;
 bool wait_for_delayed_commands = true;
@@ -61,12 +71,25 @@ qt_gui_rep::qt_gui_rep(int &argc, char **argv):
   // argv= argv2;
 
   interrupted   = false;
-  time_credit = 25;
+  time_credit = 100;
   timeout_time= texmacs_time () + time_credit;
 
+  //waitDialog = NULL;
+  
   set_output_language (get_locale_language ());
   gui_helper = new QTMGuiHelper (this);
   qApp -> installEventFilter (gui_helper);
+  
+#ifdef QT_MAC_USE_COCOA
+  //HACK: this filter is needed to overcome a bug in Qt/Cocoa
+  extern void mac_install_filter(); // defined in src/Plugins/MacOS/mac_app.mm
+  mac_install_filter();
+#endif
+  
+  
+  
+  qApp-> installTranslator(new QTMTranslator(qApp));
+  
   updatetimer = new QTimer (gui_helper);
   updatetimer->setSingleShot (true);
   QObject::connect ( updatetimer, SIGNAL(timeout()), 
@@ -93,6 +116,12 @@ qt_gui_rep::get_max_size (SI& width, SI& height) {
 
 qt_gui_rep::~qt_gui_rep()  {
   delete gui_helper;
+  
+  while (waitDialogs.count()) {
+    delete waitDialogs.last();
+    waitDialogs.removeLast();
+  }
+  
  // delete updatetimer; we do not need this given that gui_helper is the 
  // parent of updatetimer
 }
@@ -102,57 +131,64 @@ qt_gui_rep::~qt_gui_rep()  {
 ******************************************************************************/
 
 bool
-qt_gui_rep::get_selection (string key, tree& t, string& s) {
-  QClipboard *cb= QApplication::clipboard();
-  bool owns= true;
-  QClipboard::Mode mode;
-  if (key == "primary") {
-    owns= cb->ownsClipboard();
-    mode= QClipboard::Clipboard;
-  } else if (key == "mouse" && cb->supportsSelection()) {
-    owns= cb->ownsSelection();
-    mode= QClipboard::Selection;
+qt_gui_rep::get_selection (string key, tree& t, string& s, string format) {
+  (void) format;
+  QClipboard *cb= QApplication::clipboard ();
+  QClipboard::Mode mode= QClipboard::Clipboard;
+  bool owns= false;
+  if (key == "primary" || (key == "mouse" && cb->supportsSelection ())) {
+    if (key == "mouse") mode= QClipboard::Selection;
+    const QMimeData *md= cb->mimeData (mode);
+    if (md) owns= md->hasFormat ("application/x-texmacs-clipboard");  
   }
+  else owns= true;
+  
   s= "";
   t= "none";
-  
   if (owns) {
-    if (selection_t->contains (key)) {
-      t= copy (selection_t [key]);
-      s= copy (selection_s [key]);
-      return true;
-    }
-    return false;
+    if (!selection_t->contains (key)) return false;
+    t= copy (selection_t [key]);
+    s= copy (selection_s [key]);
+    return true;
   }
-  
-  QString originalText = cb->text(mode);
-  QByteArray buf = originalText.toAscii();
-  if (!(buf.isEmpty())) {
-    s << string(buf.constData(), buf.size());
+  else {
+    QString originalText= cb->text (mode);
+    QByteArray buf;
+    if (format == "verbatim" &&
+	get_preference ("verbatim->texmacs:encoding") == "utf-8")
+      buf= originalText.toUtf8 ();
+    else buf= originalText.toAscii ();
+    if (!(buf.isEmpty())) s << string (buf.constData(), buf.size());
+    t= tuple ("extern", s);
+    return true;
   }
-  
-  t= tuple ("extern", s);
-  return true;
 }
 
 bool
-qt_gui_rep::set_selection (string key, tree t, string s) {
+qt_gui_rep::set_selection (string key, tree t, string s, string format) {
   selection_t (key)= copy (t);
   selection_s (key)= copy (s);
         
-                
-  QClipboard *cb = QApplication::clipboard();
-  QClipboard::Mode mode;
-  if (key == "primary") {
-    mode= QClipboard::Clipboard;
-  } else if (key == "mouse" && cb->supportsSelection()) {
+  QClipboard *cb= QApplication::clipboard ();
+  QClipboard::Mode mode= QClipboard::Clipboard;
+  if (key == "primary");
+  else if (key == "mouse" && cb->supportsSelection())
     mode= QClipboard::Selection;
-  } else {
-    return true;
-  }
+  else return true;
+  cb->clear (mode);
 
   char *selection = as_charp (s);
-  cb->setText(selection,mode);
+  cb->setText (selection, mode);
+  QByteArray ba (selection);
+  QMimeData *md= new QMimeData;
+  md->setData ("application/x-texmacs-clipboard", ba);
+  if (format == "verbatim" &&
+      get_preference ("texmacs->verbatim:encoding") == "utf-8")
+    md->setText (QString::fromUtf8 (selection));
+  else md->setText (selection);
+  cb->setMimeData (md, mode); 
+  // according to the docs, ownership of mimedata is transferred to clipboard
+  // so no memory leak here
   tm_delete_array (selection);
   return true;
 }
@@ -161,6 +197,18 @@ void
 qt_gui_rep::clear_selection (string key) {
   selection_t->reset (key);
   selection_s->reset (key);
+  
+  QClipboard *cb= QApplication::clipboard();
+  QClipboard::Mode mode= QClipboard::Clipboard;
+  bool owns= false;
+  if (key == "primary");
+  else if (key == "mouse" && cb->supportsSelection())
+    mode= QClipboard::Selection;
+  else return;
+
+  const QMimeData *md= cb->mimeData (mode);
+  if (md) owns= md->hasFormat ("application/x-texmacs");
+  if (owns) cb->clear (mode);
 }
 
 /******************************************************************************
@@ -181,6 +229,67 @@ void qt_gui_rep::set_mouse_pointer (string curs_name, string mask_name)
 void
 qt_gui_rep::show_wait_indicator (widget w, string message, string arg)  {
   (void) w; (void) message; (void) arg;
+  
+  if (DEBUG_QT)  cout << "show_wait_indicator \"" << message << "\"\"" << arg << "\"" << LF;
+
+  qt_window_widget_rep *wid = static_cast<qt_window_widget_rep*> (w.rep);
+
+  // we move the texmacs window during an operation. 
+  // We need to disable updates of the window to avoid erasure of the canvas
+  // area
+  //  wid->wid->setUpdatesEnabled(false);
+  
+  //FIXME: we must center the wait widget wrt the current active window
+  
+  if (waitDialogs.count()) {
+    waitDialogs.last()->close();
+  }
+  
+  if (N(message)) {
+    
+    // push a new wait message in the list
+    
+    if (arg != "") message= message * " " * arg * "...";
+    
+    QLabel* lab = new  QLabel(); //(wid->wid->window());
+    lab->setWindowFlags(Qt::Window | Qt::FramelessWindowHint | Qt::WindowStaysOnTopHint);
+    //lab->setWindowModality(Qt::ApplicationModal);
+    lab->setFocusPolicy(Qt::NoFocus);
+    lab->setMargin(15);
+    lab->setText (to_qstring (tm_var_encode(message)));
+    QSize sz = lab->sizeHint();
+    QRect rect = QRect(QPoint(0,0),sz);
+    //HACK: 
+    // processEvents is needed to let Qt update windows coordinates in the case
+    qApp->processEvents();
+    //ENDHACK
+    QPoint pt = wid->wid->window()->geometry().center();
+    rect.moveCenter(pt);
+    lab->move(rect.topLeft());
+    waitDialogs << lab;
+    
+  } else {
+    // pop the next wait message from the list
+    if (waitDialogs.count()) {
+      waitDialogs.last()->deleteLater();
+      waitDialogs.removeLast();
+    }    
+  }
+
+  if (waitDialogs.count()) {
+    waitDialogs.last()->show();
+    //waitDialogs.last()->raise();
+    qApp->processEvents();
+    waitDialogs.last()->repaint();
+    qApp->processEvents();
+    QApplication::flush();
+
+  }
+
+  //    wid->wid->setUpdatesEnabled(true);
+
+  // next time we do update the dialog will disappear
+  needs_update();  
 }
 
 void (*the_interpose_handler) (void) = NULL;
@@ -190,7 +299,6 @@ void gui_interpose (void (*r) (void)) { the_interpose_handler= r; }
 void
 qt_gui_rep::event_loop () {
   QApplication *app = (QApplication*) QApplication::instance();
-
   update();
   //needs_update();
   app->exec();
@@ -207,13 +315,14 @@ void
 qt_gui_rep::add_notifier (socket_notifier sn)
 {
   QSocketNotifier *qsn;
-
-  if (DEBUG_QT) cout << "ADD NOTIFIER " << sn->fd << LF;
   
   // replace any already present notifier
 
   remove_notifier (sn);
 
+  if (DEBUG_QT) cout << "ADD NOTIFIER " << sn->fd << LF;
+
+  
   // installs both a read and a write notifier 
   // (the texmacs interface does not specify enough its needs)
   
@@ -339,6 +448,10 @@ gui_open (int& argc, char** argv) {
   // start the gui
  // new QApplication (argc,argv); now in texmacs.cpp
   the_gui = tm_new<qt_gui_rep> (argc, argv);
+  
+#ifdef MACOSX_EXTENSIONS
+  mac_begin_remote();
+#endif
 }
 
 void
@@ -353,6 +466,10 @@ gui_close () {
   ASSERT (the_gui != NULL, "gui not yet open");
   tm_delete (the_gui);
   the_gui=NULL;
+
+#ifdef MACOSX_EXTENSIONS
+  mac_end_remote();
+#endif
 }
 
 void
@@ -373,8 +490,11 @@ gui_maximal_extents (SI& width, SI& height) {
 
 void
 gui_refresh () {
-  // update and redraw all windows (e.g. on change of output language)
-  // FIXME: add suitable code
+  // called upon change of output language
+  // emit a signal which force every QTMAction to change his text
+  // according to the new language
+
+  the_gui->gui_helper->doRefresh();
 }
 
 
@@ -388,6 +508,11 @@ QTMGuiHelper::doUpdate () {
   gui->update();
 }
 
+void
+QTMGuiHelper::doRefresh () {
+  emit refresh();
+}
+
 bool
 QTMGuiHelper::eventFilter (QObject *obj, QEvent *event) {
    if (event->type() == QEvent::FileOpen) {
@@ -405,7 +530,9 @@ QTMGuiHelper::eventFilter (QObject *obj, QEvent *event) {
 
 void
 QTMGuiHelper::doWriteSocketNotification (int socket) {
-//cout << "WRITE SOCKET NOTIFICATION " << socket << " "<< texmacs_time () << LF;
+  if (DEBUG_QT) 
+    cout << "WRITE SOCKET NOTIFICATION " << socket << " "
+         << texmacs_time () << LF;
   iterator<socket_notifier> it = iterate (write_notifiers);
   while (it->busy ()) {
     socket_notifier sn= it->next ();
@@ -419,7 +546,9 @@ QTMGuiHelper::doWriteSocketNotification (int socket) {
 
 void
 QTMGuiHelper::doReadSocketNotification (int socket) {
-//cout << "READ SOCKET NOTIFICATION " << socket << " "<< texmacs_time () << LF;
+  if (DEBUG_QT) 
+    cout << "READ SOCKET NOTIFICATION " << socket << " "
+         << texmacs_time () << LF;
   iterator<socket_notifier> it = iterate (read_notifiers);
   while (it->busy ()) {
     socket_notifier sn= it->next ();
@@ -444,6 +573,7 @@ enum qp_type_id {
   QP_RESIZE,
   QP_SOCKET_NOTIFICATION,
   QP_COMMAND,
+  QP_COMMAND_ARGS,
   QP_DELAYED_COMMANDS
 };
 
@@ -475,6 +605,7 @@ static array<queued_event> waiting_events;
 
 void
 process_event (queued_event ev) {
+  //cout << "<" << (qp_type_id) ev.x1 << LF;
   switch ((qp_type_id) ev.x1) {
     case QP_NULL :
       break;
@@ -490,10 +621,6 @@ process_event (queued_event ev) {
       typedef triple<widget, bool, time_t > T;
       T x = open_box <T> (ev.x2) ;
       concrete_simple_widget(x.x1) -> handle_keyboard_focus (x.x2, x.x3) ;
-      send_invalidate_all(x.x1);
-      //FIXME: the above invalidate is due to incorrect repainting when
-      //       the widget loose the focus. I should investigate better
-      //       the problem.
     }
       break;
     case QP_MOUSE :
@@ -526,6 +653,14 @@ process_event (queued_event ev) {
       cmd->apply();
     }
       break;
+    case QP_COMMAND_ARGS :
+    {
+      typedef pair<command, object> T;
+      T x = open_box <T> (ev.x2) ;
+      // cout << "QP_COMMAND_ARGS" << LF;
+      x.x1->apply(x.x2);
+    }
+      break;
     case QP_DELAYED_COMMANDS :
     {
       // cout << "QP_DELAYED_COMMANDS" << LF;
@@ -536,9 +671,26 @@ process_event (queued_event ev) {
     default:   
       FAILED("Unexpected queued event");
   }
+  //cout << ">" << (qp_type_id) ev.x1 << LF;
 }
 
 
+
+queued_event 
+next_event() {
+  queued_event ev;
+  if (N(waiting_events)>0) 
+    ev = waiting_events[0];
+  
+  array<queued_event> a;
+
+  for(int i=1; i<N(waiting_events); i++)
+    a << waiting_events[i];
+  waiting_events = a;
+  
+  return ev;
+}
+
 void 
 qt_gui_rep::process_queued_events (int max) {
   // we process a maximum of max events. There are two kind of events: those
@@ -549,18 +701,15 @@ qt_gui_rep::process_queued_events (int max) {
   // considerer a limitation of the current implementation of interpose_handler
   // Likewise this function is just an hack to get things working properly.
   
-  array<queued_event> a = waiting_events, b;
-  waiting_events = array<queued_event> ();
-  
-  int i, n = N(a);
   int count = 0;
-  //cout << "(" << n << " events)";
-  for (i=0; i<n; i++) {
-    //cout << (int)a[i].x1 << ",";
-    if ((max < 0) || (count<max))  {
-      process_event(a[i]);
-      switch (a[i].x1) {
+  //cout << "(" << n << " events)"
+  while ((max < 0) || (count<max))  {
+    queued_event ev = next_event();
+    if (ev.x1 == QP_NULL) break;
+    process_event(ev);
+    switch (ev.x1) {
         case QP_COMMAND:
+        case QP_COMMAND_ARGS:
         case QP_SOCKET_NOTIFICATION:
         case QP_RESIZE:
         case QP_DELAYED_COMMANDS:
@@ -568,12 +717,8 @@ qt_gui_rep::process_queued_events (int max) {
         default:
           count++;
           break;
-      }
-    } else 
-      b << a[i];
+    }
   }
-  b << waiting_events;
-  waiting_events = b;
 }
 
 
@@ -587,7 +732,13 @@ add_event(const queued_event &ev)
   } else {
     waiting_events << ev;
 //    process_event(ev);
-    the_gui->update();
+    needs_update();
+    // NOTE: we cannot update now since sometimes this seems to give problems
+    // to the update of the window size after a resize. In that situation
+    // sometimes when the window take again focus, update will be called for the
+    // focus_in event and interpose_handler is run which send a slot_extent
+    // message to the widget causing a wrong resize of the window.
+    // this seems to cure the problem.
   }
 }
 
@@ -605,8 +756,8 @@ qt_gui_rep::process_keyboard_focus ( simple_widget_rep *wid, bool has_focus,
   typedef triple<widget, bool, time_t > T;
   add_event( 
     queued_event ( QP_KEYBOARD_FOCUS, close_box<T> (T(wid, has_focus, t)))); 
-//  wid -> handle_keyboard_focus (has_focus, t);
-//  needs_update ();
+  //wid -> handle_keyboard_focus (has_focus, t);
+  //needs_update ();
 }
 
 void 
@@ -643,6 +794,13 @@ qt_gui_rep::process_command (command _cmd) {
              queued_event (QP_COMMAND, close_box< command > (_cmd)));
 }
 
+void 
+qt_gui_rep::process_command (command _cmd, object _args) {
+  typedef pair<command, object > T;
+  add_event ( 
+             queued_event (QP_COMMAND_ARGS, close_box< T > (T(_cmd,_args))));
+}
+
 void 
 qt_gui_rep::process_delayed_commands () {
   add_event ( 
@@ -664,14 +822,18 @@ qt_gui_rep::check_event (int type) {
   
 // cout << "."; cout.flush();
 //  return false;
+  
+  // do not interrupt while not in update
+  // (for example while painting the icons in the menus)
+  if (!updating || disable_check_event) return false;
+  
   switch (type) {
     case INTERRUPT_EVENT:
       if (interrupted) return true;
       else {
         time_t now= texmacs_time ();
         if (now - timeout_time < 0) return false;
-        fill_event_queue();
-        timeout_time= now + (100 / (N(waiting_events) + 1));
+        timeout_time= now + time_credit;
         interrupted= (N(waiting_events) > 0);
         //if (interrupted) cout << "INTERRUPT " 
         //  << now << "------------------" << LF;
@@ -692,71 +854,101 @@ qt_gui_rep::update () {
   // the eventloop afterwards we reactivate the timer with a pause 
   // (see FIXME below) 
 
-  time_credit = 100;
-  updating = true;
+  if (updating) {
+    cout << "NESTED UPDATING: This should not happen" << LF;
+    needs_update();
+    return;
+  }
+
+  // cout << "<" << texmacs_time() << " " << N(delayed_queue) << " ";
+  
   updatetimer->stop();
+  updating = true;
   
-  int count_events = 0;
-  int max_proc_events = 2;
+  static int count_events = 0;
+  static int max_proc_events = 10;
+
+  time_t now = texmacs_time();
+  needing_update = false;
+
+  time_credit = 100 / (N(waiting_events)+1);
+
   
-  do {
-    time_t now = texmacs_time();
-    needing_update = false;
-    
+  // 1.
+  // check if a wait dialog is active and in the case remove it.
+  // if we are here then the long operation is finished.
+  
+  while (waitDialogs.count()) {
+    waitDialogs.last()->close();
+    waitDialogs.last()->deleteLater();
+    waitDialogs.removeLast();
+  }
+  
+  
+  // 2.
+  // manage delayed commands
+  
+  if (wait_for_delayed_commands && (lapse <= now)) process_delayed_commands();
+  
+  // 3.
+  // if there are pending events in the private queue process them up to some
+  // limit in processed events is reached. 
+  // if there are no events or the limit is reached then proceed to a redraw.
+
+  if (N(waiting_events) == 0) {
+    // if there are not waiting events call at least once
+    // the interpose handler
+    if (the_interpose_handler) the_interpose_handler();
+  } else while ((N(waiting_events) > 0) && (count_events < max_proc_events)) {
+    // cout << "e";
+    //cout << "PROCESS QUEUED EVENTS START..."; cout.flush();
+    process_queued_events (1);
+    //cout << "AND END"  << LF;
     count_events++;
-
-    if (wait_for_delayed_commands && (lapse <= now)) process_delayed_commands();
-
-    if (N(waiting_events) > 0) {
-      //cout << "PROCESS QUEUED EVENTS START..."; cout.flush();
-      process_queued_events (1);
-      //cout << "AND END"  << LF;
-    };
     
     //cout << "TYPESET START..."; cout.flush();
-//    if ((N(waiting_events)==0) && (lapse <= now)) exec_pending_commands();
     if (the_interpose_handler) the_interpose_handler();
     //cout << "AND END" << LF;
-
-    fill_event_queue(); 
-
-    if ((count_events > max_proc_events) || (N(waiting_events) == 0)) {
-      //if (count_events > max_proc_events) cout << "PARTIAL REDRAW" << LF;
-      count_events = 0;
-
-      // repaint invalid regions  
-      interrupted = false;
-      timeout_time = texmacs_time() + time_credit/(N(waiting_events)+1);
-      
-      //cout << "REPAINT START..."; cout.flush();
-      
-      QSetIterator<QTMWidget*> i(QTMWidget::all_widgets);
-      while (i.hasNext()) {
-        QTMWidget *w = i.next();
-        w->repaint_invalid_regions();
-      }
-      
-      if (interrupted) needing_update = true;
-      //cout << "AND END" << LF;
-    }
-  
-  } while ((N(waiting_events)>0) || needing_update);
+  }
   
+  {
+    // redrawing
+    // cout << "r";
+    count_events = 0;
     
-  updating = false;
-
+    // repaint invalid regions  
+    interrupted = false;
+    timeout_time = texmacs_time() + time_credit;
+    
+    //cout << "REPAINT START..."; cout.flush();
+    
+    QSetIterator<QTMWidget*> i(QTMWidget::all_widgets);
+    while (i.hasNext()) {
+      QTMWidget *w = i.next();
+      w->repaint_invalid_regions();
+    }
+    //cout << "AND END" << LF;
+  }
   
+  
+  if (N(waiting_events) > 0) needing_update = true;
+  if (interrupted) needing_update = true;
+ // if (interrupted)     cout << "*";
+
   if (nr_windows == 0) {
     qApp->quit();
   }
   
-  
   time_t delay = lapse - texmacs_time();
   if (delay > 1000/6) delay = 1000/6;
   if (delay < 0) delay = 0;
-  if (needing_update || interrupted) delay = 0;
+  if (needing_update) delay = 0;
+  //cout << delay << ">" <<  LF;
   updatetimer->start (delay);
   
+  updating = false;
+  
+
   // FIXME: we need to ensure that the interpose_handler is run at regular 
   //        intervals (1/6th of sec) so that informations on the footbar are 
   //        updated. (this should be better handled by promoting code in 
@@ -779,8 +971,8 @@ set_default_font (string name) {
 }
 
 font
-get_default_font (bool tt) {
-        (void) tt;      
+get_default_font (bool tt, bool mini) {
+  (void) tt; (void) mini;
   // get the default font or monospaced font (if tt is true)
         
   // return a null font since this function is not called in the Qt port.
@@ -805,17 +997,17 @@ load_system_font (string family, int size, int dpi,
 ******************************************************************************/
 
 bool
-set_selection (string key, tree t, string s) {
+set_selection (string key, tree t, string s, string format) {
   // Copy a selection 't' with string equivalent 's' to the clipboard 'cb'
   // Returns true on success
-  return the_gui->set_selection (key, t, s);
+  return the_gui->set_selection (key, t, s, format);
 }
 
 bool
-get_selection (string key, tree& t, string& s) {
+get_selection (string key, tree& t, string& s, string format) {
   // Retrieve the selection 't' with string equivalent 's' from clipboard 'cb'
   // Returns true on success; sets t to (extern s) for external selections
-  return the_gui->get_selection (key, t, s);
+  return the_gui->get_selection (key, t, s, format);
 }
 
 void
@@ -835,6 +1027,16 @@ beep () {
   QApplication::beep();
 }
 
+void
+force_update() {
+  if (updating) {
+    needing_update = true;
+  }
+  else {
+    the_gui->update();
+  }
+}
+
 void
 needs_update () {
   // 0 ms - call immediately when all other events have 
@@ -877,6 +1079,16 @@ show_wait_indicator (widget base, string message, string argument) {
   the_gui->show_wait_indicator(base,message,argument);
 }
 
+void
+external_event (string type, time_t t) {
+  // External events, such as pushing a button of a remote infrared commander
+  QTMWidget *tm_focus = qobject_cast<QTMWidget*>(qApp->focusWidget());
+  if (tm_focus) {
+    simple_widget_rep *wid = tm_focus->tm_widget();
+    if (wid) the_gui -> process_keypress (wid, type, t);
+  }
+}
+
 font x_font (string family, int size, int dpi)
 {
   (void) family; (void) size; (void) dpi;
@@ -884,3 +1096,15 @@ font x_font (string family, int size, int dpi)
   return NULL;
 }
 
+
+QString 
+QTMTranslator::translate ( const char * context, const char * sourceText, 
+                           const char * disambiguation ) const 
+{
+  (void) disambiguation;  (void) context;
+  if (DEBUG_QT) {
+    cout << "Translating: " << sourceText << LF;
+    cout << "Translation: " << qt_translate (sourceText) << LF;
+  }
+  return QString (to_qstring (qt_translate (sourceText)));
+}
