@@ -15,6 +15,8 @@
 #include <QPixmap>
 #include <QApplication>
 #include <QTimer>
+#include <QLabel>
+#include <QList>
 
 #include "timer.hpp"
 #include "gui.hpp"
@@ -39,6 +41,7 @@ public:
   time_t interrupt_time;
   QTMGuiHelper *gui_helper;
   QTimer *updatetimer;
+  QList<QLabel*> waitDialogs;
 
   hashmap<string,tree>   selection_t;
   hashmap<string,string> selection_s;
@@ -56,8 +59,8 @@ public:
   void event_loop ();
 
   /* interclient communication */
-  virtual bool get_selection (string key, tree& t, string& s);
-  virtual bool set_selection (string key, tree t, string s);
+  virtual bool get_selection (string key, tree& t, string& s, string format);
+  virtual bool set_selection (string key, tree t, string s, string format);
   virtual void clear_selection (string key);
 
   /* miscellaneous */
@@ -79,6 +82,7 @@ public:
   void process_mouse (simple_widget_rep *wid, string kind, SI x, SI y, int mods, time_t t);
   void process_resize (simple_widget_rep *wid, SI x, SI y);
   void process_command (command _cmd);
+  void process_command (command _cmd, object _args);
   void process_socket_notification (socket_notifier sn);
   void process_delayed_commands (); 
 
@@ -92,4 +96,7 @@ public:
   
 };
 
+void force_update(); 
+// force an immediate update of the internal texmacs state
+
 #endif // defined QT_GUI_HPP
