@@ -96,11 +96,10 @@ qt_renderer_rep::begin (void* handle) {
 void qt_renderer_rep::end () { painter->end (); }
 
 QColor
-qt_color(color c)
-{
-  int r, g, b;
-  get_rgb_color (c,r,g,b);
-  return QColor(r, g, b);
+qt_color(color c) {
+  int r, g, b, a;
+  get_rgb_color (c, r, g, b, a);
+  return QColor (r, g, b, a);
 }
 
 void 
@@ -127,6 +126,8 @@ qt_renderer_rep::set_clipping (SI x1, SI y1, SI x2, SI y2, bool restore)
   if ((x1<x2) && (y2<y1)) {
     QRect r(x1,y2,x2-x1,y1-y2);
     painter->setClipRect(r);
+  } else {
+    painter->setClipRect(QRect());
   }
 }
 
@@ -284,17 +285,21 @@ struct qt_cache_image_rep: cache_image_element_rep {
 
 void
 qt_renderer_rep::image (url u, SI w, SI h, SI x, SI y,
-                        double cx1, double cy1, double cx2, double cy2)
+                        double cx1, double cy1, double cx2, double cy2,
+                        int alpha)
 {
   // Given an image of original size (W, H),
   // we display the part (cx1 * W, xy1 * H, cx2 * W, cy2 * H)
   // at position (x, y) in a rectangle of size (w, h)
-
   if(cx2<=cx1 || cy2<=cy1) return;
 
   w= w/pixel; h= h/pixel;
   decode (x, y);
 
+  // safety check
+  url ru = resolve(u);
+  u = is_none (ru) ? "$TEXMACS_PATH/misc/pixmaps/unknown.ps" : ru;
+  
   QImage *pm = NULL;
   tree lookup= tuple (u->t);
   lookup << as_string (w ) << as_string (h )
@@ -307,9 +312,10 @@ qt_renderer_rep::image (url u, SI w, SI h, SI x, SI y,
     // rendering
     bool needs_crop= false;
     if (qt_supports (u)) {
-      pm= new QImage (to_qstring (concretize (u)));
+      pm= new QImage (utf8_to_qstring (concretize (u)));
       needs_crop= true;
-    } else if (suffix (u) == "ps" ||
+    }
+    else if (suffix (u) == "ps" ||
              suffix (u) == "eps" ||
              suffix (u) == "pdf") {
       url temp= url_temp (".png");
@@ -390,7 +396,10 @@ qt_renderer_rep::image (url u, SI w, SI h, SI x, SI y,
     (ci->nr)++;
   }
 
+  qreal old_opacity= painter->opacity ();
+  painter->setOpacity (qreal (alpha) / qreal (255));
   painter->drawImage (x, y-h, *pm);
+  painter->setOpacity (old_opacity);
 };
 
 
@@ -424,8 +433,8 @@ qt_renderer_rep::draw (int c, font_glyphs fng, SI x, SI y) {
   basic_character xc (c, fng, sfactor, cur_fg, 0);
   qt_image mi = character_image [xc];
   if (is_nil(mi)) {
-    int r, g, b;
-    get_rgb (cur_fg, r, g, b);
+    int r, g, b, a;
+    get_rgb (cur_fg, r, g, b, a);
     SI xo, yo;
     glyph pre_gl= fng->get (c); if (is_nil (pre_gl)) return;
     glyph gl= shrink (pre_gl, sfactor, sfactor, xo, yo);
@@ -444,7 +453,7 @@ qt_renderer_rep::draw (int c, font_glyphs fng, SI x, SI y) {
       for (j=0; j<h; j++)
         for (i=0; i<w; i++) {
           int col = gl->get_x (i, j);
-          brush.setColor (QColor (r, g, b, (255*col)/nr_cols));
+          brush.setColor (QColor (r, g, b, (a*col)/nr_cols));
           pp.fillRect (i, j, 1, 1, brush);
         }
       pp.end();
@@ -463,7 +472,7 @@ qt_renderer_rep::draw (int c, font_glyphs fng, SI x, SI y) {
       for (j=0; j<h; j++)
         for (i=0; i<w; i++) {
           int col = gl->get_x (i, j);
-          im->setPixel (i, j, qRgba (r, g, b, (255*col)/nr_cols));
+          im->setPixel (i, j, qRgba (r, g, b, (a*col)/nr_cols));
         }
     }
 #endif
@@ -476,6 +485,8 @@ qt_renderer_rep::draw (int c, font_glyphs fng, SI x, SI y) {
   }
 
   // draw the character
+  //cout << (char)c << ": " << cx1/256 << ","  << cy1/256 << ","  
+  //<< cx2/256 << ","  << cy2/256 << LF; 
   draw_clipped (mi->img, mi->w, mi->h, x- mi->xo*sfactor, y+ mi->yo*sfactor);
 }
 
@@ -491,7 +502,12 @@ qt_renderer_rep::xpm_image (url file_name) {
   qt_pixmap mi= images [as_string (file_name)];
   if (is_nil (mi)) {
     string sss;
-    load_string ("$TEXMACS_PIXMAP_PATH" * file_name, sss, false);
+    if (suffix (file_name) == "xpm") {
+      url png_equiv= glue (unglue (file_name, 3), "png");
+      load_string ("$TEXMACS_PIXMAP_PATH" * png_equiv, sss, false);
+    }
+    if (sss == "")
+      load_string ("$TEXMACS_PIXMAP_PATH" * file_name, sss, false);
     if (sss == "")
       load_string ("$TEXMACS_PATH/misc/pixmaps/TeXmacs.xpm", sss, true);
     uchar *buf= (uchar*) as_charp (sss);
@@ -523,6 +539,54 @@ qt_renderer_rep::xpm (url file_name, SI x, SI y) {
   char_clip=old_clip;
 }
 
+/******************************************************************************
+ * main qt renderer
+ ******************************************************************************/
+
+
+qt_renderer_rep*
+the_qt_renderer () {
+  static QPainter *the_painter = NULL;
+  static qt_renderer_rep* the_renderer= NULL;
+  if (!the_renderer) {
+    the_painter = new QPainter();
+    the_renderer= tm_new<qt_renderer_rep> (the_painter);
+  }
+  return the_renderer;
+}
+
+
+/******************************************************************************
+ * Shadow management methods 
+ ******************************************************************************/
+
+/* Shadows are auxiliary renderers which allows double buffering and caching of
+ * graphics. TeXmacs has explicit double buffering from the X11 port. Maybe
+ * it would be better to design a better API abstracting from the low level 
+ * details but for the moment the following code and the qt_proxy_renderer_rep
+ * and qt_shadow_renderer_rep classes are designed to solve two problems:
+ * 
+ * 1) Qt has already double buffering.
+ * 2) in Qt we are not easily allowed to read onscreen pixels (we can only ask a
+ *    widget to redraw himself on a pixmap or read the screen pixels -- this has
+ *    the drawback that if our widget is under another one we won't read the 
+ *    right pixels)
+ * 
+ * qt_proxy_renderer_rep solve the double buffering problem: when texmacs ask
+ * a qt_renderer_rep for a shadow it is given a proxy of the original renderer
+ * texmacs uses this shadow for double buffering and the proxy will simply
+ * forward the drawing operations to the original surface and neglect all the
+ * syncronization operations
+ *
+ * to solve the second problem we do not draw directly on screen in QTMWidget.
+ * Instead we maintain an internal pixmap which represent the state of the pixels
+ * according to texmacs. when we are asked to initialize a qt_shadow_renderer_rep
+ * we simply read the pixels form this backing store. At the Qt level then
+ * (in QTMWidget) we make sure that the state of the backing store is in sync
+ * with the screen via paintEvent/repaint mechanism.
+ *
+ */
+
 
 void
 qt_renderer_rep::new_shadow (renderer& ren) {
@@ -563,23 +627,25 @@ qt_renderer_rep::get_shadow (renderer ren, SI x1, SI y1, SI x2, SI y2) {
   y2= min (y2, cy2- oy);
   shadow->ox= ox;
   shadow->oy= oy;
+  shadow->master= this;
   shadow->cx1= x1+ ox;
   shadow->cy1= y1+ oy;
   shadow->cx2= x2+ ox;
   shadow->cy2= y2+ oy;
-  shadow->master= this;
-#if 0
+  
   decode (x1, y1);
   decode (x2, y2);
   if (x1<x2 && y2<y1) {
     QRect rect = QRect(x1, y2, x2-x1, y1-y2);
-    //    shadow->painter->setCompositionMode(QPainter::CompositionMode_Source);   
-    shadow->painter->drawPixmap (rect, px, rect);
+    //    shadow->painter->setCompositionMode(QPainter::CompositionMode_Source);  
+    shadow->painter->setClipRect(rect);
+//    shadow->painter->drawPixmap (rect, px, rect);
     //    cout << "qt_shadow_renderer_rep::get_shadow " 
     //         << rectangle(x1,y2,x2,y1) << LF;
     //  XCopyArea (dpy, win, shadow->win, gc, x1, y2, x2-x1, y1-y2, x1, y2);
+  } else {
+    shadow->painter->setClipRect(QRect());
   }
-#endif
 }
 
 void 
@@ -620,28 +686,10 @@ qt_renderer_rep::apply_shadow (SI x1, SI y1, SI x2, SI y2)  {
 }
 
 
-/******************************************************************************
- * main qt renderer
- ******************************************************************************/
-
-
-qt_renderer_rep*
-the_qt_renderer () {
-  static QPainter *the_painter = NULL;
-  static qt_renderer_rep* the_renderer= NULL;
-  if (!the_renderer) {
-    the_painter = new QPainter();
-    the_renderer= tm_new<qt_renderer_rep> (the_painter);
-  }
-  return the_renderer;
-}
-
 /******************************************************************************
  * proxy qt renderer
  ******************************************************************************/
 
-
-
 void 
 qt_proxy_renderer_rep::new_shadow (renderer& ren) {
   SI mw, mh, sw, sh;
@@ -664,7 +712,6 @@ qt_proxy_renderer_rep::new_shadow (renderer& ren) {
           &(static_cast<qt_shadow_renderer_rep*>(ren)->px));
 }
 
-
 void 
 qt_proxy_renderer_rep::get_shadow (renderer ren, SI x1, SI y1, SI x2, SI y2) {
   // FIXME: we should use the routine fetch later
@@ -687,6 +734,9 @@ qt_proxy_renderer_rep::get_shadow (renderer ren, SI x1, SI y1, SI x2, SI y2) {
   decode (x2, y2);
   if (x1<x2 && y2<y1) {
     QRect rect = QRect(x1, y2, x2-x1, y1-y2);
+
+    shadow->painter->setClipRect(rect);
+
     //    shadow->painter->setCompositionMode(QPainter::CompositionMode_Source);
     QPixmap *_pixmap = static_cast<QPixmap*>(painter->device()); 
     if (_pixmap) {
@@ -695,7 +745,10 @@ qt_proxy_renderer_rep::get_shadow (renderer ren, SI x1, SI y1, SI x2, SI y2) {
     //    cout << "qt_shadow_renderer_rep::get_shadow " 
     //         << rectangle(x1,y2,x2,y1) << LF;
     //  XCopyArea (dpy, win, shadow->win, gc, x1, y2, x2-x1, y1-y2, x1, y2);
+  } else {
+    shadow->painter->setClipRect(QRect());
   }
+
 }
 
 
@@ -740,11 +793,15 @@ qt_shadow_renderer_rep::get_shadow (renderer ren, SI x1, SI y1, SI x2, SI y2) {
   decode (x2, y2);
   if (x1<x2 && y2<y1) {
     QRect rect = QRect(x1, y2, x2-x1, y1-y2);
+    shadow->painter->setClipRect(rect);
+
 //    shadow->painter->setCompositionMode(QPainter::CompositionMode_Source);   
     shadow->painter->drawPixmap (rect, px, rect);
 //    cout << "qt_shadow_renderer_rep::get_shadow " 
 //         << rectangle(x1,y2,x2,y1) << LF;
 //  XCopyArea (dpy, win, shadow->win, gc, x1, y2, x2-x1, y1-y2, x1, y2);
+  } else {
+    shadow->painter->setClipRect(QRect());
   }
 }
 
