@@ -136,16 +136,23 @@ x_gui_rep::set_default_font (string name) {
 }
 
 font
-x_gui_rep::default_font_sub (bool tt) {
+x_gui_rep::default_font_sub (bool tt, bool mini) {
   string s= the_default_font;
   if (s == "") s= "ecrm11@300";
   int i, j, n= N(s);
   for (j=0; j<n; j++) if ((s[j] >= '0') && (s[j] <= '9')) break;
   string fam= s (0, j);
+  if (mini && fam == "ecrm") fam= "ecss";
   for (i=j; j<n; j++) if (s[j] == '@') break;
   int sz= (j<n? as_int (s (i, j)): 10);
   if (j<n) j++;
   int dpi= (j<n? as_int (s (j, n)): 300);
+  if (mini) { sz= (int) (0.6 * sz); dpi= (int) (1.3333333 * dpi); }
+  if (use_macos_fonts ()) {
+    tree lucida_fn= tuple ("apple-lucida", "ss", "medium", "right");
+    lucida_fn << as_string (sz) << as_string ((int) (0.95 * dpi));
+    return find_font (lucida_fn);
+  }
   if (N(fam) >= 2) {
     string ff= fam (0, 2);
     string out_lan= get_output_language ();
@@ -180,9 +187,9 @@ x_gui_rep::default_font_sub (bool tt) {
 }
 
 font
-x_gui_rep::default_font (bool tt) {
-  font fn= default_font_sub (tt);
-  the_default_wait_font= fn;
+x_gui_rep::default_font (bool tt, bool mini) {
+  font fn= default_font_sub (tt, mini);
+  if (!tt && !mini) the_default_wait_font= fn;
   return fn;
 }
 
@@ -192,8 +199,8 @@ set_default_font (string name) {
 }
 
 font
-get_default_font (bool tt) {
-  return the_gui->default_font (tt);
+get_default_font (bool tt, bool mini) {
+  return the_gui->default_font (tt, mini);
 }
 
 /******************************************************************************
@@ -235,7 +242,7 @@ x_gui_rep::get_ps_char (Font fn, int c, metric& ex, glyph& gl) {
   for (j=0; j<h; j++)
     for (i=0; i<w; i++) {
       int c = im->f.get_pixel (im, i, j);
-      int on= (c == black? 1: 0);
+      int on= (((color) c) == black? 1: 0);
       gl->set_x (i, j, on);
     }
   gl->lwidth= xcs.width;
@@ -262,15 +269,15 @@ x_gui_rep::load_system_font (string family, int size, int dpi,
 
   if (DEBUG_VERBOSE) cout << "TeXmacs] Loading ps font " << name << "\n";
   char* temp= as_charp (name);
-  Font fn = XLoadFont (dpy, temp);
+  XFontStruct *xfs = XLoadQueryFont (dpy, temp);
   tm_delete_array (temp);
-  if (XQueryFont (dpy, fn) == NULL) {
+  if (xfs == NULL) {
     if (DEBUG_VERBOSE) cout << "TeXmacs] Font " << name << " not found\n";
     if (DEBUG_VERBOSE) cout << "TeXmacs] Using default font instead\n";
-    fn = XLoadFont (dpy, "*");
-    ASSERT (XQueryFont (dpy, fn) != NULL, "could not load default X font");
+    xfs = XLoadQueryFont (dpy, "*");
+    ASSERT (xfs != NULL, "could not load default X font");
   }
-
+  Font fn = xfs->fid;
   int i;
   metric* texs= tm_new_array<metric> (256);
   glyph * gls = tm_new_array<glyph> (256);
@@ -278,6 +285,8 @@ x_gui_rep::load_system_font (string family, int size, int dpi,
     get_ps_char (fn, i, texs[i], gls[i]);
   fnm= std_font_metric (fn_name, texs, 0, 255);
   fng= std_font_glyphs (fn_name, gls , 0, 255);
+
+  XFreeFont(dpy, xfs);
 }
 
 void
