@@ -16,6 +16,7 @@
 #include "convert.hpp"
 #include "merge_sort.hpp"
 #include "drd_std.hpp"
+#include "language.hpp"
 
 tree texmacs_settings = tuple ();
 int  install_status   = 0;
@@ -80,7 +81,11 @@ plugin_list () {
 
 static void
 init_main_paths () {
+#ifdef __MINGW32__
+  if (is_none (get_env_path ("TEXMACS_HOME_PATH", get_env ("APPDATA") * "/TeXmacs"))) {
+#else
   if (is_none (get_env_path ("TEXMACS_HOME_PATH", "~/.TeXmacs"))) {
+#endif
     cerr << "\nTeXmacs]\n";
     cerr << "TeXmacs] Installation problem: please send a bug report.\n";
     cerr << "TeXmacs] 'TEXMACS_HOME_PATH' could not be set to '~/.TeXmacs'.\n";
@@ -146,7 +151,6 @@ init_user_dirs () {
 static void
 init_guile () {
   url guile_path= "$TEXMACS_PATH/progs:$GUILE_LOAD_PATH";
-
   if (!exists (guile_path * "init-texmacs.scm")) {
     cerr << "\nTeXmacs]\n";
     cerr << "TeXmacs] Installation problem: please send a bug report.\n";
@@ -239,16 +243,16 @@ init_env_vars () {
   (void) set_env_path ("TEXMACS_SECURE_PATH",
 		       get_env_path ("TEXMACS_SECURE_PATH") |
 		       "$TEXMACS_PATH:$TEXMACS_HOME_PATH");
-  (void) get_env_path ("TEXMACS_SYNTAX_PATH",
-		       "$TEXMACS_HOME_PATH/langs/mathematical/syntax" |
-		       url ("$TEXMACS_PATH/langs/mathematical/syntax"));
   (void) get_env_path ("TEXMACS_PATTERN_PATH",
 		       "$TEXMACS_HOME_PATH/misc/patterns" |
 		       url ("$TEXMACS_PATH/misc/patterns") |
 		       plugin_path ("misc/patterns"));
   (void) get_env_path ("TEXMACS_PIXMAP_PATH",
 		       "$TEXMACS_HOME_PATH/misc/pixmaps" |
-		       url ("$TEXMACS_PATH/misc/pixmaps") |
+		       url ("$TEXMACS_PATH/misc/pixmaps/modern/24x24/main") |
+		       url ("$TEXMACS_PATH/misc/pixmaps/modern/20x20/mode") |
+		       url ("$TEXMACS_PATH/misc/pixmaps/modern/16x16/focus") |
+		       url ("$TEXMACS_PATH/misc/pixmaps/traditional/--x17") |
 		       plugin_path ("misc/pixmaps"));
   (void) get_env_path ("TEXMACS_DIC_PATH",
 		       "$TEXMACS_HOME_PATH/langs/natural/dic" |
@@ -271,9 +275,10 @@ init_misc () {
 #if defined(__MINGW__) || defined(__MINGW32__) || defined (OS_WIN32)
   use_which = false;
 #else
-  use_which= (var_eval_system ("which texmacs 2> /dev/null") != "");
+  use_which = (var_eval_system ("which texmacs 2> /dev/null") != "");
 #endif
-  use_locate= exists_in_path ("locate");
+  //string loc= var_eval_system ("locate bin/locate 2> /dev/null");
+  //use_locate= (search_forwards ("bin/locate", loc) > 0);
 
   // Set extra environment variables for Cygwin
 #ifdef OS_CYGWIN
@@ -335,12 +340,6 @@ setup_texmacs () {
   url settings_file= "$TEXMACS_HOME_PATH/system/settings.scm";
   cerr << "Welcome to TeXmacs " TEXMACS_VERSION "\n";
   cerr << HRULE;
-  cerr << "Since this seems to be the first time you have run this\n";
-  cerr << "version of TeXmacs, I will first analyze your system\n";
-  cerr << "in order to set up some TeX paths in the correct way.\n";
-  cerr << "This may take some seconds; the result can be found in\n\n";
-  cerr << "\t" << settings_file << "\n\n";
-  cerr << HRULE;
 
   set_setting ("VERSION", TEXMACS_VERSION);
   setup_tex ();
@@ -370,6 +369,7 @@ setup_texmacs () {
 
 void
 init_texmacs () {
+  init_succession_status_table ();
   init_std_drd ();
   init_main_paths ();
   init_user_dirs ();
