@@ -309,14 +309,12 @@ operator * (url u1, url u2) {
   if (u2 == url_parent ()) {
     if (is_root (u1)) return u1;
     if (is_atomic (u1) && (!is_parent (u1))) return url_here ();
-    if (is_semi_root (u1))
-      return u1;
+    if (is_semi_root (u1)) return u1;
   }
   if (is_concat (u2) && (u2[1] == url_parent ())) {
     if (is_root (u1)) return u1 * u2[2];
     if (is_atomic (u1) && (!is_parent (u1))) return u2[2];
-    if (is_semi_root (u1))
-      return u1 * u2[2];
+    if (is_semi_root (u1)) return u1 * u2[2];
   }
   if (is_concat (u1)) return u1[1] * (u1[2] * u2);
   return as_url (tuple ("concat", u1->t, u2->t));
@@ -516,7 +514,7 @@ suffix (url u) {
     if (s[i]=='.') break;
   if ((i>0) && (i<n-1)) {
     string r= s (i+1, n);
-    while ((N(r)>0) && (r[N(r)-1]=='~')) r= r(0, N(r)-1);
+    while ((N(r)>0) && (r[N(r)-1]=='~' || r[N(r)-1]=='#')) r= r(0, N(r)-1);
     return r;
   }
   return "";
@@ -587,6 +585,12 @@ static url
 expand (url u1, url u2) {
   if (is_or (u1)) return expand (u1[1], u2) | expand (u1[2], u2);
   if (is_or (u2)) return expand (u1, u2[1]) | expand (u1, u2[2]);
+  if (is_ancestor (u2)) {
+    if (is_concat (u1)) return u1 | expand (u1[1], u2);
+    return u1 | u2;
+  }
+  if (is_concat (u2) && is_ancestor (u2[1]))
+    return expand (expand (u1, u2[1]), u2[2]);
   return u1 * u2;
 }
 
