@@ -22,6 +22,7 @@
 #include <stddef.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <errno.h>
 #include <dirent.h>
 #ifdef OS_WIN32
 #include <sys/misc.h>
@@ -32,6 +33,10 @@
 #endif
 #include <sys/types.h>
 
+#ifdef MACOSX_EXTENSIONS
+#include "MacOS/mac_images.h"
+#endif
+
 /******************************************************************************
 * New style loading and saving
 ******************************************************************************/
@@ -60,26 +65,33 @@ load_string (url u, string& s, bool fatal) {
     bench_start ("load file");
     char* _name= as_charp (name);
     // cout << "OPEN :" << _name << LF;
-#ifdef OS_WIN32
+#if defined (OS_WIN32)
     FILE* fin= _fopen (_name, "rb");
-#else
-#ifdef __MINGW32__
+#elif defined (__MINGW__) || defined (__MINGW32__)
     FILE* fin= fopen (_name, "rb");
 #else
-	FILE* fin= fopen (_name, "r");
+    FILE* fin= fopen (_name, "r");
 #endif
-#endif
-    if (fin == NULL) err= true;
+    if (fin == NULL) {
+      err= true;
+      if (!occurs ("system", name))
+        cerr << "TeXmacs] warning, load error for " << name << ", "
+             << sys_errlist[errno] << "\n";
+    }
     int size= 0;
     if (!err) {
       if (fseek (fin, 0L, SEEK_END) < 0) err= true;
       else {
-	size = ftell (fin);
+	size= ftell (fin);
 	if (size<0) err= true;
       }
+      if (err) {
+        cerr << "TeXmacs] warning, seek failed for " << as_string (u) << "\n";
+        fclose (fin);
+      }
     }
     if (!err) {
-      rewind(fin);
+      rewind (fin);
       s->resize (size);
       int read= fread (&(s[0]), 1, size, fin);
       if (read < size) s->resize (read);
@@ -119,12 +131,18 @@ save_string (url u, string s, bool fatal) {
   if (!err) {
     string name= concretize (r);
     char* _name= as_charp (name);
-#ifdef OS_WIN32
+#if defined (OS_WIN32)
     FILE* fout= _fopen (_name, "wb");
+#elif defined (__MINGW__) || defined (__MINGW32__)
+    FILE* fout= fopen (_name, "wb");
 #else
     FILE* fout= fopen (_name, "w");
 #endif
-    if (fout == NULL) err= true;
+    if (fout == NULL) {
+      err= true;
+      cerr << "TeXmacs] warning, save error for " << name << ", "
+           << sys_errlist[errno] << "\n";
+    }
     if (!err) {
       int i, n= N(s);
       for (i=0; i<n; i++)
@@ -249,7 +267,7 @@ is_of_type (url name, string filter) {
     return true;
 
   // Normal files
-#ifdef OS_WIN32
+#if defined (OS_WIN32) || defined (__MINGW__) || defined (__MINGW32__)
   if ((filter == "x") && (suffix(name) != "exe") && (suffix(name) != "bat"))
     name = glue (name, ".exe");
 #endif
@@ -289,7 +307,7 @@ is_of_type (url name, string filter) {
 #endif
       break;
     case 'x':
-#ifdef OS_WIN32
+#if defined (OS_WIN32) || defined (__MINGW__) || defined (__MINGW32__)
       if (suffix(name) == "bat") break;
 #endif
 #ifndef __MINGW32__
@@ -329,11 +347,11 @@ is_newer (url which, url than) {
 url
 url_temp (string suffix) {
 #ifdef __MINGW32__
-  int rnd= texmacs_time ();
+  int rnd= raw_time ();
 #else
   static bool initialized= false;
   if (!initialized) {
-    srandom ((int) texmacs_time ());
+    srandom ((int) raw_time ());
     initialized= true;
   }
   int rnd= random ();
@@ -345,8 +363,7 @@ url_temp (string suffix) {
 }
 
 url
-url_scratch (string prefix, string postfix, int i) {
-  url dir ("$TEXMACS_HOME_PATH/texts/scratch");
+url_numbered (url dir, string prefix, string postfix, int i) {
   if (!exists (dir)) mkdir (dir);
   for (; true; i++) {
     url name= dir * (prefix * as_string (i) * postfix);
@@ -355,6 +372,12 @@ url_scratch (string prefix, string postfix, int i) {
   return dir * (prefix * "x" * postfix);
 }
 
+url
+url_scratch (string prefix, string postfix, int i) {
+  url dir ("$TEXMACS_HOME_PATH/texts/scratch");
+  return url_numbered (dir, prefix, postfix, i);
+}
+
 bool
 is_scratch (url u) {
   return head (u) == url ("$TEXMACS_HOME_PATH/texts/scratch");
@@ -575,7 +598,63 @@ ps2pdf (url u1, url u2) {
   XPs2Pdf (_u1, _u2);
   tm_delete_array (_u1);
   tm_delete_array (_u2);
+#else
+#ifdef MACOSX_EXTENSIONS
+  mac_ps_to_pdf (u1, u2);
 #else
   system ("ps2pdf", u1, u2);
 #endif
+#endif
 }
+
+
+/******************************************************************************
+ * Tab-completion for file names
+ ******************************************************************************/
+
+#ifdef OS_WIN32
+#define URL_CONCATER  '\\'
+#else
+#define URL_CONCATER  '/'
+#endif
+
+static void
+file_completions_file (array<string>& a, url search, url u) {
+  if (is_or (u)) {
+    file_completions_file (a, search, u[1]);
+    file_completions_file (a, search, u[2]);
+  }
+  else {
+    url v= delta (search * url ("dummy"), u);
+    if (is_none (v)) return;
+    string s= as_string (v);
+    if (is_directory (u)) s= s * string (URL_CONCATER);
+    a << s;
+  }
+}
+
+static void
+file_completions_dir (array<string>& a, url search, url dir) {
+  if (is_or (search)) {
+    file_completions_dir (a, search[1], dir);
+    file_completions_dir (a, search[2], dir);
+  }
+  else if (is_or (dir)) {
+    file_completions_dir (a, search, dir[1]);
+    file_completions_dir (a, search, dir[2]);
+  }
+  else {
+    url u= search * dir * url_wildcard ("*");
+    u= complete (u, "r");
+    u= expand (u);
+    file_completions_file (a, search, u);
+  }
+}
+
+array<string>
+file_completions (url search, url dir) {
+  array<string> a;
+  file_completions_dir (a, search, dir);
+  return a;
+}
+
