@@ -30,7 +30,8 @@ mathemagix_language_rep::mathemagix_language_rep (string name):
 }
 
 text_property
-mathemagix_language_rep::advance (string s, int& pos) {
+mathemagix_language_rep::advance (tree t, int& pos) {
+  string s= t->label;
   if (pos==N(s)) return &tp_normal_rep;
   char c= s[pos];
   if (c == ' ') {
@@ -72,11 +73,19 @@ mathemagix_color_setup_constants (hashmap<string, string> & t) {
   t ("cpp_libs")= c;
   t ("cpp_preamble")= c;
   t ("cpp_macro")= c;
+  t ("cpp_include")= c;
   t ("true")= c;
   t ("false")= c;
   t ("mmout")= c;
   t ("mmin")= c;
   t ("mmerr")= c;
+  t ("blank")= c;
+  t ("stroke")= c;
+  t ("indent")= c;
+  t ("unindent")= c;
+  t ("lf")= c;
+  t ("hrule")= c;
+  t ("flush_now")= c;
   t ("nil")= c;
 }
 
@@ -99,6 +108,7 @@ mathemagix_color_setup_keywords (hashmap<string, string> & t)  {
   t ("constructor")= c;
   t ("continue")= c;
   t ("convert")= c;
+  t ("debugger")= c;
   t ("destructor")= c;
   t ("direct")= c;
   t ("div")= c;
@@ -131,6 +141,7 @@ mathemagix_color_setup_keywords (hashmap<string, string> & t)  {
   t ("infix")= c;
   t ("inherit")= c;
   t ("inline")= d;
+  t ("inplace")= c;
   t ("interactive")= c;
   t ("intern")= c;
   t ("join")= c;
@@ -152,6 +163,7 @@ mathemagix_color_setup_keywords (hashmap<string, string> & t)  {
   t ("operator")= c;
   t ("or")= c;
   t ("packed")= c;
+  t ("penalty")= c;
   t ("postfix")= "postfix";
   t ("prefix")= c;
   t ("private")= c;
@@ -159,7 +171,9 @@ mathemagix_color_setup_keywords (hashmap<string, string> & t)  {
   t ("public")= c;
   t ("outline")= c;
   t ("quit")= c;
+  t ("quo")= c;
   t ("raise")= c;
+  t ("rem")= c ;
   t ("require")= c;
   t ("return")= c ;
   t ("sequel")= c;
