@@ -4,10 +4,9 @@
 * DESCRIPTION: Loading and saving TeXmacs files
 * COPYRIGHT  : (C) 1999  Joris van der Hoeven
 *******************************************************************************
-* This software falls under the GNU general public license and comes WITHOUT
-* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
-* If you don't have this file, write to the Free Software Foundation, Inc.,
-* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+* This software falls under the GNU general public license version 3 or later.
+* It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
+* in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ******************************************************************************/
 
 #include "tm_data.hpp"
@@ -15,18 +14,28 @@
 #include "file.hpp"
 #include "convert.hpp"
 #include "merge_sort.hpp"
+#include "drd_std.hpp"
 
 /******************************************************************************
 * Loading files
 ******************************************************************************/
 
+static void
+notify_recent_buffer (string name) {
+  if (ends (name, "~") || ends (name, "#")) name= name (0, N(name) - 1);
+  object a= call ("assoc-set!", null_object (), object ("0"), object (name));
+  call ("learn-interactive", object ("recent-buffer"), a);
+}
+
 tree
 tm_data_rep::load_tree (url u, string fm) {
   string s, suf= suffix (u);
   string action= "load " * fm * " file";
   u= resolve (u);
+  set_file_focus (u);
   if (is_none (u) || load_string (u, s, false)) {
-    set_message ("Error: file#" * as_string (u) * "#not found", action);
+    tree vname= verbatim (as_string (u));
+    set_message (concat ("Error: file ", vname, " not found"), action);
     return "error";
   }
   if ((fm == "generic") || (fm == "help")) fm= get_format (s, suf);
@@ -41,6 +50,7 @@ void
 tm_data_rep::load_buffer (url u, string fm, int where, bool autosave_flag) {
   // cout << "Load= " << u << ", " << fm << "\n";
   string name= as_string (tail (u));
+  tree vname= verbatim (name);
   string action= "load " * fm * " file";
   if (fm == "generic")
     action= "load " * suffix_to_format (suffix (u)) * " file";
@@ -57,11 +67,11 @@ tm_data_rep::load_buffer (url u, string fm, int where, bool autosave_flag) {
 	case 0: new_buffer_in_this_window (v, doc); break;
 	case 1: new_buffer_in_new_window (v, doc); break;
 	case 2: new_buffer (v, doc); break;
-	default: fatal_error ("Bad value for 'where'", "load_buffer");
+	default: FAILED ("bad value for 'where'");
 	}
       }
     if (!no_bufs ())
-      set_message ("Error: file#" * name * "#not found", action);
+      set_message (concat ("Error: file ", vname, " not found"), action);
     return;
   }
 
@@ -84,7 +94,7 @@ tm_data_rep::load_buffer (url u, string fm, int where, bool autosave_flag) {
   case 0: new_buffer_in_this_window (v, doc); break;
   case 1: new_buffer_in_new_window (v, doc); break;
   case 2: new_buffer (v, doc); break;
-  default: fatal_error ("Bad value for 'where'", "load_buffer");
+  default: FAILED ("bad value for 'where'");
   }
   nr= find_buffer (v);
   if (nr != -1) {
@@ -95,6 +105,8 @@ tm_data_rep::load_buffer (url u, string fm, int where, bool autosave_flag) {
       buf->read_only= true;
     }
   }
+  if ((fm == "generic") || (fm == "texmacs"))
+    notify_recent_buffer (as_string (u));
 }
 
 tm_buffer
@@ -136,29 +148,134 @@ load_inclusion (url name) {
 }
 
 /******************************************************************************
-* Saving files
+* Get environment and drd of style files
 ******************************************************************************/
 
-struct less_eq_associate {
-  static inline bool leq (tree& a, tree& b) {
-    return as_string(a[0]) <= as_string(b[0]); }
-};
+static hashmap<string,bool> style_busy (false);
+static hashmap<string,tree> style_void (UNINIT);
+static hashmap<tree,hashmap<string,tree> > style_cached (style_void);
+static drd_info drd_void ("void");
+static hashmap<tree,drd_info> drd_cached (drd_void);
 
-template <class T, class U> static
-tree make_collection (hashmap<T,U> h) {
-  tree t(h);
-  array<tree> a=A(h);
-  merge_sort_leq <tree, less_eq_associate> (a);
-  int i, n=N(a);
-  for (i=0; i<n; i++) t[i] = a[i];
-  return t;
+bool
+compute_env_and_drd (tree style) {
+  ASSERT (is_tuple (style), "style tuple expected");
+  bool busy= false;
+  for (int i=0; i<N(style); i++)
+    busy= busy || style_busy->contains (as_string (style[i]));
+  hashmap<string,bool> old_busy= copy (style_busy);
+  for (int i=0; i<N(style); i++)
+    style_busy (as_string (style[i]))= true;
+
+  //cout << "Get environment of " << style << INDENT << LF;
+  hashmap<string,tree> H;
+  drd_info drd ("none", std_drd);
+  url none= url ("$PWD/none");
+  hashmap<string,tree> lref;
+  hashmap<string,tree> gref;
+  hashmap<string,tree> laux;
+  hashmap<string,tree> gaux;
+  edit_env env (drd, none, lref, gref, laux, gaux);
+  if (!busy) {
+    tree t;
+    bool ok;
+    get_server () -> style_get_cache (style, H, t, ok);
+    if (ok) {
+      env->patch_env (H);
+      ok= drd->set_locals (t);
+      drd->set_environment (H);
+    }
+    if (!ok) {
+      env->exec (tree (USE_PACKAGE, A (style)));
+      env->read_env (H);
+      drd->heuristic_init (H);
+    }
+    style_cached (style)= H;
+    drd_cached (style)= drd;
+  }
+  //cout << UNINDENT << "Got environment of " << style << LF;
+
+  style_busy= old_busy;
+  return !busy;
 }
 
+hashmap<string,tree>
+get_style_env (tree style) {
+  if (style_cached->contains (style))
+    return style_cached [style];
+  else if (compute_env_and_drd (style))
+    return style_cached [style];
+  else {
+    //cout << "Busy style: " << style << "\n";
+    return hashmap<string,tree> ();
+  }
+}
+
+drd_info
+get_style_drd (tree style) {
+  init_std_drd ();
+  if (drd_cached->contains (style))
+    return drd_cached [style];
+  else if (compute_env_and_drd (style))
+    return drd_cached [style];
+  else {
+    //cout << "Busy drd: " << style << "\n";
+    return std_drd;
+  }
+}
+
+tree
+get_document_preamble (tree t) {
+  if (is_atomic (t)) return "";
+  else if (is_compound (t, "hide-preamble", 1)) return t[0];
+  else if (is_compound (t, "show-preamble", 1)) return t[0];
+  else {
+    int i, n= N(t);
+    for (i=0; i<n; i++) {
+      tree p= get_document_preamble (t[i]);
+      if (p != "") return p;
+    }
+    return "";
+  }
+}
+
+drd_info
+get_document_drd (tree doc) {
+  tree style= extract (doc, "style");
+  if (extract (doc, "TeXmacs") == "") {
+    if (the_drd->get_syntax (make_tree_label ("theorem")) != tree (UNINIT))
+      return the_drd;
+    style= tree (TUPLE, "generic");
+  }
+  //cout << "style= " << style << "\n";
+  drd_info drd= get_style_drd (style);
+  tree p= get_document_preamble (doc);
+  if (p != "") {
+    drd= drd_info ("preamble", drd);
+    url none= url ("$PWD/none");
+    hashmap<string,tree> lref;
+    hashmap<string,tree> gref;
+    hashmap<string,tree> laux;
+    hashmap<string,tree> gaux;
+    edit_env env (drd, none, lref, gref, laux, gaux);
+    hashmap<string,tree> H;
+    env->exec (tree (USE_PACKAGE, A (style)));
+    env->exec (p);
+    env->read_env (H);
+    drd->heuristic_init (H);
+  }
+  return drd;
+}
+
+/******************************************************************************
+* Saving files
+******************************************************************************/
+
 tree
 tm_data_rep::make_document (tm_view vw, string fm) {
   tree body= subtree (the_et, vw->buf->rp);
   if (fm == "verbatim")
-    body= vw->ed->exec_texmacs (body);
+    body= vw->ed->exec_verbatim (body);
   if (fm == "html")
     body= vw->ed->exec_html (body);
   if (fm == "latex")
@@ -194,6 +311,7 @@ tm_data_rep::make_document (tm_view vw, string fm) {
 void
 tm_data_rep::save_buffer (url u, string fm) {
   string name= as_string (u);
+  tree vname= verbatim (name);
   if (fm == "generic") {
     fm= suffix_to_format (suffix (u));
     if (fm == "generic") fm= "verbatim";
@@ -201,7 +319,7 @@ tm_data_rep::save_buffer (url u, string fm) {
   string action= "save " * fm * " file";
   u= resolve (u, "");
   if (is_none (u)) {
-    set_message ("Error: can't save file#" * name, action);
+    set_message (concat ("Error: can't save file ", vname), action);
     return;
   }
 
@@ -223,15 +341,18 @@ tm_data_rep::save_buffer (url u, string fm) {
   string s= tree_to_generic (doc, fm * "-document");
   if (s != "* error: unknown format *") {
     if (save_string (u, s))
-      set_message ("Error:#" * name * "#did not open", action);
+      set_message (concat ("Error: ", vname, " did not open"), action);
     else {
-      set_message ("saved " * name, action);
+      set_message (concat ("saved ", vname), action);
       if (fm == "texmacs") {
-	if (no_name () && exists (get_name_buffer ()))
-	  remove (get_name_buffer ());
-	set_name_buffer (u);
-	pretend_save_buffer ();
-	if (suffix (u) == "ts") style_clear_cache ();
+        if (no_name () && exists (get_name_buffer ()))
+          remove (get_name_buffer ());
+        set_name_buffer (u);
+        pretend_save_buffer ();
+        if (suffix (u) == "ts") style_clear_cache ();
+        if ((fm == "generic") || (fm == "texmacs"))
+          if (!no_name ())
+            notify_recent_buffer (as_string (u));
       }
     }
   }
@@ -245,21 +366,24 @@ tm_data_rep::auto_save () {
     tm_buffer buf= bufs[i];
     if ((buf->needs_to_be_autosaved () && (!buf->read_only))) {
       url name= buf->name;
+      tree vname= verbatim (as_string (name));
       if (!is_scratch (name))
-	name= glue (buf->name, "~");
+	name= glue (buf->name, rescue_mode? "#": "~");
       if (N(buf->vws)!=0) {
 	tree doc= make_document (buf->vws[0]);
-	if (save_string (name, tree_to_texmacs (doc)))
-	  set_message ("Error: " * as_string (name) * " did not open",
-		       "save TeXmacs file");
-	else {
-	  call ("set-temporary-message",
-		"saved " * as_string (name), "save TeXmacs file", 2500);
-	  buf->mark_undo_block ();
-	  buf->need_autosave= false;
-	  buf->last_autosave= buf->undo_depth- 1;
-	}
+        bool err= save_string (name, tree_to_texmacs (doc));
+        if (!rescue_mode) {
+          if (!err)
+            call ("set-temporary-message",
+                  "saved " * as_string (name), "save TeXmacs file", 2500);
+	  else
+            set_message (concat ("Error: ", vname, " did not open"),
+                         "save TeXmacs file");
+        }
       }
+      if (!rescue_mode)
+        for (int j=0; j<N(buf->vws); j++)
+          buf->vws[j]->ed->notify_save (false);
     }
   }
   call ("delayed-auto-save");
@@ -301,7 +425,6 @@ tm_data_rep::exists_unsaved_buffer () {
 void
 tm_data_rep::pretend_save_buffer () {
   tm_buffer buf= get_buffer ();
-  buf->mark_undo_block ();
-  buf->need_save= buf->need_autosave= false;
-  buf->last_save= buf->last_autosave= buf->undo_depth- 1;
+  for (int i=0; i<N(buf->vws); i++)
+    buf->vws[i]->ed->notify_save ();
 }
