@@ -13,6 +13,8 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <unistd.h>
+#include <locale.h> // for setlocale
+#include <signal.h>
 
 #include "boot.hpp"
 #include "file.hpp"
@@ -25,10 +27,13 @@ void mac_fix_paths ();
 
 #ifdef QTTEXMACS
 #include "Qt/qt_utilities.hpp"
-#include <QCoreApplication>
 #include <QApplication>
 #endif
 
+#ifdef MACOSX_EXTENSIONS
+#include "MacOS/mac_utilities.h"
+#endif
+
 #if defined(X11TEXMACS) && defined(MACOSX_EXTENSIONS)
 #include "MacOS/mac_app.h"
 #endif
@@ -62,6 +67,15 @@ test_routines () {
 }
 #endif
 
+/******************************************************************************
+* Clean exit on segmentation faults
+******************************************************************************/
+
+void 
+clean_exit_on_segfault (int sig_num) {
+  FAILED ("segmentation fault");
+}
+
 /******************************************************************************
 * Texmacs paths
 ******************************************************************************/
@@ -79,8 +93,12 @@ TeXmacs_init_paths (int& argc, char** argv) {
 #endif
 
 #if (defined(QTTEXMACS) && defined(Q_WS_MAC)) 
-  // inibith external plugin loading
-  QCoreApplication::setLibraryPaths(QStringList());
+  // the following line inibith external plugin loading
+  // QCoreApplication::setLibraryPaths(QStringList());
+  // ideally we would like to control the external plugins
+  // and add the most useful (gif, jpeg, svg converters)
+  // to the bundle package. I still do not have a reliable solution
+  // so just allow everything that is reachable.
   {
     // ensure that private versions of the Qt frameworks have priority on
     // other instances.
@@ -104,8 +122,14 @@ TeXmacs_init_paths (int& argc, char** argv) {
     set_env ("TEXMACS_PATH", as_string(exedir * "../Resources/share/TeXmacs"));
   //cout << get_env("PATH") * ":" * as_string(url("$PWD") * argv[0]
   // * "../../Resources/share/TeXmacs/bin") << LF;
-  set_env ("PATH", get_env("PATH") * ":" *
-	           as_string (exedir * "../Resources/share/TeXmacs/bin"));
+  if (exists("/bin/bash")) {
+    string shell_env =  eval_system("PATH='' /bin/bash -l -c 'echo $PATH'");
+    set_env ("PATH", get_env("PATH") * ":" * shell_env * ":" *
+           as_string (exedir * "../Resources/share/TeXmacs/bin"));
+  } else {
+    set_env ("PATH", get_env("PATH") * ":" *
+               as_string (exedir * "../Resources/share/TeXmacs/bin"));
+  }
   // system("set");
 #endif
 
@@ -156,6 +180,10 @@ TeXmacs_main (int argc, char** argv) {
       else if (s == "-debug-bench") debug (DEBUG_FLAG_BENCH, true);
       else if (s == "-debug-history") debug (DEBUG_FLAG_HISTORY, true);
       else if (s == "-debug-qt") debug (DEBUG_FLAG_QT, true);
+      else if (s == "-debug-keyboard") debug (DEBUG_FLAG_KEYBOARD, true);
+      else if (s == "-debug-packrat") debug (DEBUG_FLAG_PACKRAT, true);
+      else if (s == "-debug-flatten") debug (DEBUG_FLAG_FLATTEN, true);
+      else if (s == "-debug-correct") debug (DEBUG_FLAG_CORRECT, true);
       else if (s == "-debug-all") {
 	debug (DEBUG_FLAG_EVENTS, true);
 	debug (DEBUG_FLAG_STD, true);
@@ -164,7 +192,6 @@ TeXmacs_main (int argc, char** argv) {
 	debug (DEBUG_FLAG_BENCH, true);
 	debug (DEBUG_FLAG_QT, true);
       }
-      else if (s == "-enable-fastenv") enable_fastenv= true;
       else if ((s == "-fn") || (s == "-font")) {
 	i++;
 	if (i<argc) the_default_font= argv[i];
@@ -288,6 +315,12 @@ TeXmacs_main (int argc, char** argv) {
   { // opening scope for server sv
   server sv;
 
+  // HACK:
+  // Qt and Guile want to change the locale. 
+  // We need to force it to C to parse correctly the configuration files
+  // (see as_double() in string.cpp)
+  setlocale(LC_NUMERIC, "C");    
+    
   for (i=1; i<argc; i++) {
     if (argv[i] == NULL) break;
     string s= argv[i];
@@ -326,6 +359,7 @@ TeXmacs_main (int argc, char** argv) {
 
   if (DEBUG_STD) cout << "TeXmacs] Starting event loop...\n";
   texmacs_started= true;
+  signal (SIGSEGV, clean_exit_on_segfault);
   gui_start_loop ();
 
   if (DEBUG_STD) cout << "TeXmacs] Stopping server...\n";
@@ -396,6 +430,16 @@ int
 main (int argc, char** argv) {
   immediate_options (argc, argv);
   set_env ("LC_NUMERIC", "POSIX");
+#ifdef MACOSX_EXTENSIONS
+  // Reset TeXmacs if Alt is pressed during startup
+  if (mac_alternate_startup()) {
+    cout << "TeXmacs] Performing setup (Alt on startup)" << LF; 
+    remove (url ("$TEXMACS_HOME_PATH/system/settings.scm"));
+    remove (url ("$TEXMACS_HOME_PATH/system/setup.scm"));
+    remove (url ("$TEXMACS_HOME_PATH/system/cache") * url_wildcard ("*"));
+    remove (url ("$TEXMACS_HOME_PATH/fonts/error") * url_wildcard ("*"));    
+  }
+#endif
 #ifdef QTTEXMACS
   // initialize the Qt application infrastructure
   new QApplication (argc, argv);
