@@ -27,8 +27,6 @@ public:
   string fm;              // buffer format
   url extra;              // for special buffers, like help buffer
   array<tm_view> vws;     // views attached to buffer
-  bool need_save;         // (non textual) modification since last save?
-  bool need_autosave;     // (non textual) modification since last autosave?
   bool read_only;         // buffer is read only?
   bool secure;            // is the buffer secure?
   tm_buffer prj;          // buffer which corresponds to the project
@@ -42,46 +40,18 @@ public:
   hashmap<string,tree> ref;   // all labels with references
   hashmap<string,tree> aux;   // auxiliary output: toc, bib, etc.
 
-  observer undo_obs;      // observer for undoing changes
-  tree undo;              // for undoing changes
-  tree redo;              // for redoing changes
-  tree exdo;              // for undoing redone changes
-  int  undo_depth;        // number of changes
-  int  redo_depth;        // number of undone changes
-  bool undo_flag;         // when undoing some text
-  bool redo_flag;         // when redoing some text
-  int  last_save;         // how many changes at last save
-  int  last_autosave;     // how many changes at last autosave
-
   inline tm_buffer_rep (url name2):
     name (name2), abbr (as_string (tail (name))),
     fm ("texmacs"), extra (url_none ()), vws (0),
-    need_save (false), need_autosave (false),
     read_only (false), secure (is_secure (name2)),
     prj (NULL), in_menu (true),
     rp (new_document ()),
     project (""), style ("style"),
-    init ("?"), fin ("?"), ref ("?"), aux ("?"),
-    undo_obs (undo_observer (this)),
-    undo ("nil"), redo ("nil"), exdo ("nil"),
-    undo_depth (0), redo_depth (0),
-    undo_flag (false), redo_flag (false),
-    last_save (0), last_autosave (0)
-  {
-    attach_observer (subtree (the_et, rp), undo_obs);
-  }
+    init ("?"), fin ("?"), ref ("?"), aux ("?") {}
 
   inline ~tm_buffer_rep () {
-    detach_observer (subtree (the_et, rp), undo_obs);
-    delete_document (rp);
-  }
+    delete_document (rp); }
 
-  void mark_undo_block ();
-  void mark_redo_block ();
-  void unmark_undo_block ();
-  void unmark_redo_block ();
-  void redo_to_undo ();
-  void truncate_undos (int nr);
   bool needs_to_be_saved ();
   bool needs_to_be_autosaved ();
 };
