@@ -21,14 +21,19 @@
 SI italic_correction (box left, box right);
 
 void
-concater_rep::print (int type, box b) {
-  a << line_item (type, b, HYPH_INVALID);
+concater_rep::print (box b) {
+  a << line_item (STD_ITEM, env->mode_op, b, HYPH_INVALID);
+}
+
+void
+concater_rep::print (int type, int op_type, box b) {
+  a << line_item (type, op_type, b, HYPH_INVALID);
 }
 
 void
 concater_rep::control (tree t, path ip) {
   box b= empty_box (ip, 0, 0, 0, env->fn->yx);
-  a << line_item (CONTROL_ITEM, b, HYPH_INVALID, t);
+  a << line_item (CONTROL_ITEM, OP_SKIP, b, HYPH_INVALID, t);
 }
 
 void
@@ -38,7 +43,7 @@ concater_rep::marker (path ip) {
   int sz= script (env->fn_size, env->index_level);
   font gfn (tex_font (fn_name, sz, (int) (env->magn*env->dpi)));
   box b= text_box (ip->next, ip->item, "", gfn, blue);
-  a << line_item (STD_ITEM, b, HYPH_INVALID);
+  a << line_item (MARKER_ITEM, OP_SKIP, b, HYPH_INVALID);
 }
 
 void
@@ -69,37 +74,38 @@ concater_rep::ghost (string s, path ip, color col) {
   box b= text_box (decorate (ip), 0, s, gfn, col);
   array<box> bs (1);
   bs[0]= b;
-  a << line_item (STD_ITEM, composite_box (decorate (ip), bs), HYPH_INVALID);
+  a << line_item (STD_ITEM, OP_SKIP,
+                  composite_box (decorate (ip), bs), HYPH_INVALID);
 }
 
 void
 concater_rep::flag_ok (string s, path ip, color col) {
   path dip = decorate_right (ip);
   SI h= 4*env->fn->wfn/5;
-  int r, g, b;
-  get_rgb_color (col, r, g, b);
+  int r, g, b, a;
+  get_rgb_color (col, r, g, b, a);
   r= 255- (255 - r)/6;
   g= 255- (255 - g)/6;
   b= 255- (255 - b)/6;
-  color light= rgb_color (r, g, b);
-  string info_flag= env->get_string (INFO_FLAG);
-  if (info_flag == "short") {
+  color light= rgb_color (r, g, b, a);
+  int info= env->info_level;
+  if (info == INFO_MINIMAL || info == INFO_SHORT) {
     box infob= info_box (dip, h, env->fn->wline, col, light);
     box specb= specific_box (ip, infob, false, env->fn);
-    print (STD_ITEM, specb);
+    print (specb);
   }
-  if (info_flag == "detailed" || info_flag == "paper") {
+  else if (info == INFO_DETAILED || info == INFO_PAPER) {
     int sz= script (env->fn_size, env->index_level+2);
     font gfn (tex_font ("ecrm", sz, (int) (env->magn*env->dpi)));
     box textb= text_box (decorate (ip), 0, s, gfn, col);
     box flagb= flag_box (dip, textb, h, env->fn->wline, col, light);
-    if (info_flag == "detailed") {
+    if (info == INFO_DETAILED) {
       box specb= specific_box (ip, flagb, false, env->fn);
-      print (STD_ITEM, specb);
+      print (specb);
     }
     else {
       box b= resize_box (ip, flagb, 0, 0, 0, env->fn->yx);
-      print (STD_ITEM, b);
+      print (b);
     }
   }
 }
@@ -159,14 +165,14 @@ concater_rep::typeset (tree t, path ip) {
       ip= ip2;
   }
 
+  if (env->hl_lan != 0)
+    env->lan->highlight (t);
+
   if (is_atomic (t)) {
-    if (env->mode == 1)
-      typeset_text_string (t->label, ip);
-    else if (env->mode == 2)
-      typeset_math_string (t->label, ip);
-    else if (env->mode == 3)
-      typeset_prog_string (t, ip);
-    else typeset_text_string (t->label, ip);
+    if      (env->mode == 1) typeset_text_string (t, ip, 0, N(t->label));
+    else if (env->mode == 2) typeset_math_string (t, ip, 0, N(t->label));
+    else if (env->mode == 3) typeset_prog_string (t, ip, 0, N(t->label));
+    else                     typeset_text_string (t, ip, 0, N(t->label));
     return;
   }
 
@@ -190,19 +196,22 @@ concater_rep::typeset (tree t, path ip) {
   case CONCAT:
     typeset_concat (t, ip);
     break;
-  case GROUP:
-    typeset_group (t, ip);
+  case RIGID:
+    typeset_rigid (t, ip);
     break;
   case HIDDEN:
     //(void) env->exec (t);
-    (void) typeset_as_concat (env, t[0], descend (ip, 0));
+    if (N(t) != 1) typeset_error (t, ip);
+    else (void) typeset_as_concat (env, t[0], descend (ip, 0));
     break;
   case FREEZE:
+    if (N(t) != 1) typeset_error (t, ip);
+    else typeset (attach_middle (t[0], ip));
     //typeset (freeze (t[0]), decorate_middle (ip));
-    typeset (attach_middle (t[0], ip));
     break;
   case UNFREEZE:
-    typeset (t[0], descend (ip, 0));
+    if (N(t) != 1) typeset_error (t, ip);
+    else typeset (t[0], descend (ip, 0));
     break;
   case HSPACE:
     t= env->exec (t);
@@ -223,13 +232,17 @@ concater_rep::typeset (tree t, path ip) {
     typeset_space (attach_here (t, ip));
     break;
   case HTAB:
-    if (N(a)==0) print (STD_ITEM, empty_box (ip, 0, 0, 0, env->fn->yx));
+    if (N(t) != 1 && N(t) != 2) { typeset_error (t, ip); break; }
+    if (N(a)==0) print (empty_box (ip, 0, 0, 0, env->fn->yx));
     print (space (env->as_length (t[0])));
     control (t, ip);
     break;
   case MOVE:
     typeset_move (t, ip);
     break;
+  case SHIFT:
+    typeset_shift (t, ip);
+    break;
   case RESIZE:
     typeset_resize (t, ip);
     break;
@@ -313,18 +326,26 @@ concater_rep::typeset (tree t, path ip) {
       break;
     }
 
+  case AROUND:
+  case VAR_AROUND:
+  case BIG_AROUND:
+    typeset_around (t, ip, env->get_string (MATH_NESTING_MODE) != "off");
+    break;
   case LEFT:
-    typeset_large (t, ip, LEFT_BRACKET_ITEM, "<left-");
+    typeset_large (t, ip, LEFT_BRACKET_ITEM, OP_OPENING_BRACKET, "<left-");
     break;
   case MID:
-    typeset_large (t, ip, MIDDLE_BRACKET_ITEM, "<mid-");
+    typeset_large (t, ip, MIDDLE_BRACKET_ITEM, OP_MIDDLE_BRACKET, "<mid-");
     break;
   case RIGHT:
-    typeset_large (t, ip, RIGHT_BRACKET_ITEM, "<right-");
+    typeset_large (t, ip, RIGHT_BRACKET_ITEM, OP_CLOSING_BRACKET, "<right-");
     break;
   case BIG:
     typeset_bigop (t, ip);
     break;
+  case LONG_ARROW:
+    typeset_long_arrow (t, ip);
+    break;
   case LPRIME:
     typeset_lprime (t, ip);
     break;
@@ -363,6 +384,9 @@ concater_rep::typeset (tree t, path ip) {
   case TREE:
     typeset_tree (t, ip);
     break;
+  case SYNTAX:
+    typeset_syntax (t, ip);
+    break;
 
   case TFORMAT:
     if ((N(t)>0) && is_table (t[N(t)-1])) typeset_table (t, ip);
@@ -481,6 +505,8 @@ concater_rep::typeset (tree t, path ip) {
   case OVER:
   case DIV:
   case MOD:
+  case MINIMUM:
+  case MAXIMUM:
   case MATH_SQRT:
   case EXP:
   case LOG:
@@ -490,7 +516,11 @@ concater_rep::typeset (tree t, path ip) {
   case TAN:
   case MERGE:
   case LENGTH:
+    typeset_executable (t, ip);
+    break;
   case RANGE:
+    typeset_range (t, ip);
+    break;
   case NUMBER:
   case _DATE:
   case TRANSLATE:
@@ -653,6 +683,9 @@ concater_rep::typeset (tree t, path ip) {
   case TEXT_AT:
     typeset_text_at (t, ip);
     break;
+  case MATH_AT:
+    typeset_math_at (t, ip);
+    break;
   case _POINT:
     typeset_point (t, ip);
     break;
@@ -680,8 +713,8 @@ concater_rep::typeset (tree t, path ip) {
   case FILL:
     typeset_fill (t, ip);
     break;
-  case POSTSCRIPT:
-    typeset_postscript (t, ip);
+  case IMAGE:
+    typeset_image (t, ip);
     break;
 
   case CANVAS:
@@ -692,7 +725,7 @@ concater_rep::typeset (tree t, path ip) {
     break;
 
   default:
-    if (L(t) < START_EXTENSIONS) print (STD_ITEM, test_box (ip));
+    if (L(t) < START_EXTENSIONS) print (test_box (ip));
     else typeset_compound (t, ip);
     break;
   }
@@ -702,7 +735,8 @@ concater_rep::typeset (tree t, path ip) {
 * User interface
 ******************************************************************************/
 
-concater_rep::concater_rep (edit_env env2): env (env2) {}
+concater_rep::concater_rep (edit_env env2, bool rigid2):
+  env (env2), rigid (rigid2) {}
 
 array<line_item>
 typeset_concat (edit_env env, tree t, path ip) {
@@ -725,7 +759,7 @@ typeset_marker (edit_env env, path ip) {
 
 box
 typeset_as_concat (edit_env env, tree t, path ip) {
-  concater ccc= tm_new<concater_rep> (env);
+  concater ccc= tm_new<concater_rep> (env, true);
   ccc->typeset (t, ip);
   ccc->finish ();
   array<line_item> a= ccc->a;
